@@ -76,7 +76,6 @@ import org.apache.calcite.tools.RelBuilderFactory;
 import org.apache.calcite.util.ImmutableBitSet;
 import org.apache.calcite.util.ImmutableIntList;
 
-import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableSet;
 
@@ -85,10 +84,13 @@ import org.immutables.value.Value;
 
 import java.util.List;
 import java.util.Map;
-import java.util.Objects;
 import java.util.Set;
 import java.util.SortedSet;
 
+import static com.google.common.base.Preconditions.checkArgument;
+
+import static java.util.Objects.requireNonNull;
+
 /**
  * Utilities pertaining to {@link BindableRel} and {@link BindableConvention}.
  */
@@ -217,9 +219,9 @@ public class Bindables {
         RelOptTable table, ImmutableList<RexNode> filters,
         ImmutableIntList projects) {
       super(cluster, traitSet, ImmutableList.of(), table);
-      this.filters = Objects.requireNonNull(filters, "filters");
-      this.projects = Objects.requireNonNull(projects, "projects");
-      Preconditions.checkArgument(canHandle(table));
+      this.filters = requireNonNull(filters, "filters");
+      this.projects = requireNonNull(projects, "projects");
+      checkArgument(canHandle(table));
     }
 
     /** Creates a BindableTableScan. */
@@ -835,6 +837,11 @@ public class Bindables {
           constants, getRowType(), groups);
     }
 
+    @Override public Window copy(List<RexLiteral> constants) {
+      return new BindableWindow(getCluster(), traitSet, getInput(),
+          constants, getRowType(), groups);
+    }
+
     @Override public @Nullable RelOptCost computeSelfCost(RelOptPlanner planner,
         RelMetadataQuery mq) {
       RelOptCost cost = super.computeSelfCost(planner, mq);
