@@ -57,7 +57,6 @@ import org.apache.calcite.util.graph.DirectedGraph;
 import org.apache.calcite.util.graph.TopologicalOrderIterator;
 import org.apache.calcite.util.mapping.IntPair;
 
-import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableListMultimap;
 import com.google.common.collect.ImmutableSortedSet;
@@ -86,6 +85,8 @@ import java.util.function.Function;
 import java.util.function.IntFunction;
 import java.util.stream.Collectors;
 
+import static com.google.common.base.Preconditions.checkArgument;
+
 import static org.apache.calcite.linq4j.Nullness.castNonNull;
 
 import static java.util.Objects.requireNonNull;
@@ -130,7 +131,7 @@ public class Lattice {
       // [CALCITE-429] Add statistics SPI for lattice optimization algorithm
       rowCountEstimate = 1000d;
     }
-    Preconditions.checkArgument(rowCountEstimate > 0d);
+    checkArgument(rowCountEstimate > 0d);
     this.rowCountEstimate = rowCountEstimate;
     @SuppressWarnings("argument.type.incompatible")
     LatticeStatisticProvider statisticProvider =
@@ -250,6 +251,14 @@ public class Lattice {
    * given set of columns and measures, optionally grouping. */
   public String sql(ImmutableBitSet groupSet, boolean group,
       List<Measure> aggCallList) {
+    final SqlDialect dialect = SqlDialect.DatabaseProduct.CALCITE.getDialect();
+    return sql(groupSet, group, aggCallList, dialect);
+  }
+
+  /** Generates a SQL query to populate a tile of the lattice specified by a
+   * given set of columns and measures, optionally grouping and dialect. */
+  public String sql(ImmutableBitSet groupSet, boolean group,
+      List<Measure> aggCallList, SqlDialect dialect) {
     final List<LatticeNode> usedNodes = new ArrayList<>();
     if (group) {
       final ImmutableBitSet.Builder columnSetBuilder = groupSet.rebuild();
@@ -276,7 +285,6 @@ public class Lattice {
       usedNodes.addAll(rootNode.descendants);
     }
 
-    final SqlDialect dialect = SqlDialect.DatabaseProduct.CALCITE.getDialect();
     final StringBuilder buf = new StringBuilder("SELECT ");
     final StringBuilder groupBuf = new StringBuilder("\nGROUP BY ");
     int k = 0;
@@ -798,7 +806,7 @@ public class Lattice {
 
     public Builder(LatticeSpace space, CalciteSchema schema, String sql) {
       this.rootSchema = requireNonNull(schema.root());
-      Preconditions.checkArgument(rootSchema.isRoot(), "must be root schema");
+      checkArgument(rootSchema.isRoot(), "must be root schema");
       CalcitePrepare.ConvertResult parsed =
           Schemas.convert(MaterializedViewTable.MATERIALIZATION_CONNECTION,
               schema, schema.path(null), sql);
@@ -810,8 +818,7 @@ public class Lattice {
 
       // Get aliases.
       List<@Nullable String> aliases = new ArrayList<>();
-      SqlNode from = ((SqlSelect) parsed.sqlNode).getFrom();
-      assert from != null : "from must not be null";
+      SqlNode from = requireNonNull(((SqlSelect) parsed.sqlNode).getFrom());
       populateAliases(from, aliases, null);
 
       // Build a graph.
@@ -863,7 +870,7 @@ public class Lattice {
         }
         map.put(vertex.table, node);
       }
-      assert root != null;
+      requireNonNull(root, "root");
       final Fixer fixer = new Fixer();
       fixer.fixUp(root);
       baseColumns = fixer.columnList.build();
@@ -934,7 +941,7 @@ public class Lattice {
                   LatticeStatisticProvider.Factory.class,
                   this.statisticProvider)
               : Lattices.CACHED_SQL;
-      Preconditions.checkArgument(rootSchema.isRoot(), "must be root schema");
+      checkArgument(rootSchema.isRoot(), "must be root schema");
       final ImmutableList.Builder<Column> columnBuilder =
           ImmutableList.<Column>builder()
           .addAll(baseColumns)
@@ -977,7 +984,7 @@ public class Lattice {
      */
     private Column resolveColumnByAlias(String name) {
       final ImmutableList<Column> list = columnsByAlias.get(name);
-      if (list == null || list.size() == 0) {
+      if (list.isEmpty()) {
         throw new RuntimeException("Unknown lattice column '" + name + "'");
       } else if (list.size() == 1) {
         return list.get(0);
