@@ -40,6 +40,7 @@ import org.apache.calcite.rel.type.RelDataTypeField;
 import org.apache.calcite.rex.RexBuilder;
 import org.apache.calcite.rex.RexCall;
 import org.apache.calcite.rex.RexCorrelVariable;
+import org.apache.calcite.rex.RexDynamicParam;
 import org.apache.calcite.rex.RexFieldAccess;
 import org.apache.calcite.rex.RexFieldCollation;
 import org.apache.calcite.rex.RexLiteral;
@@ -49,6 +50,7 @@ import org.apache.calcite.rex.RexSlot;
 import org.apache.calcite.rex.RexWindow;
 import org.apache.calcite.rex.RexWindowBound;
 import org.apache.calcite.rex.RexWindowBounds;
+import org.apache.calcite.rex.RexWindowExclusion;
 import org.apache.calcite.sql.SqlAggFunction;
 import org.apache.calcite.sql.SqlFunction;
 import org.apache.calcite.sql.SqlIdentifier;
@@ -444,7 +446,6 @@ public class RelJson {
     return map;
   }
 
-  @SuppressWarnings({"BetaApi", "UnstableApiUsage"}) // RangeSet GA in Guava 32
   public @Nullable Object toJson(@Nullable Object value) {
     if (value == null
         || value instanceof Number
@@ -505,7 +506,6 @@ public class RelJson {
     return map;
   }
 
-  @SuppressWarnings({"BetaApi", "UnstableApiUsage"}) // RangeSet GA in Guava 32
   public <C extends Comparable<C>> List<List<String>> toJson(
       RangeSet<C> rangeSet) {
     final List<List<String>> list = new ArrayList<>();
@@ -578,6 +578,13 @@ public class RelJson {
   public Object toJson(RexNode node) {
     final Map<String, @Nullable Object> map;
     switch (node.getKind()) {
+    case DYNAMIC_PARAM:
+      map = jsonBuilder().map();
+      final RexDynamicParam rexDynamicParam = (RexDynamicParam) node;
+      final RelDataType rdpType = rexDynamicParam.getType();
+      map.put("dynamicParam", rexDynamicParam.getIndex());
+      map.put("type", toJson(rdpType));
+      return map;
     case FIELD_ACCESS:
       map = jsonBuilder().map();
       final RexFieldAccess fieldAccess = (RexFieldAccess) node;
@@ -622,7 +629,9 @@ public class RelJson {
         }
         map.put("operands", list);
         switch (node.getKind()) {
+        case MINUS:
         case CAST:
+        case SAFE_CAST:
           map.put("type", toJson(node.getType()));
           break;
         default:
@@ -651,10 +660,10 @@ public class RelJson {
 
   private Object toJson(RexWindow window) {
     final Map<String, @Nullable Object> map = jsonBuilder().map();
-    if (window.partitionKeys.size() > 0) {
+    if (!window.partitionKeys.isEmpty()) {
       map.put("partition", toJson(window.partitionKeys));
     }
-    if (window.orderKeys.size() > 0) {
+    if (!window.orderKeys.isEmpty()) {
       map.put("order", toJson(window.orderKeys));
     }
     if (window.getLowerBound() == null) {
@@ -762,11 +771,18 @@ public class RelJson {
             upperBound = null;
             physical = false;
           }
+          final RexWindowExclusion exclude;
+          if (window.get("exclude") != null) {
+            exclude = toRexWindowExclusion((Map) window.get("exclude"));
+          } else {
+            exclude = RexWindowExclusion.EXCLUDE_NO_OTHER;
+          }
           final boolean distinct = get((Map<String, Object>) map, "distinct");
           return rexBuilder.makeOver(type, operator, rexOperands, partitionKeys,
               ImmutableList.copyOf(orderKeys),
               requireNonNull(lowerBound, "lowerBound"),
               requireNonNull(upperBound, "upperBound"),
+              requireNonNull(exclude, "exclude"),
               physical,
               true, false, distinct, false);
         } else {
@@ -829,6 +845,12 @@ public class RelJson {
         Sarg sarg = sargFromJson((Map) sargObject);
         return rexBuilder.makeSearchArgumentLiteral(sarg, type);
       }
+      if (map.containsKey("dynamicParam")) {
+        final Object dynamicParamObject = requireNonNull(map.get("dynamicParam"));
+        final Integer index = (Integer) dynamicParamObject;
+        final RelDataType type = toType(typeFactory, get(map, "type"));
+        return rexBuilder.makeDynamicParam(type, index);
+      }
       throw new UnsupportedOperationException("cannot convert to rex " + o);
     } else if (o instanceof Boolean) {
       return rexBuilder.makeLiteral((Boolean) o);
@@ -855,8 +877,6 @@ public class RelJson {
    * nullAs: "UNKNOWN"}} represents the range x &ge; 0 and x &le; 5 or
    * x &gt; 10.
    */
-  // BetaApi is no longer a concern; the Beta tag was removed in Guava 32.0
-  @SuppressWarnings({"BetaApi", "unchecked"})
   public static <C extends Comparable<C>> Sarg<C> sargFromJson(
       Map<String, Object> map) {
     final String nullAs = requireNonNull((String) map.get("nullAs"), "nullAs");
@@ -867,7 +887,6 @@ public class RelJson {
   }
 
   /** Converts a JSON list to a {@link RangeSet}. */
-  @SuppressWarnings({"BetaApi", "UnstableApiUsage"}) // RangeSet GA in Guava 32
   public static <C extends Comparable<C>> RangeSet<C> rangeSetFromJson(
       List<List<String>> rangeSetsJson) {
     final ImmutableRangeSet.Builder<C> builder = ImmutableRangeSet.builder();
@@ -953,6 +972,26 @@ public class RelJson {
     }
   }
 
+  private static @Nullable RexWindowExclusion toRexWindowExclusion(
+      @Nullable Map<String, Object> map) {
+    if (map == null) {
+      return null;
+    }
+    final String type = get(map, "type");
+    switch (type) {
+    case "CURRENT_ROW":
+      return RexWindowExclusion.EXCLUDE_CURRENT_ROW;
+    case "GROUP":
+      return RexWindowExclusion.EXCLUDE_GROUP;
+    case "TIES":
+      return RexWindowExclusion.EXCLUDE_TIES;
+    case "NO OTHERS":
+      return RexWindowExclusion.EXCLUDE_NO_OTHER;
+    default:
+      throw new UnsupportedOperationException(
+          "cannot convert " + type + " to rex window exclusion");
+    }
+  }
   private @Nullable RexWindowBound toRexWindowBound(RelInput input,
       @Nullable Map<String, Object> map) {
     if (map == null) {
@@ -972,7 +1011,7 @@ public class RelJson {
     case "FOLLOWING":
       return RexWindowBounds.following(toRex(input, get(map, "offset")));
     default:
-      throw new UnsupportedOperationException("cannot convert type to rex window bound " + type);
+      throw new UnsupportedOperationException("cannot convert " + type + " to rex window bound");
     }
   }
 
