@@ -39,6 +39,7 @@ import org.apache.calcite.rel.core.Sort;
 import org.apache.calcite.rel.core.TableModify;
 import org.apache.calcite.rel.core.TableScan;
 import org.apache.calcite.rel.core.Union;
+import org.apache.calcite.rel.core.Values;
 import org.apache.calcite.rex.RexBuilder;
 import org.apache.calcite.rex.RexCall;
 import org.apache.calcite.rex.RexExecutor;
@@ -56,6 +57,7 @@ import org.apache.calcite.sql.fun.SqlStdOperatorTable;
 import org.apache.calcite.util.BitSets;
 import org.apache.calcite.util.Bug;
 import org.apache.calcite.util.ImmutableBitSet;
+import org.apache.calcite.util.Sarg;
 import org.apache.calcite.util.Util;
 import org.apache.calcite.util.mapping.Mapping;
 import org.apache.calcite.util.mapping.MappingType;
@@ -63,6 +65,9 @@ import org.apache.calcite.util.mapping.Mappings;
 
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Iterables;
+import com.google.common.collect.Range;
+import com.google.common.collect.RangeSet;
+import com.google.common.collect.TreeRangeSet;
 
 import org.checkerframework.checker.nullness.qual.Nullable;
 
@@ -341,6 +346,28 @@ public class RelMdPredicates
     return joinInference.inferPredicates(false);
   }
 
+  /** Check whether the fields specified by the predicateColumns appear in all
+   * the groupSets of the aggregate.
+   *
+   * @param predicateColumns  A list of columns used in a pulled predicate.
+   * @param aggregate         An aggregation operation.
+   * @return                  Whether all columns appear in all groupsets.
+   */
+  boolean allGroupSetsOverlap(ImmutableBitSet predicateColumns, Aggregate aggregate) {
+    // Consider this example:
+    // select deptno, sal, count(*)
+    // from emp where deptno = 10
+    // group by rollup(sal, deptno)
+    // Because of the ROLLUP, we cannot assume
+    // that deptno = 10 in the result: deptno may be NULL as well.
+    for (ImmutableBitSet groupSet : aggregate.groupSets) {
+      if (!groupSet.contains(predicateColumns)) {
+        return false;
+      }
+    }
+    return true;
+  }
+
   /**
    * Infers predicates for an Aggregate.
    *
@@ -365,7 +392,7 @@ public class RelMdPredicates
       // it is not valid to pull up predicates. In particular, consider the
       // predicate "false": it is valid on all input rows (trivially - there are
       // no rows!) but not on the output (there is one row).
-      return RelOptPredicateList.EMPTY;
+      return RelOptPredicateList.of(rexBuilder, aggPullUpPredicates);
     }
     Mapping m =
         Mappings.create(MappingType.PARTIAL_FUNCTION,
@@ -379,7 +406,8 @@ public class RelMdPredicates
 
     for (RexNode r : inputInfo.pulledUpPredicates) {
       ImmutableBitSet rCols = RelOptUtil.InputFinder.bits(r);
-      if (groupKeys.contains(rCols)) {
+
+      if (groupKeys.contains(rCols) && this.allGroupSetsOverlap(rCols, agg)) {
         r = r.accept(new RexPermuteInputsShuttle(m, input));
         aggPullUpPredicates.add(r);
       }
@@ -533,6 +561,63 @@ public class RelMdPredicates
     return mq.getPulledUpPredicates(input);
   }
 
+  /**
+   * Infers predicates for a Values.
+   *
+   * <p>The predicates on {@code T (w, x, y, z)} with rows
+   * {@code (1, 2, 3, null), (1, 2, null, null), (5, 2, 3, null)} are
+   * {@code 'SEARCH($0, Sarg[1, 5])'},
+   * {@code '=($1, 2)'},
+   * {@code 'SEARCH($2, Sarg[3; NULL AS TRUE])'} and
+   * {@code '[IS NULL($3)'}.
+   */
+  public RelOptPredicateList getPredicates(Values values, RelMetadataQuery mq) {
+    ImmutableList<ImmutableList<RexLiteral>> tuples = values.tuples;
+    if (!tuples.isEmpty()) {
+      List<RexLiteral> firstTuple = tuples.get(0);
+      List<HashSet<RexLiteral>> valueList = new ArrayList<>();
+      for (int i = 0; i < firstTuple.size(); i++) {
+        valueList.add(i, new HashSet<>());
+      }
+      for (int i = 0; i < tuples.size(); i++) {
+        List<RexLiteral> tuple = tuples.get(i);
+        for (int j = 0; j < tuple.size(); j++) {
+          RexLiteral rexLiteral = tuple.get(j);
+          valueList.get(j).add(rexLiteral);
+        }
+      }
+      RexBuilder rexBuilder = values.getCluster().getRexBuilder();
+      List<RexNode> predicates = new ArrayList<>();
+      for (int i = 0; i < valueList.size(); i++) {
+        HashSet<RexLiteral> rexLiteralSet = valueList.get(i);
+        if (rexLiteralSet.size() == 1) {
+          for (RexLiteral rexLiteral : rexLiteralSet) {
+            predicates.add(i,
+                eqConstant(values, rexBuilder, i, rexLiteral));
+          }
+        } else {
+          RexUnknownAs rexUnknownAs = RexUnknownAs.UNKNOWN;
+          RangeSet<Comparable> rangeSet = TreeRangeSet.create();
+          for (RexLiteral rexLiteral : rexLiteralSet) {
+            if (RexUtil.isNull(rexLiteral)) {
+              rexUnknownAs = RexUnknownAs.TRUE;
+              continue;
+            }
+            rangeSet.add(Range.singleton(requireNonNull(rexLiteral.getValueAs(Comparable.class))));
+          }
+          final Sarg sarg = Sarg.of(rexUnknownAs, rangeSet);
+          predicates.add(
+              i, rexBuilder.makeCall(SqlStdOperatorTable.SEARCH,
+                  rexBuilder.makeInputRef(values, i),
+                  rexBuilder.makeSearchArgumentLiteral(sarg,
+                      values.getRowType().getFieldList().get(i).getType())));
+        }
+      }
+      return RelOptPredicateList.of(rexBuilder, predicates);
+    }
+    return RelOptPredicateList.EMPTY;
+  }
+
   // CHECKSTYLE: IGNORE 1
   /**
    * Returns the
@@ -689,6 +774,8 @@ public class RelMdPredicates
       case INNER:
       case LEFT:
       case ANTI:
+      case ASOF:
+      case LEFT_ASOF:
         infer(leftChildPredicates, allExprs, inferredPredicates,
             includeEqualityInference,
             joinType == JoinRelType.LEFT ? rightFieldsBitSet
@@ -749,7 +836,7 @@ public class RelMdPredicates
                     RelOptUtil.conjunctions(joinRel.getCondition())),
                 inferredPredicates);
         return RelOptPredicateList.of(rexBuilder, pulledUpPredicates,
-          leftInferredPredicates, rightInferredPredicates);
+            leftInferredPredicates, rightInferredPredicates);
       case LEFT:
       case ANTI:
         return RelOptPredicateList.of(rexBuilder,
@@ -760,7 +847,7 @@ public class RelMdPredicates
             RelOptUtil.conjunctions(rightChildPredicates),
             inferredPredicates, EMPTY_LIST);
       default:
-        assert inferredPredicates.size() == 0;
+        assert inferredPredicates.isEmpty();
         return RelOptPredicateList.EMPTY;
       }
     }
@@ -785,7 +872,7 @@ public class RelMdPredicates
           RexNode tr =
               r.accept(
                   new RexPermuteInputsShuttle(m, joinRel.getInput(0),
-                  joinRel.getInput(1)));
+                      joinRel.getInput(1)));
           // Filter predicates can be already simplified, so we should work with
           // simplified RexNode versions as well. It also allows prevent of having
           // some duplicates in in result pulledUpPredicates
