@@ -47,7 +47,6 @@ import org.apache.calcite.util.ImmutableBitSet;
 import org.apache.calcite.util.NumberUtil;
 import org.apache.calcite.util.Util;
 
-import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
 
 import org.checkerframework.checker.nullness.qual.Nullable;
@@ -59,8 +58,12 @@ import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
 
+import static com.google.common.base.Preconditions.checkArgument;
+
 import static org.apache.calcite.util.NumberUtil.multiply;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * RelMdUtil provides utility methods used by the metadata provider methods.
  */
@@ -107,9 +110,7 @@ public class RelMdUtil {
     RexCall call = (RexCall) artificialSelectivityFuncNode;
     assert call.getOperator() == ARTIFICIAL_SELECTIVITY_FUNC;
     RexNode operand = call.getOperands().get(0);
-    @SuppressWarnings("unboxing.of.nullable")
-    double doubleValue = ((RexLiteral) operand).getValueAs(Double.class);
-    return doubleValue;
+    return RexLiteral.numberValue(operand).doubleValue();
   }
 
   /**
@@ -830,6 +831,10 @@ public class RelMdUtil {
     }
     double innerRowCount = left * right * selectivity;
     switch (join.getJoinType()) {
+    case ASOF:
+      return left * selectivity;
+    case LEFT_ASOF:
+      return left;
     case INNER:
       return innerRowCount;
     case LEFT:
@@ -883,8 +888,8 @@ public class RelMdUtil {
    */
   public static double linear(int x, int minX, int maxX, double minY, double
       maxY) {
-    Preconditions.checkArgument(minX < maxX);
-    Preconditions.checkArgument(minY < maxY);
+    checkArgument(minX < maxX);
+    checkArgument(minY < maxY);
     if (x < minX) {
       return minY;
     }
@@ -900,12 +905,12 @@ public class RelMdUtil {
    * cardinality of its result. */
   private static class CardOfProjExpr extends RexVisitorImpl<@Nullable Double> {
     private final RelMetadataQuery mq;
-    private Project rel;
+    private final Project rel;
 
     CardOfProjExpr(RelMetadataQuery mq, Project rel) {
       super(true);
-      this.mq = mq;
-      this.rel = rel;
+      this.mq = requireNonNull(mq, "mq");
+      this.rel = requireNonNull(rel, "rel");
     }
 
     @Override public @Nullable Double visitInputRef(RexInputRef var) {
