@@ -272,7 +272,7 @@ public class CoreRules {
    * {@link org.apache.calcite.rel.rules.FilterProjectTransposeRule}.
    *
    * <p>It does not allow a Filter to be pushed past the Project if
-   * {@link RexUtil#containsCorrelation there is a correlation condition})
+   * {@link RexUtil#containsCorrelation there is a correlation condition}
    * anywhere in the Filter, since in some cases it can prevent a
    * {@link Correlate} from being de-correlated.
    */
@@ -431,6 +431,10 @@ public class CoreRules {
   public static final SubQueryRemoveRule JOIN_SUB_QUERY_TO_CORRELATE =
       SubQueryRemoveRule.Config.JOIN.toRule();
 
+  /** Rule that converts SUM to SUM0 in OVER clauses in a project list. */
+  public static final ProjectOverSumToSum0Rule PROJECT_OVER_SUM_TO_SUM0_RULE =
+      ProjectOverSumToSum0Rule.Config.DEFAULT.toRule();
+
   /** Rule that transforms a {@link Project}
    * into a mixture of {@code LogicalProject}
    * and {@link LogicalWindow}. */
@@ -771,6 +775,11 @@ public class CoreRules {
   public static final UnionToDistinctRule UNION_TO_DISTINCT =
       UnionToDistinctRule.Config.DEFAULT.toRule();
 
+  /** Rule that translates a {@link Union} to {@link Values}
+   * if it's all input are {@link Values}. */
+  public static final UnionToValuesRule UNION_TO_VALUES =
+      UnionToValuesRule.Config.DEFAULT.toRule();
+
   /** Rule that rewrite {@link Sample} which is bernoulli to the {@link Filter}. */
   public static final SampleToFilterRule SAMPLE_TO_FILTER =
       SampleToFilterRule.Config.DEFAULT.toRule();
