@@ -335,19 +335,15 @@ public class LoptOptimizeJoinRule
       for (RexNode filter : multiJoin.getJoinFilters()) {
         ImmutableBitSet joinFactors =
             multiJoin.getFactorsRefByJoinFilter(filter);
-        if ((joinFactors.cardinality() == 2)
+        if (joinFactors.cardinality() == 2
             && joinFactors.get(factor1)
             && joinFactors.get(factor2)) {
           selfJoinFilters.add(filter);
         }
       }
-      if ((selfJoinFilters.size() > 0)
-          && isSelfJoinFilterUnique(
-            mq,
-            multiJoin,
-            factor1,
-            factor2,
-            selfJoinFilters)) {
+      if (!selfJoinFilters.isEmpty()
+          && isSelfJoinFilterUnique(mq, multiJoin, factor1, factor2,
+              selfJoinFilters)) {
         multiJoin.addRemovableSelfJoinPair(factor1, factor2);
       }
     }
@@ -754,7 +750,7 @@ public class LoptOptimizeJoinRule
       prevFactor = nextFactor;
     }
 
-    assert filtersToAdd.size() == 0;
+    assert filtersToAdd.isEmpty();
     return joinTree;
   }
 
