@@ -51,12 +51,13 @@ import org.immutables.value.Value;
 
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Objects;
 import java.util.Set;
 import java.util.stream.Collectors;
 
 import static org.apache.calcite.util.Util.last;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * Transform that converts IN, EXISTS and scalar sub-queries into joins.
  *
@@ -79,7 +80,7 @@ public class SubQueryRemoveRule
   /** Creates a SubQueryRemoveRule. */
   protected SubQueryRemoveRule(Config config) {
     super(config);
-    Objects.requireNonNull(config.matchHandler());
+    requireNonNull(config.matchHandler());
   }
 
   @Override public void onMatch(RelOptRuleCall call) {
@@ -830,8 +831,7 @@ public class SubQueryRemoveRule
     final Project project = call.rel(0);
     final RelBuilder builder = call.builder();
     final RexSubQuery e =
-        RexUtil.SubQueryFinder.find(project.getProjects());
-    assert e != null;
+        requireNonNull(RexUtil.SubQueryFinder.find(project.getProjects()));
     final RelOptUtil.Logic logic =
         LogicVisitor.find(RelOptUtil.Logic.TRUE_FALSE_UNKNOWN,
             project.getProjects(), e);
@@ -887,8 +887,7 @@ public class SubQueryRemoveRule
     final Join join = call.rel(0);
     final RelBuilder builder = call.builder();
     final RexSubQuery e =
-        RexUtil.SubQueryFinder.find(join.getCondition());
-    assert e != null;
+        requireNonNull(RexUtil.SubQueryFinder.find(join.getCondition()));
     final RelOptUtil.Logic logic =
         LogicVisitor.find(RelOptUtil.Logic.TRUE,
             ImmutableList.of(join.getCondition()), e);
