@@ -31,7 +31,8 @@ import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
-import java.util.Objects;
+
+import static java.util.Objects.requireNonNull;
 
 /**
  * RelDataTypeFactory is a factory for datatype descriptors. It defines methods
@@ -134,6 +135,17 @@ public interface RelDataTypeFactory {
       RelDataType keyType,
       RelDataType valueType);
 
+  /**
+   * Creates a function type.
+   *
+   * @param parameterType type of parameters
+   * @param returnType type of lambda expression return type
+   * @return function type descriptor
+   */
+  RelDataType createFunctionSqlType(
+      RelDataType parameterType,
+      RelDataType returnType);
+
   /**
    * Creates a measure type.
    *
@@ -457,7 +469,7 @@ public interface RelDataTypeFactory {
      * Creates a Builder with the given type factory.
      */
     public Builder(RelDataTypeFactory typeFactory) {
-      this.typeFactory = Objects.requireNonNull(typeFactory, "typeFactory");
+      this.typeFactory = requireNonNull(typeFactory, "typeFactory");
     }
 
     /**
