@@ -39,10 +39,11 @@ import org.apache.calcite.util.Litmus;
 import org.apache.calcite.util.NlsString;
 import org.apache.calcite.util.Sarg;
 import org.apache.calcite.util.TimeString;
+import org.apache.calcite.util.TimeWithTimeZoneString;
 import org.apache.calcite.util.TimestampString;
+import org.apache.calcite.util.TimestampWithTimeZoneString;
 import org.apache.calcite.util.Util;
 
-import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
 
 import org.checkerframework.checker.initialization.qual.UnknownInitialization;
@@ -54,6 +55,7 @@ import org.locationtech.jts.geom.Geometry;
 
 import java.io.PrintWriter;
 import java.math.BigDecimal;
+import java.math.MathContext;
 import java.nio.ByteBuffer;
 import java.nio.charset.Charset;
 import java.text.SimpleDateFormat;
@@ -64,6 +66,8 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.TimeZone;
 
+import static com.google.common.base.Preconditions.checkArgument;
+
 import static org.apache.calcite.linq4j.Nullness.castNonNull;
 import static org.apache.calcite.rel.type.RelDataTypeImpl.NON_NULLABLE_SUFFIX;
 
@@ -109,9 +113,11 @@ import static java.util.Objects.requireNonNull;
  * <td>{@link BigDecimal}</td>
  * </tr>
  * <tr>
- * <td>{@link SqlTypeName#DOUBLE}</td>
+ * <td>{@link SqlTypeName#DOUBLE},
+ *     {@link SqlTypeName#REAL},
+ *     {@link SqlTypeName#FLOAT}</td>
  * <td>Approximate number, for example <code>6.023E-23</code>.</td>
- * <td>{@link BigDecimal}</td>
+ * <td>{@link Double}.</td>
  * </tr>
  * <tr>
  * <td>{@link SqlTypeName#DATE}</td>
@@ -190,7 +196,7 @@ public class RexLiteral extends RexNode {
   /**
    * The value of this literal. Must be consistent with its type, as per
    * {@link #valueMatchesType}. For example, you can't store an
-   * {@link Integer} value here just because you feel like it -- all numbers are
+   * {@link Integer} value here just because you feel like it -- all exact numbers are
    * represented by a {@link BigDecimal}. But since this field is private, it
    * doesn't really matter how the values are stored.
    */
@@ -201,12 +207,9 @@ public class RexLiteral extends RexNode {
    */
   private final RelDataType type;
 
-  // TODO jvs 26-May-2006:  Use SqlTypeFamily instead; it exists
-  // for exactly this purpose (to avoid the confusion which results
-  // from overloading SqlTypeName).
   /**
    * An indication of the broad type of this literal -- even if its type isn't
-   * a SQL type. Sometimes this will be different than the SQL type; for
+   * a SQL type. Sometimes this will be different from the SQL type; for
    * example, all exact numbers, including integers have typeName
    * {@link SqlTypeName#DECIMAL}. See {@link #valueMatchesType} for the
    * definitive story.
@@ -228,9 +231,9 @@ public class RexLiteral extends RexNode {
     this.value = value;
     this.type = requireNonNull(type, "type");
     this.typeName = requireNonNull(typeName, "typeName");
-    Preconditions.checkArgument(valueMatchesType(value, typeName, true));
-    Preconditions.checkArgument((value == null) == type.isNullable());
-    Preconditions.checkArgument(typeName != SqlTypeName.ANY);
+    checkArgument(valueMatchesType(value, typeName, true));
+    checkArgument((value == null) == type.isNullable());
+    checkArgument(typeName != SqlTypeName.ANY);
     this.digest = computeDigest(RexDigestIncludeType.OPTIONAL);
   }
 
@@ -291,10 +294,10 @@ public class RexLiteral extends RexNode {
   }
 
   /**
-   * Returns true if {@link RexDigestIncludeType#OPTIONAL} digest would include data type.
+   * Returns whether {@link RexDigestIncludeType} digest would include data type.
    *
    * @see RexCall#computeDigest(boolean)
-   * @return true if {@link RexDigestIncludeType#OPTIONAL} digest would include data type
+   * @return whether {@link RexDigestIncludeType} digest would include data type
    */
   @RequiresNonNull("type")
   RexDigestIncludeType digestIncludesType(
@@ -325,21 +328,24 @@ public class RexLiteral extends RexNode {
       }
       // fall through
     case DECIMAL:
+    case BIGINT:
+      return value instanceof BigDecimal;
     case DOUBLE:
     case FLOAT:
     case REAL:
-    case BIGINT:
-      return value instanceof BigDecimal;
+      return value instanceof Double;
     case DATE:
       return value instanceof DateString;
     case TIME:
-      return value instanceof TimeString;
     case TIME_WITH_LOCAL_TIME_ZONE:
       return value instanceof TimeString;
+    case TIME_TZ:
+      return value instanceof TimeWithTimeZoneString;
     case TIMESTAMP:
-      return value instanceof TimestampString;
     case TIMESTAMP_WITH_LOCAL_TIME_ZONE:
       return value instanceof TimestampString;
+    case TIMESTAMP_TZ:
+      return value instanceof TimestampWithTimeZoneString;
     case INTERVAL_YEAR:
     case INTERVAL_YEAR_MONTH:
     case INTERVAL_MONTH:
@@ -406,6 +412,7 @@ public class RexLiteral extends RexNode {
       return SqlTypeName.DECIMAL;
     case REAL:
     case FLOAT:
+    case DOUBLE:
       return SqlTypeName.DOUBLE;
     case VARBINARY:
       return SqlTypeName.BINARY;
@@ -646,20 +653,27 @@ public class RexLiteral extends RexNode {
       break;
     case BOOLEAN:
       assert value instanceof Boolean;
-      sb.append(value.toString());
+      sb.append(value);
       break;
     case DECIMAL:
       assert value instanceof BigDecimal;
-      sb.append(value.toString());
+      sb.append(value);
       break;
     case DOUBLE:
-      assert value instanceof BigDecimal;
-      sb.append(Util.toScientificNotation((BigDecimal) value));
+    case FLOAT:
+      if (value instanceof BigDecimal) {
+        sb.append(Util.toScientificNotation((BigDecimal) value));
+      } else {
+        assert value instanceof Double;
+        Double d = (Double) value;
+        String repr = Util.toScientificNotation(d);
+        sb.append(repr);
+      }
       break;
     case BIGINT:
       assert value instanceof BigDecimal;
       long narrowLong = ((BigDecimal) value).longValue();
-      sb.append(String.valueOf(narrowLong));
+      sb.append(narrowLong);
       sb.append('L');
       break;
     case BINARY:
@@ -681,22 +695,30 @@ public class RexLiteral extends RexNode {
     case SYMBOL:
       assert value instanceof Enum;
       sb.append("FLAG(");
-      sb.append(value.toString());
+      sb.append(value);
       sb.append(")");
       break;
     case DATE:
       assert value instanceof DateString;
-      sb.append(value.toString());
+      sb.append(value);
       break;
     case TIME:
     case TIME_WITH_LOCAL_TIME_ZONE:
       assert value instanceof TimeString;
-      sb.append(value.toString());
+      sb.append(value);
+      break;
+    case TIME_TZ:
+      assert value instanceof TimeWithTimeZoneString;
+      sb.append(value);
       break;
     case TIMESTAMP:
     case TIMESTAMP_WITH_LOCAL_TIME_ZONE:
       assert value instanceof TimestampString;
-      sb.append(value.toString());
+      sb.append(value);
+      break;
+    case TIMESTAMP_TZ:
+      assert value instanceof TimestampWithTimeZoneString;
+      sb.append(value);
       break;
     case INTERVAL_YEAR:
     case INTERVAL_YEAR_MONTH:
@@ -712,7 +734,7 @@ public class RexLiteral extends RexNode {
     case INTERVAL_MINUTE_SECOND:
     case INTERVAL_SECOND:
       assert value instanceof BigDecimal;
-      sb.append(value.toString());
+      sb.append(value);
       break;
     case MULTISET:
     case ROW:
@@ -806,6 +828,8 @@ public class RexLiteral extends RexNode {
       return new RexLiteral(b, type, typeName);
     case DECIMAL:
     case DOUBLE:
+    case REAL:
+    case FLOAT:
       BigDecimal d = new BigDecimal(literal);
       return new RexLiteral(d, type, typeName);
     case BINARY:
@@ -855,7 +879,7 @@ public class RexLiteral extends RexNode {
         break;
       default:
         // Allow fractional seconds for times and timestamps
-        assert format != null;
+        requireNonNull(format, "format");
         final DateTimeUtils.PrecisionTime ts =
             DateTimeUtils.parsePrecisionDateTimeLiteral(literal,
                 new SimpleDateFormat(format, Locale.ROOT), tz, -1);
@@ -953,10 +977,12 @@ public class RexLiteral extends RexNode {
     case DECIMAL:
     case TIMESTAMP:
     case TIMESTAMP_WITH_LOCAL_TIME_ZONE:
+    case TIMESTAMP_TZ:
       return getValueAs(Long.class);
     case DATE:
     case TIME:
     case TIME_WITH_LOCAL_TIME_ZONE:
+    case TIME_TZ:
       return getValueAs(Integer.class);
     default:
       return value;
@@ -1056,22 +1082,48 @@ public class RexLiteral extends RexNode {
     case BIGINT:
     case INTEGER:
     case SMALLINT:
-    case TINYINT:
+    case TINYINT: {
+      BigDecimal bd = (BigDecimal) value;
+      if (clazz == Long.class) {
+        return clazz.cast(bd.longValue());
+      } else if (clazz == Integer.class) {
+        return clazz.cast(bd.intValue());
+      } else if (clazz == Short.class) {
+        return clazz.cast(bd.shortValue());
+      } else if (clazz == Byte.class) {
+        return clazz.cast(bd.byteValue());
+      } else if (clazz == Double.class) {
+        return clazz.cast(bd.doubleValue());
+      } else if (clazz == Float.class) {
+        return clazz.cast(bd.floatValue());
+      }
+      break;
+    }
     case DOUBLE:
     case REAL:
     case FLOAT:
-      if (clazz == Long.class) {
-        return clazz.cast(((BigDecimal) value).longValue());
-      } else if (clazz == Integer.class) {
-        return clazz.cast(((BigDecimal) value).intValue());
-      } else if (clazz == Short.class) {
-        return clazz.cast(((BigDecimal) value).shortValue());
-      } else if (clazz == Byte.class) {
-        return clazz.cast(((BigDecimal) value).byteValue());
-      } else if (clazz == Double.class) {
-        return clazz.cast(((BigDecimal) value).doubleValue());
-      } else if (clazz == Float.class) {
-        return clazz.cast(((BigDecimal) value).floatValue());
+      if (value instanceof Double) {
+        Double d = (Double) value;
+        if (clazz == Long.class) {
+          return clazz.cast(d.longValue());
+        } else if (clazz == Integer.class) {
+          return clazz.cast(d.intValue());
+        } else if (clazz == Short.class) {
+          return clazz.cast(d.shortValue());
+        } else if (clazz == Byte.class) {
+          return clazz.cast(d.byteValue());
+        } else if (clazz == Double.class) {
+          // Cast still needed, since the Java compiler does not understand
+          // that T is double.
+          return clazz.cast(d);
+        } else if (clazz == Float.class) {
+          return clazz.cast(d.floatValue());
+        } else if (clazz == BigDecimal.class) {
+          // This particular conversion is lossy, since in general BigDecimal cannot
+          // represent accurately FP values.  However, this is the best we can do.
+          // This conversion used to be in RexBuilder, used when creating a RexLiteral.
+          return clazz.cast(new BigDecimal(d, MathContext.DECIMAL64).stripTrailingZeros());
+        }
       }
       break;
     case DATE:
@@ -1095,6 +1147,11 @@ public class RexLiteral extends RexNode {
         return clazz.cast(((TimeString) value).getMillisOfDay());
       }
       break;
+    case TIME_TZ:
+      if (clazz == Integer.class) {
+        return clazz.cast(((TimeWithTimeZoneString) value).getLocalTimeString().getMillisOfDay());
+      }
+      break;
     case TIMESTAMP:
       if (clazz == Long.class) {
         // Milliseconds since 1970-01-01 00:00:00
@@ -1104,6 +1161,16 @@ public class RexLiteral extends RexNode {
         return clazz.cast(((TimestampString) value).toCalendar());
       }
       break;
+    case TIMESTAMP_TZ:
+      if (clazz == Long.class) {
+        return clazz.cast(((TimestampWithTimeZoneString) value)
+            .getLocalTimestampString()
+            .getMillisSinceEpoch());
+      } else if (clazz == Calendar.class) {
+        TimestampWithTimeZoneString ts = (TimestampWithTimeZoneString) value;
+        return clazz.cast(ts.getLocalTimestampString().toCalendar(ts.getTimeZone()));
+      }
+      break;
     case TIMESTAMP_WITH_LOCAL_TIME_ZONE:
       if (clazz == Long.class) {
         // Milliseconds since 1970-01-01 00:00:00
@@ -1179,9 +1246,18 @@ public class RexLiteral extends RexNode {
     return findValue(node);
   }
 
-  public static int intValue(RexNode node) {
+  /** Returns the value of a literal, cast, or unary minus, as a number;
+   * never null. */
+  public static Number numberValue(RexNode node) {
     final Comparable value = castNonNull(findValue(node));
-    return ((Number) value).intValue();
+    return (Number) value;
+  }
+
+  /** Returns the value of a literal, cast, or unary minus, as an int;
+   * never null. */
+  public static int intValue(RexNode node) {
+    final Number number = numberValue(node);
+    return number.intValue();
   }
 
   public static @Nullable String stringValue(RexNode node) {
