@@ -62,20 +62,21 @@ import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.nio.charset.UnsupportedCharsetException;
 import java.sql.DatabaseMetaData;
-import java.sql.SQLException;
 import java.text.MessageFormat;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
-import java.util.Objects;
 import java.util.function.Predicate;
 import java.util.stream.Collectors;
 
 import static org.apache.calcite.util.Static.RESOURCE;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * Contains utility functions related to SQL parsing, all static.
  */
@@ -127,7 +128,9 @@ public abstract class SqlUtil {
       SqlSelect query,
       int ordinal) {
     SqlNode from = query.getFrom();
-    assert from != null : "from must not be null for " + query;
+    if (from == null) {
+      throw new AssertionError("from must not be null for " + query);
+    }
     ArrayList<SqlNode> list = flatten(from);
     return list.get(ordinal);
   }
@@ -158,9 +161,7 @@ public abstract class SqlUtil {
   /** Converts a SqlNode array to a SqlNodeList. */
   public static SqlNodeList toNodeList(SqlNode[] operands) {
     SqlNodeList ret = new SqlNodeList(SqlParserPos.ZERO);
-    for (SqlNode node : operands) {
-      ret.add(node);
-    }
+    Collections.addAll(ret, operands);
     return ret;
   }
 
@@ -206,10 +207,8 @@ public abstract class SqlUtil {
     if (allowCast && node != null) {
       if (node.getKind() == SqlKind.CAST) {
         SqlCall call = (SqlCall) node;
-        if (isNullLiteral(call.operand(0), false)) {
-          // node is "CAST(NULL as type)"
-          return true;
-        }
+        // node is "CAST(NULL as type)"
+        return isNullLiteral(call.operand(0), false);
       }
     }
     return false;
@@ -244,7 +243,7 @@ public abstract class SqlUtil {
    * @return Whether the node is a literal
    */
   public static boolean isLiteral(SqlNode node, boolean allowCast) {
-    assert node != null;
+    requireNonNull(node, "node");
     if (node instanceof SqlLiteral) {
       return true;
     }
@@ -288,7 +287,7 @@ public abstract class SqlUtil {
    * @return Whether the node is a literal chain
    */
   public static boolean isLiteralChain(SqlNode node) {
-    assert node != null;
+    requireNonNull(node, "node");
     if (node instanceof SqlCall) {
       SqlCall call = (SqlCall) node;
       return call.getKind() == SqlKind.LITERAL_CHAIN;
@@ -418,7 +417,7 @@ public abstract class SqlUtil {
         writer.sep(".");
         final String name = identifier.names.get(i);
         final SqlParserPos pos = identifier.getComponentParserPosition(i);
-        if (name.equals("")) {
+        if (name.isEmpty()) {
           writer.print("*");
           writer.setNeedWhitespace(true);
         } else {
@@ -519,7 +518,8 @@ public abstract class SqlUtil {
   private static Iterator<SqlOperator> filterOperatorRoutinesByKind(
       Iterator<SqlOperator> routines, final SqlKind sqlKind) {
     return Iterators.filter(routines,
-        operator -> Objects.requireNonNull(operator, "operator").getKind() == sqlKind);
+        operator -> requireNonNull(operator, "operator")
+            .getKind().getFunctionKind() == sqlKind);
   }
 
   /**
@@ -629,7 +629,8 @@ public abstract class SqlUtil {
           Predicates.instanceOf(SqlFunction.class));
     default:
       return Iterators.filter(sqlOperators.iterator(),
-          operator -> Objects.requireNonNull(operator, "operator").getSyntax() == syntax);
+          operator ->
+              requireNonNull(operator, "operator").getSyntax() == syntax);
     }
   }
 
@@ -637,7 +638,7 @@ public abstract class SqlUtil {
       Iterator<SqlOperator> routines,
       final List<RelDataType> argTypes) {
     return Iterators.filter(routines,
-        operator -> Objects.requireNonNull(operator, "operator")
+        operator -> requireNonNull(operator, "operator")
             .getOperandCountRange().isValidCount(argTypes.size()));
   }
 
@@ -660,7 +661,7 @@ public abstract class SqlUtil {
         Iterators.filter(routines, SqlFunction.class),
         function -> {
           SqlOperandTypeChecker operandTypeChecker =
-              Objects.requireNonNull(function, "function").getOperandTypeChecker();
+              requireNonNull(function, "function").getOperandTypeChecker();
           if (operandTypeChecker == null
               || !operandTypeChecker.isFixedParameters()) {
             // no parameter information for builtins; keep for now,
@@ -805,15 +806,17 @@ public abstract class SqlUtil {
     switch (query.getKind()) {
     case SELECT:
       SqlSelect select = (SqlSelect) query;
-      final SqlNode from = stripAs(select.getFrom());
-      if (from != null && from.getKind() == SqlKind.VALUES) {
-        // They wrote "VALUES (x, y)", but the validator has
-        // converted this into "SELECT * FROM VALUES (x, y)".
-        return getSelectListItem(from, i);
+      SqlNode from = select.getFrom();
+      if (from != null) {
+        from = stripAs(from);
+        if (from.getKind() == SqlKind.VALUES) {
+          // They wrote "VALUES (x, y)", but the validator has
+          // converted this into "SELECT * FROM VALUES (x, y)".
+          return getSelectListItem(from, i);
+        }
       }
-      final SqlNodeList fields = select.getSelectList();
 
-      assert fields != null : "fields must not be null in " + select;
+      final SqlNodeList fields = select.getSelectList();
       // Range check the index to avoid index out of range.  This
       // could be expanded to actually check to see if the select
       // list is a "*"
@@ -847,8 +850,9 @@ public abstract class SqlUtil {
    * @return true if alias is generated by calcite, otherwise false
    */
   public static boolean isGeneratedAlias(String alias) {
-    assert alias != null;
-    return alias.toUpperCase(Locale.ROOT).startsWith(GENERATED_EXPR_ALIAS_PREFIX);
+    return requireNonNull(alias, "alias")
+        .toUpperCase(Locale.ROOT)
+        .startsWith(GENERATED_EXPR_ALIAS_PREFIX);
   }
 
   /**
@@ -1032,9 +1036,10 @@ public abstract class SqlUtil {
    *     is available in this instance of the Java virtual machine
    */
   public static Charset getCharset(String charsetName) {
-    assert charsetName != null;
-    charsetName = charsetName.toUpperCase(Locale.ROOT);
-    String javaCharsetName = translateCharacterSetName(charsetName);
+    String javaCharsetName =
+        translateCharacterSetName(
+            requireNonNull(charsetName, "charsetName")
+                .toUpperCase(Locale.ROOT));
     if (javaCharsetName == null) {
       throw new UnsupportedCharsetException(charsetName);
     }
@@ -1049,7 +1054,6 @@ public abstract class SqlUtil {
    * @throws RuntimeException If the given value cannot be represented in the
    *     given charset
    */
-  @SuppressWarnings("BetaApi")
   public static void validateCharset(ByteString value, Charset charset) {
     if (charset == StandardCharsets.UTF_8) {
       final byte[] bytes = value.getBytes();
@@ -1097,7 +1101,7 @@ public abstract class SqlUtil {
       throw new AssertionError("not found: " + predicate + " in " + root);
     } catch (Util.FoundOne e) {
       //noinspection unchecked
-      return (ImmutableList<SqlNode>) Objects.requireNonNull(
+      return (ImmutableList<SqlNode>) requireNonNull(
           e.getNode(),
           "Genealogist result");
     }
@@ -1115,7 +1119,7 @@ public abstract class SqlUtil {
    */
   public static List<RelHint> getRelHint(HintStrategyTable hintStrategies,
       @Nullable SqlNodeList sqlHints) {
-    if (sqlHints == null || sqlHints.size() == 0) {
+    if (sqlHints == null || sqlHints.isEmpty()) {
       return ImmutableList.of();
     }
     final ImmutableList.Builder<RelHint> relHints = ImmutableList.builder();
@@ -1161,7 +1165,7 @@ public abstract class SqlUtil {
       List<RelHint> hints,
       Hintable rel) {
     final List<RelHint> relHints = hintStrategies.apply(hints, (RelNode) rel);
-    if (relHints.size() > 0) {
+    if (!relHints.isEmpty()) {
       return rel.attachHints(relHints);
     }
     return (RelNode) rel;
@@ -1262,7 +1266,7 @@ public abstract class SqlUtil {
 
   /** Returns whether an AST tree contains a call that matches a given
    * predicate. */
-  private static boolean containsCall(SqlNode node,
+  public static boolean containsCall(SqlNode node,
       Predicate<SqlCall> callPredicate) {
     try {
       SqlVisitor<Void> visitor =
@@ -1302,11 +1306,11 @@ public abstract class SqlUtil {
       this.identifierQuoteString = identifierQuoteString;
     }
 
-    public String getDatabaseProductName() throws SQLException {
+    public String getDatabaseProductName() {
       return databaseProductName;
     }
 
-    public String getIdentifierQuoteString() throws SQLException {
+    public String getIdentifierQuoteString() {
       return identifierQuoteString;
     }
   }
