@@ -20,14 +20,23 @@ import org.apache.calcite.avatica.util.TimeUnitRange;
 import org.apache.calcite.rel.type.RelDataType;
 import org.apache.calcite.rel.type.RelDataTypeComparability;
 import org.apache.calcite.rel.type.RelDataTypeFactory;
+import org.apache.calcite.sql.SqlCall;
 import org.apache.calcite.sql.SqlCallBinding;
+import org.apache.calcite.sql.SqlIdentifier;
 import org.apache.calcite.sql.SqlIntervalQualifier;
+import org.apache.calcite.sql.SqlLambda;
 import org.apache.calcite.sql.SqlLiteral;
 import org.apache.calcite.sql.SqlNode;
 import org.apache.calcite.sql.SqlOperandCountRange;
 import org.apache.calcite.sql.SqlOperator;
+import org.apache.calcite.sql.SqlOperatorBinding;
 import org.apache.calcite.sql.SqlUtil;
+import org.apache.calcite.sql.util.SqlBasicVisitor;
+import org.apache.calcite.sql.validate.SqlLambdaScope;
+import org.apache.calcite.sql.validate.SqlValidator;
 import org.apache.calcite.sql.validate.SqlValidatorScope;
+import org.apache.calcite.sql.validate.implicit.AbstractTypeCoercion;
+import org.apache.calcite.sql.validate.implicit.TypeCoercion;
 import org.apache.calcite.util.ImmutableIntList;
 import org.apache.calcite.util.Pair;
 import org.apache.calcite.util.Util;
@@ -49,6 +58,7 @@ import java.util.function.Predicate;
 
 import static com.google.common.base.Preconditions.checkArgument;
 
+import static org.apache.calcite.sql.type.NonNullableAccessors.getKeyTypeOrThrow;
 import static org.apache.calcite.util.Static.RESOURCE;
 
 import static java.util.Objects.requireNonNull;
@@ -101,6 +111,27 @@ public abstract class OperandTypes {
     return family(families, i -> false);
   }
 
+  /**
+   * Creates a checker that passes if the operand is a function with
+   * a given return type and parameter types. This method can be used
+   * to check a lambda expression.
+   */
+  public static SqlSingleOperandTypeChecker function(SqlTypeFamily returnTypeFamily,
+      SqlTypeFamily... paramTypeFamilies) {
+    return new LambdaFamilyOperandTypeChecker(
+        returnTypeFamily, ImmutableList.copyOf(paramTypeFamilies));
+  }
+
+  /**
+   * Creates a checker that passes if the operand is a function with
+   * a given return type and parameter types. This method can be used
+   * to check a lambda expression.
+   */
+  public static SqlSingleOperandTypeChecker function(SqlTypeFamily returnTypeFamily,
+      List<SqlTypeFamily> paramTypeFamilies) {
+    return new LambdaFamilyOperandTypeChecker(returnTypeFamily, paramTypeFamilies);
+  }
+
   /**
    * Creates a single-operand checker that passes if the operand's type has a
    * particular {@link SqlTypeName}.
@@ -237,6 +268,18 @@ public abstract class OperandTypes {
         ImmutableList.copyOf(rules), allowedSignatures, null, null);
   }
 
+  /**
+   * Creates an operand checker from a sequence of single-operand checkers,
+   * generating the signature from the components.
+   */
+  public static SqlOperandTypeChecker sequence(
+      BiFunction<SqlOperator, String, String> signatureGenerator,
+      SqlSingleOperandTypeChecker... rules) {
+    return new CompositeOperandTypeChecker(
+        CompositeOperandTypeChecker.Composition.SEQUENCE,
+        ImmutableList.copyOf(rules), null, signatureGenerator, null);
+  }
+
   /**
    * Creates a checker that passes if all of the rules pass for each operand,
    * using a given operand count strategy.
@@ -355,6 +398,9 @@ public abstract class OperandTypes {
   public static final SqlSingleOperandTypeChecker INTEGER =
       family(SqlTypeFamily.INTEGER);
 
+  public static final SqlSingleOperandTypeChecker INTEGER_INTEGER =
+      family(SqlTypeFamily.INTEGER, SqlTypeFamily.INTEGER);
+
   public static final SqlSingleOperandTypeChecker NUMERIC_OPTIONAL_NUMERIC =
       family(ImmutableList.of(SqlTypeFamily.NUMERIC, SqlTypeFamily.NUMERIC),
           // Second operand optional (operand index 0, 1)
@@ -365,6 +411,13 @@ public abstract class OperandTypes {
           // Second operand optional (operand index 0, 1)
           number -> number == 1);
 
+  public static final SqlOperandTypeChecker NUMERIC_INT32 =
+      sequence(
+          (operator, name) -> operator.getName() + "(<NUMERIC>, <INTEGER>)",
+          family(SqlTypeFamily.NUMERIC),
+          // Only 32-bit integer allowed for the second argument
+          new TypeNameChecker(SqlTypeName.INTEGER));
+
   public static final SqlSingleOperandTypeChecker NUMERIC_CHARACTER =
       family(SqlTypeFamily.NUMERIC, SqlTypeFamily.CHARACTER);
 
@@ -415,6 +468,10 @@ public abstract class OperandTypes {
           // Second and Third operand both are optional (operand index 0, 1, 2)
           number -> number == 1 || number == 2);
 
+  public static final FamilyOperandTypeChecker STRING_STRING_STRING_STRING =
+      family(SqlTypeFamily.STRING, SqlTypeFamily.STRING, SqlTypeFamily.STRING,
+          SqlTypeFamily.STRING);
+
   public static final FamilyOperandTypeChecker STRING_NUMERIC_OPTIONAL_STRING =
       family(
           ImmutableList.of(SqlTypeFamily.STRING, SqlTypeFamily.NUMERIC,
@@ -547,6 +604,112 @@ public abstract class OperandTypes {
         }
       };
 
+  /** Operand type-checking strategy that the first argument is of string type,
+   * and the remaining arguments can be any type. */
+  public static final SqlOperandTypeChecker STRING_FIRST_OBJECT_REPEAT =
+      new StringFirstAndRepeatOperandTypeChecker(
+          SqlOperandCountRanges.from(2), "(<STRING>(,<ANY>)+)") {
+        @Override public boolean checkRepeatOperandTypes(SqlCallBinding callBinding,
+            boolean throwOnFailure) {
+          ImmutableList.Builder<SqlTypeFamily> builder = ImmutableList.builder();
+          for (int i = 0; i < callBinding.getOperandCount(); i++) {
+            TypeCoercion coercion = callBinding.getValidator().getTypeCoercion();
+            RelDataType operandType = callBinding.getOperandType(i);
+            RelDataType cast =
+                ((AbstractTypeCoercion) coercion).implicitCast(operandType, SqlTypeFamily.STRING);
+            SqlTypeFamily family =
+                cast != null ? SqlTypeFamily.STRING
+                    : operandType.getSqlTypeName().getFamily();
+            requireNonNull(family, "family");
+            builder.add(family);
+          }
+          ImmutableList<SqlTypeFamily> families = builder.build();
+          return family(families).checkOperandTypes(callBinding, throwOnFailure);
+        }
+      };
+
+  /** Operand type-checking strategy that the first argument is of string type,
+   * and the remaining arguments can be of string or array of string type. */
+  public static final SqlOperandTypeChecker STRING_FIRST_STRING_ARRAY_REPEAT =
+      new StringFirstAndRepeatOperandTypeChecker(
+          SqlOperandCountRanges.from(1), "(<STRING>[,<STRING> | ARRAY<STRING>]+)") {
+        @Override public boolean checkRepeatOperandTypes(SqlCallBinding callBinding,
+            boolean throwOnFailure) {
+          // Check Array element is String type
+          if (!checkArrayString(callBinding, throwOnFailure)) {
+            return false;
+          }
+          // Check Operand Types with Type Coercion
+          return checkFamilyOperandTypes(callBinding, throwOnFailure);
+        }
+
+        private boolean checkFamilyOperandTypes(SqlCallBinding callBinding,
+            boolean throwOnFailure) {
+          ImmutableList.Builder<SqlTypeFamily> builder = ImmutableList.builder();
+          for (int i = 0; i < callBinding.getOperandCount(); i++) {
+            boolean isArray = callBinding.getOperandType(i).getSqlTypeName() == SqlTypeName.ARRAY;
+            SqlTypeFamily family = isArray ? SqlTypeFamily.ARRAY : SqlTypeFamily.STRING;
+            builder.add(family);
+          }
+          ImmutableList<SqlTypeFamily> families = builder.build();
+          return family(families).checkOperandTypes(callBinding, throwOnFailure);
+        }
+
+        private boolean checkArrayString(SqlCallBinding binding, boolean throwOnFailure) {
+          for (int i = 0; i < binding.getOperandCount(); i++) {
+            RelDataType type = binding.getOperandType(i);
+            RelDataType componentType = type.getComponentType();
+            boolean isString = componentType != null
+                && (SqlTypeUtil.isNull(componentType) || SqlTypeUtil.isString(componentType));
+            if (type.getSqlTypeName() == SqlTypeName.ARRAY && !isString) {
+              if (throwOnFailure) {
+                throw binding.newValidationSignatureError();
+              }
+              return false;
+            }
+          }
+          return true;
+        }
+      };
+
+  /** Operand type-checking strategy where the first argument is of string type,
+   * and the remaining arguments follow a repeating type pattern.
+   *
+   * <p>The method {@link #checkRepeatOperandTypes} is an abstract method designed
+   * to check the types of these repeating arguments. */
+  abstract static class StringFirstAndRepeatOperandTypeChecker implements SqlOperandTypeChecker {
+
+    private final SqlOperandCountRange countRange;
+    private final String signatures;
+
+    StringFirstAndRepeatOperandTypeChecker(
+        SqlOperandCountRange countRange,
+        String signatures) {
+      this.countRange = countRange;
+      this.signatures = signatures;
+    }
+
+    @Override public boolean checkOperandTypes(SqlCallBinding callBinding, boolean throwOnFailure) {
+      // Check first operand is String type
+      if (!STRING.checkSingleOperandType(callBinding, callBinding.operand(0), 0,
+          throwOnFailure)) {
+        return false;
+      }
+      return checkRepeatOperandTypes(callBinding, throwOnFailure);
+    }
+
+    @Override public SqlOperandCountRange getOperandCountRange() {
+      return countRange;
+    }
+
+    @Override public String getAllowedSignatures(SqlOperator op, String opName) {
+      return opName + signatures;
+    }
+
+    public abstract boolean checkRepeatOperandTypes(SqlCallBinding callBinding,
+        boolean throwOnFailure);
+  }
+
   /** Checks that returns whether a value is a multiset or an array.
    * Cf Java, where list and set are collections but a map is not. */
   public static final SqlSingleOperandTypeChecker COLLECTION =
@@ -560,8 +723,19 @@ public abstract class OperandTypes {
   public static final SqlSingleOperandTypeChecker MAP =
       family(SqlTypeFamily.MAP);
 
+  public static final SqlOperandTypeChecker ARRAY_FUNCTION =
+      new ArrayFunctionOperandTypeChecker();
+
   public static final SqlOperandTypeChecker ARRAY_ELEMENT =
-      new ArrayElementOperandTypeChecker();
+      new ArrayElementOperandTypeChecker(true, true);
+
+  public static final SqlOperandTypeChecker ARRAY_ELEMENT_NONNULL =
+      new ArrayElementOperandTypeChecker(false, true);
+
+  /** Type checker that accepts an ARRAY as the first argument, but not
+   * an expression with type NULL (i.e. a NULL literal). */
+  public static final SqlOperandTypeChecker ARRAY_NONNULL =
+      family(SqlTypeFamily.ARRAY).and(new NotNullOperandTypeChecker(1, false));
 
   public static final SqlOperandTypeChecker ARRAY_INSERT =
       new ArrayInsertOperandTypeChecker();
@@ -572,6 +746,9 @@ public abstract class OperandTypes {
   public static final SqlSingleOperandTypeChecker MAP_FUNCTION =
       new MapFunctionOperandTypeChecker();
 
+  public static final SqlOperandTypeChecker MAP_KEY =
+      new MapKeyOperandTypeChecker();
+
   /**
    * Operand type-checking strategy where type must be a literal or NULL.
    */
@@ -584,6 +761,12 @@ public abstract class OperandTypes {
   public static final SqlSingleOperandTypeChecker LITERAL =
       new LiteralOperandTypeChecker(false);
 
+  /**
+   * Operand type-checking strategy where all types must be non-NULL value.
+   */
+  public static final SqlSingleOperandTypeChecker NONNULL_NONNULL =
+      new NotNullOperandTypeChecker(2, false);
+
   /**
    * Operand type-checking strategy type must be a boolean non-NULL literal.
    */
@@ -918,6 +1101,19 @@ public abstract class OperandTypes {
 
   public static final SqlSingleOperandTypeChecker ANY_ANY =
       family(SqlTypeFamily.ANY, SqlTypeFamily.ANY);
+
+  /**
+   * Operand type-checking strategy where the second and third operands must be comparable.
+   * This is used when the operator has three operands and only the
+   * second and third operands need to be comparable.
+   */
+  public static final SqlSingleOperandTypeChecker SECOND_THIRD_SAME =
+      new SameOperandTypeChecker(3) {
+        @Override protected List<Integer> getOperandList(int operandCount) {
+          // Only check the second and third operands
+          return ImmutableList.of(1, 2);
+        }
+      };
   public static final SqlSingleOperandTypeChecker ANY_IGNORE =
       family(SqlTypeFamily.ANY, SqlTypeFamily.IGNORE);
   public static final SqlSingleOperandTypeChecker IGNORE_ANY =
@@ -959,7 +1155,7 @@ public abstract class OperandTypes {
   public static final SqlSingleOperandTypeChecker CURSOR =
       family(SqlTypeFamily.CURSOR);
 
-  public static final SqlOperandTypeChecker MEASURE =
+  public static final SqlSingleOperandTypeChecker MEASURE =
       new FamilyOperandTypeChecker(ImmutableList.of(SqlTypeFamily.ANY),
           i -> false) {
         @Override public boolean checkSingleOperandType(
@@ -982,6 +1178,9 @@ public abstract class OperandTypes {
         }
       };
 
+  public static final SqlOperandTypeChecker MEASURE_BOOLEAN =
+      sequence("'<MEASURE>, <BOOLEAN>'", MEASURE, BOOLEAN);
+
   /**
    * Parameter type-checking strategy where type must a nullable time interval,
    * nullable time interval.
@@ -1004,6 +1203,9 @@ public abstract class OperandTypes {
   public static final SqlSingleOperandTypeChecker DATE_INTERVAL =
       family(SqlTypeFamily.DATE, SqlTypeFamily.DATETIME_INTERVAL);
 
+  public static final SqlSingleOperandTypeChecker DATE_ANY =
+      family(SqlTypeFamily.DATE, SqlTypeFamily.ANY);
+
   public static final SqlSingleOperandTypeChecker DATE_CHARACTER =
       family(SqlTypeFamily.DATE, SqlTypeFamily.CHARACTER);
 
@@ -1032,7 +1234,8 @@ public abstract class OperandTypes {
       family(SqlTypeFamily.DATETIME_INTERVAL, SqlTypeFamily.DATETIME);
 
   public static final SqlSingleOperandTypeChecker INTERVALINTERVAL_INTERVALDATETIME =
-      INTERVAL_SAME_SAME.or(INTERVAL_DATETIME);
+      INTERVAL_SAME_SAME.or(INTERVAL_DATETIME)
+          .or(family(SqlTypeFamily.INTERVAL_DAY_TIME, SqlTypeFamily.INTERVAL_YEAR_MONTH));
 
   // TODO: datetime+interval checking missing
   // TODO: interval+datetime checking missing
@@ -1104,6 +1307,39 @@ public abstract class OperandTypes {
         }
       };
 
+  public static final SqlOperandTypeChecker EXISTS =
+      new SqlOperandTypeChecker() {
+        @Override public boolean checkOperandTypes(
+            SqlCallBinding callBinding,
+            boolean throwOnFailure) {
+          // The first operand must be an array type
+          ARRAY.checkSingleOperandType(callBinding, callBinding.operand(0), 0, throwOnFailure);
+          final RelDataType arrayType =
+              SqlTypeUtil.deriveType(callBinding, callBinding.operand(0));
+          final RelDataType componentType =
+              requireNonNull(arrayType.getComponentType(), "componentType");
+
+          // The second operand is a function(array_element_type)->boolean type
+          LambdaRelOperandTypeChecker lambdaChecker =
+              new LambdaRelOperandTypeChecker(
+                  SqlTypeFamily.BOOLEAN,
+                  ImmutableList.of(componentType));
+          return lambdaChecker.checkSingleOperandType(
+              callBinding,
+              callBinding.operand(1),
+              1,
+              throwOnFailure);
+        }
+
+        @Override public SqlOperandCountRange getOperandCountRange() {
+          return SqlOperandCountRanges.of(2);
+        }
+
+        @Override public String getAllowedSignatures(SqlOperator op, String opName) {
+          return "EXISTS(<ARRAY>, <FUNCTION(ARRAY_ELEMENT_TYPE)->BOOLEAN>)";
+        }
+      };
+
   /**
    * Checker for record just has one field.
    */
@@ -1225,6 +1461,58 @@ public abstract class OperandTypes {
     }
   }
 
+  /**
+   * Operand type-checking strategy for a ARRAY function, it allows empty array.
+   */
+  private static class ArrayFunctionOperandTypeChecker
+      extends SameOperandTypeChecker {
+
+    ArrayFunctionOperandTypeChecker() {
+      // The args of array are non-fixed, so we set to -1 here. then operandCount
+      // can dynamically set according to the number of input args.
+      // details please see SameOperandTypeChecker#getOperandList.
+      super(-1);
+    }
+
+    @Override protected boolean checkOperandTypesImpl(
+        SqlOperatorBinding operatorBinding,
+        boolean throwOnFailure,
+        @Nullable SqlCallBinding callBinding) {
+      if (throwOnFailure && callBinding == null) {
+        throw new IllegalArgumentException(
+            "callBinding must be non-null in case throwOnFailure=true");
+      }
+      int nOperandsActual = nOperands;
+      if (nOperandsActual == -1) {
+        nOperandsActual = operatorBinding.getOperandCount();
+      }
+      RelDataType[] types = new RelDataType[nOperandsActual];
+      final List<Integer> operandList =
+          getOperandList(operatorBinding.getOperandCount());
+      for (int i : operandList) {
+        types[i] = operatorBinding.getOperandType(i);
+      }
+      for (int i : operandList) {
+        if (i > 0) {
+          // we replace SqlTypeUtil.isComparable with SqlTypeUtil.leastRestrictiveForComparison
+          // to handle struct type and NULL constant.
+          // details please see: https://issues.apache.org/jira/browse/CALCITE-6163
+          RelDataType type =
+              SqlTypeUtil.leastRestrictiveForComparison(operatorBinding.getTypeFactory(),
+                  types[i], types[i - 1]);
+          if (type == null) {
+            if (!throwOnFailure) {
+              return false;
+            }
+            throw requireNonNull(callBinding, "callBinding").newValidationError(
+                RESOURCE.needSameTypeParameter());
+          }
+        }
+      }
+      return true;
+    }
+  }
+
   /**
    * Operand type-checking strategy for a MAP function, it allows empty map.
    */
@@ -1324,6 +1612,47 @@ public abstract class OperandTypes {
     }
   }
 
+  /**
+   * Parameter type-checking strategy where types must be Map and Map key type.
+   */
+  private static class MapKeyOperandTypeChecker extends SameOperandTypeChecker {
+    MapKeyOperandTypeChecker() {
+      super(2);
+    }
+
+    @Override public boolean checkOperandTypes(
+        SqlCallBinding callBinding,
+        boolean throwOnFailure) {
+      final SqlNode op0 = callBinding.operand(0);
+      if (!OperandTypes.MAP.checkSingleOperandType(
+          callBinding,
+          op0,
+          0,
+          throwOnFailure)) {
+        return false;
+      }
+
+      final RelDataType mapKeyType =
+          getKeyTypeOrThrow(SqlTypeUtil.deriveType(callBinding, op0));
+      final SqlNode op1 = callBinding.operand(1);
+      RelDataType opType1 = SqlTypeUtil.deriveType(callBinding, op1);
+
+      RelDataType biggest =
+          callBinding.getTypeFactory().leastRestrictive(
+              ImmutableList.of(mapKeyType, opType1));
+      if (biggest == null) {
+        if (throwOnFailure) {
+          throw callBinding.newError(
+              RESOURCE.typeNotComparable(
+                  mapKeyType.toString(), opType1.toString()));
+        }
+
+        return false;
+      }
+      return true;
+    }
+  }
+
   /** Checker that passes if the operand's type has a particular
    * {@link SqlTypeName}. */
   private static class TypeNameChecker implements SqlSingleOperandTypeChecker,
@@ -1356,4 +1685,204 @@ public abstract class OperandTypes {
       return opName + "(" + typeName.getSpaceName() + ")";
     }
   }
+
+  /**
+   * Operand type-checking strategy where the type of the operand is a lambda
+   * expression with a given return type and argument {@link SqlTypeFamily}s.
+   */
+  private static class LambdaFamilyOperandTypeChecker
+      extends LambdaOperandTypeChecker {
+
+    private final List<SqlTypeFamily> argFamilies;
+
+    LambdaFamilyOperandTypeChecker(
+        SqlTypeFamily returnTypeFamily,
+        List<SqlTypeFamily> argFamilies) {
+      super(returnTypeFamily);
+      this.argFamilies = ImmutableList.copyOf(argFamilies);
+    }
+
+    @Override public String getAllowedSignatures(SqlOperator op, String opName) {
+      ImmutableList.Builder<SqlTypeFamily> builder = ImmutableList.builder();
+      builder.addAll(argFamilies);
+      builder.add(returnTypeFamily);
+
+      return SqlUtil.getAliasedSignature(op, opName, builder.build());
+    }
+
+    @Override public boolean checkOperandTypes(SqlCallBinding callBinding,
+        boolean throwOnFailure) {
+      return false;
+    }
+
+    @Override public boolean checkSingleOperandType(
+        SqlCallBinding callBinding, SqlNode operand, int iFormalOperand, boolean throwOnFailure) {
+      if (!(operand instanceof SqlLambda)
+          || ((SqlLambda) operand).getParameters().size() != argFamilies.size()) {
+        if (throwOnFailure) {
+          throw callBinding.newValidationSignatureError();
+        }
+        return false;
+      }
+
+      final SqlLambda lambdaExpr = (SqlLambda) operand;
+      if (SqlUtil.isNullLiteral(lambdaExpr.getExpression(), false)) {
+        checkNull(callBinding, lambdaExpr, throwOnFailure);
+      }
+
+      final SqlValidator validator = callBinding.getValidator();
+      if (!lambdaExpr.getParameters().isEmpty()
+          && !argFamilies.stream().allMatch(f -> f == SqlTypeFamily.ANY)) {
+        // Replace the parameter types in the lambda expression.
+        final SqlLambdaScope scope =
+            (SqlLambdaScope) validator.getLambdaScope(lambdaExpr);
+        for (int i = 0; i < argFamilies.size(); i++) {
+          final SqlNode param = lambdaExpr.getParameters().get(i);
+          final RelDataType type =
+              argFamilies.get(i).getDefaultConcreteType(callBinding.getTypeFactory());
+          if (type != null) {
+            scope.getParameterTypes().put(param.toString(), type);
+          }
+        }
+        lambdaExpr.accept(new TypeRemover(validator));
+        // Given the new relDataType, re-validate the lambda expression.
+        validator.validateLambda(lambdaExpr);
+      }
+
+      return checkReturnType(validator, callBinding, lambdaExpr, throwOnFailure);
+    }
+  }
+
+  /**
+   * Operand type-checking strategy where the type of the operand is a lambda
+   * expression with a given return type and argument {@link RelDataType}s.
+   */
+  private static class LambdaRelOperandTypeChecker
+      extends LambdaOperandTypeChecker {
+    private final List<RelDataType> argTypes;
+
+    LambdaRelOperandTypeChecker(
+        SqlTypeFamily returnTypeFamily,
+        List<RelDataType> argTypes) {
+      super(returnTypeFamily);
+      this.argTypes = argTypes;
+    }
+
+    @Override public String getAllowedSignatures(SqlOperator op, String opName) {
+      ImmutableList.Builder<SqlTypeFamily> builder = ImmutableList.builder();
+      argTypes.stream()
+          .map(t -> requireNonNull(t.getSqlTypeName().getFamily()))
+          .forEach(builder::add);
+      builder.add(returnTypeFamily);
+
+      return SqlUtil.getAliasedSignature(op, opName, builder.build());
+    }
+
+    @Override public boolean checkSingleOperandType(SqlCallBinding callBinding, SqlNode operand,
+        int iFormalOperand,
+        boolean throwOnFailure) {
+      if (!(operand instanceof SqlLambda)
+          || ((SqlLambda) operand).getParameters().size() != argTypes.size()) {
+        if (throwOnFailure) {
+          throw callBinding.newValidationSignatureError();
+        }
+        return false;
+      }
+
+      final SqlLambda lambdaExpr = (SqlLambda) operand;
+      if (SqlUtil.isNullLiteral(lambdaExpr.getExpression(), false)) {
+        checkNull(callBinding, lambdaExpr, throwOnFailure);
+      }
+
+      // Replace the parameter types in the lambda expression.
+      final SqlValidator validator = callBinding.getValidator();
+      final SqlLambdaScope scope =
+          (SqlLambdaScope) validator.getLambdaScope(lambdaExpr);
+      for (int i = 0; i < argTypes.size(); i++) {
+        final SqlNode param = lambdaExpr.getParameters().get(i);
+        final RelDataType type = argTypes.get(i);
+        if (type != null) {
+          scope.getParameterTypes().put(param.toString(), type);
+        }
+      }
+      lambdaExpr.accept(new TypeRemover(validator));
+      // Given the new relDataType, re-validate the lambda expression.
+      validator.validateLambda(lambdaExpr);
+
+      return checkReturnType(validator, callBinding, lambdaExpr, throwOnFailure);
+    }
+  }
+
+  /**
+   * Abstract base class for type-checking strategies involving lambda expressions.
+   * This class provides common functionality for checking the type of lambda expression.
+   */
+  private abstract static class LambdaOperandTypeChecker
+      implements SqlSingleOperandTypeChecker {
+    protected final SqlTypeFamily returnTypeFamily;
+
+    LambdaOperandTypeChecker(SqlTypeFamily returnTypeFamily) {
+      this.returnTypeFamily = requireNonNull(returnTypeFamily, "returnTypeFamily");
+    }
+
+    protected boolean checkNull(
+        SqlCallBinding callBinding,
+        SqlLambda lambdaExpr,
+        boolean throwOnFailure) {
+      if (callBinding.isTypeCoercionEnabled()) {
+        return true;
+      }
+
+      if (throwOnFailure) {
+        throw callBinding.getValidator().newValidationError(lambdaExpr.getExpression(),
+            RESOURCE.nullIllegal());
+      }
+      return false;
+    }
+
+    protected boolean checkReturnType(
+        SqlValidator validator,
+        SqlCallBinding callBinding,
+        SqlLambda lambdaExpr,
+        boolean throwOnFailure) {
+      final RelDataType newType = validator.getValidatedNodeType(lambdaExpr);
+      assert newType instanceof FunctionSqlType;
+      final SqlTypeName returnTypeName =
+          ((FunctionSqlType) newType).getReturnType().getSqlTypeName();
+      if (returnTypeName == SqlTypeName.ANY
+          || returnTypeFamily.getTypeNames().contains(returnTypeName)) {
+        return true;
+      }
+
+      if (throwOnFailure) {
+        throw callBinding.newValidationSignatureError();
+      }
+      return false;
+    }
+
+    /**
+     * Visitor that removes the relDataType of a sqlNode and its children in the
+     * validator. Now this visitor is only used for removing the relDataType
+     * when we check lambda operand. Since lambda expressions will be
+     * validated for the second time based on the given parameter type,
+     * the type cached during the first validation must be cleared.
+     */
+    protected static class TypeRemover extends SqlBasicVisitor<Void> {
+      private final SqlValidator validator;
+
+      protected TypeRemover(SqlValidator validator) {
+        this.validator = validator;
+      }
+
+      @Override public Void visit(SqlIdentifier id) {
+        validator.removeValidatedNodeType(id);
+        return super.visit(id);
+      }
+
+      @Override public Void visit(SqlCall call) {
+        validator.removeValidatedNodeType(call);
+        return super.visit(call);
+      }
+    }
+  }
 }
