@@ -22,6 +22,7 @@ import org.apache.calcite.rel.RelCollations;
 import org.apache.calcite.rel.RelFieldCollation;
 import org.apache.calcite.rel.core.AggregateCall;
 import org.apache.calcite.rel.type.RelDataType;
+import org.apache.calcite.rel.type.RelDataTypeFactory;
 import org.apache.calcite.rex.RexBuilder;
 import org.apache.calcite.rex.RexInputRef;
 import org.apache.calcite.rex.RexNode;
@@ -38,10 +39,13 @@ import org.apache.calcite.sql.SqlNode;
 import org.apache.calcite.sql.SqlNodeList;
 import org.apache.calcite.sql.SqlSelectKeyword;
 import org.apache.calcite.sql.SqlUtil;
+import org.apache.calcite.sql.fun.SqlInternalOperators;
+import org.apache.calcite.sql.fun.SqlLibraryOperators;
 import org.apache.calcite.sql.fun.SqlStdOperatorTable;
 import org.apache.calcite.sql.parser.SqlParserPos;
 import org.apache.calcite.sql.util.SqlVisitor;
 import org.apache.calcite.sql.validate.AggregatingSelectScope;
+import org.apache.calcite.sql.validate.SqlValidator;
 import org.apache.calcite.sql.validate.SqlValidatorUtil;
 import org.apache.calcite.util.ImmutableBitSet;
 import org.apache.calcite.util.Litmus;
@@ -59,6 +63,7 @@ import java.util.Map;
 import java.util.stream.Collectors;
 
 import static org.apache.calcite.linq4j.Nullness.castNonNull;
+import static org.apache.calcite.sql.type.SqlTypeUtil.fromMeasure;
 
 /**
  * Converts expressions to aggregates.
@@ -237,6 +242,16 @@ class AggConverter implements SqlVisitor<Void> {
   }
 
   @Override public Void visit(SqlIdentifier id) {
+    if (isMeasureExpr(id)) {
+      final SqlCall call =
+          SqlInternalOperators.AGG_M2V.createCall(SqlParserPos.ZERO, id);
+      final SqlValidator validator = bb.getValidator();
+      final RelDataType measureType = validator.getValidatedNodeType(id);
+      final RelDataTypeFactory typeFactory = bb.getTypeFactory();
+      final RelDataType valueType = fromMeasure(typeFactory, measureType);
+      validator.setValidatedNodeType(call, valueType);
+      translateAgg(call);
+    }
     return null;
   }
 
@@ -485,6 +500,9 @@ class AggConverter implements SqlVisitor<Void> {
 
     SqlAggFunction aggFunction =
         (SqlAggFunction) call.getOperator();
+    if (aggFunction == SqlLibraryOperators.AGGREGATE) {
+      aggFunction = SqlInternalOperators.AGG_M2V;
+    }
     final RelDataType type = bb.getValidator().deriveType(bb.scope, call);
     boolean distinct = false;
     SqlLiteral quantifier = call.getFunctionQuantifier();
@@ -499,7 +517,7 @@ class AggConverter implements SqlVisitor<Void> {
       approximate = true;
     }
     final RelCollation collation;
-    if (orderList == null || orderList.size() == 0) {
+    if (orderList == null || orderList.isEmpty()) {
       collation = RelCollations.EMPTY;
     } else {
       try {
@@ -521,6 +539,7 @@ class AggConverter implements SqlVisitor<Void> {
     }
     final AggregateCall aggCall =
         AggregateCall.create(
+            call.getParserPosition(),
             aggFunction,
             distinct,
             approximate,
@@ -538,8 +557,16 @@ class AggConverter implements SqlVisitor<Void> {
             groupExprs.size(),
             aggCalls,
             aggCallMapping,
-            i -> convertedInputExprs.leftList().get(i).getType().isNullable());
+            i -> convertedInputExprs.left(i).getType().isNullable());
     aggMapping.put(outerCall, rex);
+    if (aggFunction.kind == SqlKind.AGG_M2V
+        && !distinct
+        && filterArg < 0
+        && distinctKeys == null
+        && args.size() == 1) {
+      // Allow "AGG_M2V(m)" to also be accessed via "m"
+      aggMapping.put(outerCall.operand(0), rex);
+    }
   }
 
   private RelFieldCollation sortToFieldCollation(SqlNode expr,
@@ -577,7 +604,7 @@ class AggConverter implements SqlVisitor<Void> {
   }
 
   boolean isMeasureExpr(SqlNode expr) {
-    return SqlUtil.indexOfDeep(measureExprs, expr, Litmus.IGNORE) >= 0;
+    return SqlUtil.indexOfDeep(getResolved().measureExprList, expr, Litmus.IGNORE) >= 0;
   }
 
   @Nullable RexNode lookupMeasure(SqlNode expr) {
@@ -595,7 +622,7 @@ class AggConverter implements SqlVisitor<Void> {
         final RexBuilder rexBuilder = bb.getRexBuilder();
         final int groupOrdinal = e.getValue().i;
         return converter.convert(rexBuilder,
-            convertedInputExprs.leftList().get(groupOrdinal),
+            convertedInputExprs.left(groupOrdinal),
             rexBuilder.makeInputRef(castNonNull(bb.root), groupOrdinal));
       }
     }
