@@ -69,6 +69,7 @@ import java.lang.reflect.Array;
 import java.lang.reflect.Field;
 import java.math.BigDecimal;
 import java.net.MalformedURLException;
+import java.net.URI;
 import java.net.URL;
 import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
@@ -115,6 +116,7 @@ import java.util.stream.Collector;
 import static org.apache.calcite.linq4j.Nullness.castNonNull;
 import static org.apache.calcite.util.ReflectUtil.isStatic;
 
+import static java.lang.Integer.parseInt;
 import static java.util.Objects.requireNonNull;
 
 /**
@@ -219,10 +221,7 @@ public class Util {
    * you are not interested in, but you don't want the compiler to warn that
    * you are not using it.
    */
-  public static void discard(@Nullable Object o) {
-    if (false) {
-      discard(o);
-    }
+  public static void discard(@Nullable Object unused) {
   }
 
   /**
@@ -230,10 +229,7 @@ public class Util {
    * you are not interested in, but you don't want the compiler to warn that
    * you are not using it.
    */
-  public static void discard(int i) {
-    if (false) {
-      discard(i);
-    }
+  public static void discard(int unused) {
   }
 
   /**
@@ -241,10 +237,7 @@ public class Util {
    * you are not interested in, but you don't want the compiler to warn that
    * you are not using it.
    */
-  public static void discard(boolean b) {
-    if (false) {
-      discard(b);
-    }
+  public static void discard(boolean unused) {
   }
 
   /**
@@ -252,10 +245,7 @@ public class Util {
    * you are not interested in, but you don't want the compiler to warn that
    * you are not using it.
    */
-  public static void discard(double d) {
-    if (false) {
-      discard(d);
-    }
+  public static void discard(double unused) {
   }
 
   /**
@@ -544,6 +534,21 @@ public class Util {
     pw.println();
   }
 
+  /**
+   * Formats a double value to a String ensuring that the output
+   * is in scientific notation if the value is not "special".
+   * (Special values include infinities and NaN.)
+   */
+  public static String toScientificNotation(Double d) {
+    String repr = Double.toString(d);
+    if (!repr.toLowerCase(Locale.ENGLISH).contains("e")
+        && !d.isInfinite()
+        && !d.isNaN()) {
+      repr += "E0";
+    }
+    return repr;
+  }
+
   /**
    * Formats a {@link BigDecimal} value to a string in scientific notation For
    * example<br>
@@ -568,6 +573,10 @@ public class Util {
     int len = unscaled.length();
     int scale = bd.scale();
     int e = len - scale - 1;
+    if (bd.stripTrailingZeros().equals(BigDecimal.ZERO)) {
+      // Without this adjustment 0.0 generates 0E-1
+      e = 0;
+    }
 
     StringBuilder ret = new StringBuilder();
     if (bd.signum() < 0) {
@@ -654,7 +663,7 @@ public class Util {
       }
     }
     path = "file://" + path;
-    return new URL(path);
+    return URI.create(path).toURL();
   }
 
   /**
@@ -1074,7 +1083,7 @@ public class Util {
    * feature has not been implemented, but should be.
    *
    * <p>If every 'hole' in our functionality uses this method, it will be
-   * easier for us to identity the holes. Throwing a
+   * easier for us to identify the holes. Throwing a
    * {@link java.lang.UnsupportedOperationException} isn't as good, because
    * sometimes we actually want to partially implement an API.
    *
@@ -1563,7 +1572,7 @@ public class Util {
     String value =
         requireNonNull(matcher.group(index),
             () -> "no group for index " + index + ", matcher " + matcher);
-    return Integer.parseInt(value);
+    return parseInt(value);
   }
 
   /**
@@ -2582,8 +2591,7 @@ public class Util {
     }
   }
 
-  /** Creates a {@link Calendar} in the UTC time zone and root locale.
-   * Does not use the time zone or locale. */
+  /** Creates a {@link Calendar} in the UTC time zone and root locale. */
   public static Calendar calendar() {
     return Calendar.getInstance(DateTimeUtils.UTC_ZONE, Locale.ROOT);
   }
@@ -2596,6 +2604,13 @@ public class Util {
     return calendar;
   }
 
+  /** Creates a {@link Calendar} in the specified time zone. */
+  public static Calendar calendar(long millis, TimeZone timeZone) {
+    Calendar calendar = Calendar.getInstance(timeZone, Locale.ROOT);
+    calendar.setTimeInMillis(millis);
+    return calendar;
+  }
+
   /**
    * Returns a {@code Collector} that accumulates the input elements into a
    * Guava {@link ImmutableList} via a {@link ImmutableList.Builder}.
@@ -2632,7 +2647,9 @@ public class Util {
   /** Transforms a list, applying a function to each element. */
   public static <F, T> List<T> transform(List<? extends F> list,
       java.util.function.Function<? super F, ? extends T> function) {
-    if (list instanceof RandomAccess) {
+    if (list.isEmpty() && list instanceof ImmutableList) {
+      return ImmutableList.of(); // save ourselves some effort
+    } else if (list instanceof RandomAccess) {
       return new RandomAccessTransformingList<>(list, function);
     } else {
       return new TransformingList<>(list, function);
@@ -2643,7 +2660,9 @@ public class Util {
    * the element's index in the list. */
   public static <F, T> List<T> transformIndexed(List<? extends F> list,
       BiFunction<? super F, Integer, ? extends T> function) {
-    if (list instanceof RandomAccess) {
+    if (list.isEmpty() && list instanceof ImmutableList) {
+      return ImmutableList.of(); // save ourselves some effort
+    } else if (list instanceof RandomAccess) {
       return new RandomAccessTransformingIndexedList<>(list, function);
     } else {
       return new TransformingIndexedList<>(list, function);
