@@ -15,6 +15,7 @@
  * limitations under the License.
  */
 package org.apache.calcite.plan;
+
 import org.apache.calcite.adapter.java.ReflectiveSchema;
 import org.apache.calcite.avatica.util.TimeUnit;
 import org.apache.calcite.plan.volcano.VolcanoPlanner;
@@ -44,6 +45,7 @@ import org.apache.calcite.rel.type.RelDataTypeFactory;
 import org.apache.calcite.rel.type.RelDataTypeField;
 import org.apache.calcite.rex.RexBuilder;
 import org.apache.calcite.rex.RexCorrelVariable;
+import org.apache.calcite.rex.RexDynamicParam;
 import org.apache.calcite.rex.RexFieldCollation;
 import org.apache.calcite.rex.RexInputRef;
 import org.apache.calcite.rex.RexLiteral;
@@ -113,6 +115,7 @@ import static java.util.Objects.requireNonNull;
 /**
  * Unit test for {@link org.apache.calcite.rel.externalize.RelJson}.
  */
+@SuppressWarnings("ConcatenationWithEmptyString")
 class RelWriterTest {
   public static final String XX = "{\n"
       + "  \"rels\": [\n"
@@ -520,11 +523,11 @@ class RelWriterTest {
         Frameworks.withPlanner((cluster, relOptSchema, rootSchema) -> {
           rootSchema.add("hr",
               new ReflectiveSchema(new HrSchema()));
+          final RelOptTable table =
+              requireNonNull(
+                  relOptSchema.getTableForMember(Arrays.asList("hr", "emps")));
           LogicalTableScan scan =
-              LogicalTableScan.create(cluster,
-                  relOptSchema.getTableForMember(
-                      Arrays.asList("hr", "emps")),
-                  ImmutableList.of());
+              LogicalTableScan.create(cluster, table, ImmutableList.of());
           final RexBuilder rexBuilder = cluster.getRexBuilder();
           LogicalFilter filter =
               LogicalFilter.create(scan,
@@ -567,11 +570,11 @@ class RelWriterTest {
         Frameworks.withPlanner((cluster, relOptSchema, rootSchema) -> {
           rootSchema.add("hr",
               new ReflectiveSchema(new HrSchema()));
+          final RelOptTable table =
+              requireNonNull(
+                  relOptSchema.getTableForMember(Arrays.asList("hr", "emps")));
           LogicalTableScan scan =
-              LogicalTableScan.create(cluster,
-                  relOptSchema.getTableForMember(
-                      Arrays.asList("hr", "emps")),
-                  ImmutableList.of());
+              LogicalTableScan.create(cluster, table, ImmutableList.of());
           final RexBuilder rexBuilder = cluster.getRexBuilder();
           final RelDataType bigIntType =
               cluster.getTypeFactory().createSqlType(SqlTypeName.BIGINT);
@@ -786,7 +789,8 @@ class RelWriterTest {
       throw TestUtil.rethrow(e);
     }
     final RelJson relJson = RelJson.create()
-        .withInputTranslator(RelWriterTest::translateInput);
+        .withInputTranslator(RelWriterTest::translateInput)
+        .withLibraryOperatorTable();
     final RexNode e = relJson.toRex(cluster, o);
     assertThat(e, hasToString(matcher));
   }
@@ -954,7 +958,7 @@ class RelWriterTest {
     rel.explain(jsonWriter);
     final String relJson = jsonWriter.asString();
     String s = deserializeAndDump(getSchema(rel), relJson, format);
-    String expected = null;
+    final String expected;
     switch (format) {
     case TEXT:
       expected = ""
@@ -973,6 +977,8 @@ class RelWriterTest {
           + "$5\\n\" [label=\"0\"]\n"
           + "}\n";
       break;
+    default:
+      throw new AssertionError();
     }
     assertThat(s, isLinux(expected));
   }
@@ -1052,6 +1058,69 @@ class RelWriterTest {
     assertThat(result, isLinux(expected));
   }
 
+  /** Test case for
+   * <a href="https://issues.apache.org/jira/browse/CALCITE-5607">[CALCITE-5607]
+   * Datetime MINUS throws ArrayIndexOutOfBounds error when serializing toRex</a>.
+   */
+  @Test void testDeserializeMinusDateOperator() {
+    final FrameworkConfig config = RelBuilderTest.config().build();
+    final RelBuilder builder = RelBuilder.create(config);
+    final RexBuilder rb = builder.getRexBuilder();
+    final RelDataTypeFactory typeFactory = rb.getTypeFactory();
+    final SqlIntervalQualifier qualifier =
+        new SqlIntervalQualifier(TimeUnit.MONTH, null, SqlParserPos.ZERO);
+    final RexNode op1 = rb.makeTimestampLiteral(new TimestampString("2012-12-03 12:34:44"), 0);
+    final RexNode op2 = rb.makeTimestampLiteral(new TimestampString("2014-12-23 12:34:44"), 0);
+    final RelDataType intervalType =
+        typeFactory.createTypeWithNullability(
+            typeFactory.createSqlIntervalType(qualifier),
+            op1.getType().isNullable() || op2.getType().isNullable());
+    final RelNode rel = builder
+        .scan("EMP")
+        .project(builder.field("JOB"),
+            rb.makeCall(intervalType, SqlStdOperatorTable.MINUS_DATE,
+                ImmutableList.of(op2, op1))).build();
+    final RelJsonWriter jsonWriter =
+        new RelJsonWriter(new JsonBuilder(), RelJson::withLibraryOperatorTable);
+    rel.explain(jsonWriter);
+    String relJson = jsonWriter.asString();
+    String result = deserializeAndDumpToTextFormat(getSchema(rel), relJson);
+    final String expected =
+        "LogicalProject(JOB=[$2], $f1=[-(2014-12-23 12:34:44, 2012-12-03 12:34:44)])\n"
+        + "  LogicalTableScan(table=[[scott, EMP]])\n";
+    assertThat(result, isLinux(expected));
+  }
+
+  /** Test case for
+   * <a href="https://issues.apache.org/jira/browse/CALCITE-6323">[CALCITE-6323]</a>
+   *
+   * <p>Before the fix, RelJson.toRex would throw an ArrayIndexOutOfBounds error
+   * when deserializing SAFE_CAST due to type inference requiring 2 operands.
+   *
+   * <p>The solution is to add in 'type' when serializing to JSON.
+   */
+  @Test void testDeserializeSafeCastOperator() {
+    final FrameworkConfig config = RelBuilderTest.config().build();
+    final RelBuilder builder = RelBuilder.create(config);
+    final RexBuilder rb = builder.getRexBuilder();
+    final RelDataTypeFactory typeFactory = rb.getTypeFactory();
+    final RelDataType type = typeFactory.createSqlType(SqlTypeName.DATE);
+    final RelNode rel = builder
+        .scan("EMP")
+        .project(builder.field("JOB"),
+            rb.makeCall(type, SqlLibraryOperators.SAFE_CAST,
+                ImmutableList.of(builder.field("SAL")))).build();
+    final RelJsonWriter jsonWriter =
+        new RelJsonWriter(new JsonBuilder(), RelJson::withLibraryOperatorTable);
+    rel.explain(jsonWriter);
+    String relJson = jsonWriter.asString();
+    String result = deserializeAndDumpToTextFormat(getSchema(rel), relJson);
+    final String expected =
+        "LogicalProject(JOB=[$2], $f1=[SAFE_CAST($5)])\n"
+        + "  LogicalTableScan(table=[[scott, EMP]])\n";
+    assertThat(result, isLinux(expected));
+  }
+
   @Test void testAggregateWithoutAlias() {
     final FrameworkConfig config = RelBuilderTest.config().build();
     final RelBuilder builder = RelBuilder.create(config);
@@ -1151,8 +1220,7 @@ class RelWriterTest {
     final Function<RelBuilder, RelNode> relFn = b ->
         mockCountOver(b, "EMP", ImmutableList.of(), ImmutableList.of("DEPTNO"));
     final String expected = ""
-        + "LogicalProject($f0=[COUNT() OVER (ORDER BY $7 NULLS LAST "
-        + "ROWS UNBOUNDED PRECEDING)])\n"
+        + "LogicalProject($f0=[COUNT() OVER (ORDER BY $7 NULLS LAST)])\n"
         + "  LogicalTableScan(table=[[scott, EMP]])\n";
     relFn(relFn)
         .assertThatPlan(isLinux(expected));
@@ -1290,7 +1358,7 @@ class RelWriterTest {
             return super.visit(scan);
           }
         });
-    return schemaHolder.get();
+    return requireNonNull(schemaHolder.get());
   }
 
   /**
@@ -1362,7 +1430,7 @@ class RelWriterTest {
     for (String orderKeyName : orderKeyNames) {
       orderKeys.add(new RexFieldCollation(builder.field(orderKeyName), ImmutableSet.of()));
     }
-    final RelNode rel = builder
+    return builder
         .project(
             rexBuilder.makeOver(
                 type,
@@ -1372,9 +1440,8 @@ class RelWriterTest {
                 ImmutableList.copyOf(orderKeys),
                 RexWindowBounds.UNBOUNDED_PRECEDING,
                 RexWindowBounds.CURRENT_ROW,
-                true, true, false, false, false))
+                false, true, false, false, false))
         .build();
-    return rel;
   }
 
   @Test void testHashDistributionWithoutKeys() {
@@ -1388,6 +1455,53 @@ class RelWriterTest {
         .assertThatPlan(isLinux(expected));
   }
 
+  /** Test case for
+   * <a href="https://issues.apache.org/jira/browse/CALCITE-6200">[CALCITE-6200]
+   * RelJson throw UnsupportedOperationException for RexDynamicParam</a>. */
+  @Test void testDynamicParam() {
+    final Function<RelBuilder, RelNode> relFn = relBuilder -> {
+      final RexBuilder rexBuilder = relBuilder.getRexBuilder();
+      final RelDataTypeFactory typeFactory = relBuilder.getTypeFactory();
+      final RelDataType intType = typeFactory.createSqlType(SqlTypeName.INTEGER);
+      final RexDynamicParam dynamicParam = rexBuilder.makeDynamicParam(intType, 0);
+      return relBuilder
+          .scan("EMP")
+          .sortLimit(null, dynamicParam, relBuilder.fields(RelCollations.EMPTY))
+          .build();
+    };
+
+    final String expectedJson = "{\n"
+        + "  \"rels\": [\n"
+        + "    {\n"
+        + "      \"id\": \"0\",\n"
+        + "      \"relOp\": \"LogicalTableScan\",\n"
+        + "      \"table\": [\n"
+        + "        \"scott\",\n"
+        + "        \"EMP\"\n"
+        + "      ],\n"
+        + "      \"inputs\": []\n"
+        + "    },\n"
+        + "    {\n"
+        + "      \"id\": \"1\",\n"
+        + "      \"relOp\": \"LogicalSort\",\n"
+        + "      \"collation\": [],\n"
+        + "      \"fetch\": {\n"
+        + "        \"dynamicParam\": 0,\n"
+        + "        \"type\": {\n"
+        + "          \"type\": \"INTEGER\",\n"
+        + "          \"nullable\": false\n"
+        + "        }\n"
+        + "      }\n"
+        + "    }\n"
+        + "  ]\n"
+        + "}";
+    final String expectedPlan = "LogicalSort(fetch=[?0])\n"
+        + "  LogicalTableScan(table=[[scott, EMP]])\n";
+    relFn(relFn)
+        .assertThatJson(is(expectedJson))
+        .assertThatPlan(isLinux(expectedPlan));
+  }
+
   @Test void testWriteSortExchangeWithHashDistribution() {
     final Function<RelBuilder, RelNode> relFn = b ->
         createSortPlan(b, RelDistributions.hash(Lists.newArrayList(0)));
@@ -1415,10 +1529,14 @@ class RelWriterTest {
         .project(b.fields(), ImmutableList.of(), true)
             .let(b2 -> {
               final RelNode input = b2.build();
-              final RelOptTable table = input.getInput(0).getTable();
+              final RelOptTable table =
+                  requireNonNull(input.getInput(0).getTable());
+              final Prepare.CatalogReader schema =
+                  (Prepare.CatalogReader)
+                      requireNonNull(table.getRelOptSchema());
               final LogicalTableModify modify =
                   LogicalTableModify.create(table,
-                      (Prepare.CatalogReader) table.getRelOptSchema(),
+                      schema,
                       input,
                       TableModify.Operation.INSERT,
                       null,
@@ -1443,10 +1561,14 @@ class RelWriterTest {
                 b.equals(b.field("JOB"), b.literal("c")))
             .let(b2 -> {
               final RelNode filter = b2.build();
-              final RelOptTable table = filter.getInput(0).getTable();
+              final RelOptTable table =
+                  requireNonNull(filter.getInput(0).getTable());
+              final Prepare.CatalogReader schema =
+                  (Prepare.CatalogReader)
+                      requireNonNull(table.getRelOptSchema());
               final LogicalTableModify modify =
                   LogicalTableModify.create(table,
-                      (Prepare.CatalogReader) table.getRelOptSchema(),
+                      schema,
                       filter,
                       TableModify.Operation.UPDATE,
                       ImmutableList.of("ENAME"),
@@ -1470,10 +1592,14 @@ class RelWriterTest {
             .filter(b.equals(b.field("JOB"), b.literal("c")))
             .let(b2 -> {
               final RelNode filter = b2.build();
-              final RelOptTable table = filter.getInput(0).getTable();
+              final RelOptTable table =
+                  requireNonNull(filter.getInput(0).getTable());
+              final Prepare.CatalogReader schema =
+                  (Prepare.CatalogReader)
+                      requireNonNull(table.getRelOptSchema());
               LogicalTableModify modify =
                   LogicalTableModify.create(table,
-                      (Prepare.CatalogReader) table.getRelOptSchema(),
+                      schema,
                       filter,
                       TableModify.Operation.DELETE,
                       null,
@@ -1534,9 +1660,12 @@ class RelWriterTest {
               //     INSERT VALUES (0, 'x', 'x', 0, '20200501 10:00:00',
               //         0, 0, 0, 0)
               final RelNode project = b.build();
+              final Prepare.CatalogReader schema =
+                  (Prepare.CatalogReader)
+                      requireNonNull(emp.get().getRelOptSchema());
               LogicalTableModify modify =
                   LogicalTableModify.create(emp.get(),
-                      (Prepare.CatalogReader) emp.get().getRelOptSchema(),
+                      schema,
                       project,
                       TableModify.Operation.MERGE,
                       ImmutableList.of("ENAME"),
@@ -1603,6 +1732,7 @@ class RelWriterTest {
       return this;
     }
 
+    @SuppressWarnings("UnusedReturnValue")
     Fixture assertThatPlan(Matcher<String> matcher) {
       final FrameworkConfig config = RelBuilderTest.config().build();
       final RelBuilder b = RelBuilder.create(config);
