@@ -34,11 +34,13 @@ import org.apache.calcite.sql.fun.SqlStdOperatorTable;
 import org.apache.calcite.sql.type.BasicSqlType;
 import org.apache.calcite.sql.type.SqlTypeFactoryImpl;
 import org.apache.calcite.sql.type.SqlTypeName;
+import org.apache.calcite.test.CustomTypeSystems;
 import org.apache.calcite.test.RexImplicationCheckerFixtures;
 import org.apache.calcite.util.DateString;
 import org.apache.calcite.util.Litmus;
 import org.apache.calcite.util.NlsString;
 import org.apache.calcite.util.TimeString;
+import org.apache.calcite.util.TimeWithTimeZoneString;
 import org.apache.calcite.util.TimestampString;
 import org.apache.calcite.util.TimestampWithTimeZoneString;
 import org.apache.calcite.util.Util;
@@ -52,6 +54,7 @@ import org.junit.jupiter.params.provider.Arguments;
 import org.junit.jupiter.params.provider.MethodSource;
 
 import java.math.BigDecimal;
+import java.math.RoundingMode;
 import java.nio.charset.StandardCharsets;
 import java.util.Arrays;
 import java.util.Calendar;
@@ -62,12 +65,13 @@ import java.util.stream.Stream;
 
 import static org.hamcrest.CoreMatchers.containsString;
 import static org.hamcrest.CoreMatchers.equalTo;
+import static org.hamcrest.CoreMatchers.instanceOf;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.CoreMatchers.notNullValue;
 import static org.hamcrest.CoreMatchers.nullValue;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.hasToString;
-import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
@@ -117,7 +121,7 @@ class RexBuilderTest {
         builder.ensureType(typeFactory.createSqlType(SqlTypeName.ANY), node,
             true);
 
-    assertEquals(node, ensuredNode);
+    assertThat(ensuredNode, is(node));
   }
 
   /**
@@ -134,7 +138,7 @@ class RexBuilderTest {
         builder.ensureType(typeFactory.createSqlType(SqlTypeName.BOOLEAN), node,
             true);
 
-    assertEquals(node, ensuredNode);
+    assertThat(ensuredNode, is(node));
   }
 
   /**
@@ -152,7 +156,8 @@ class RexBuilderTest {
             true);
 
     assertNotEquals(node, ensuredNode);
-    assertEquals(ensuredNode.getType(), typeFactory.createSqlType(SqlTypeName.INTEGER));
+    assertThat(typeFactory.createSqlType(SqlTypeName.INTEGER),
+        is(ensuredNode.getType()));
   }
 
   private static final long MOON = -14159025000L;
@@ -212,6 +217,58 @@ class RexBuilderTest {
         hasToString("1969-07-21 02:56:15.102"));
   }
 
+  /** Test cases for
+   * <a href="https://issues.apache.org/jira/browse/CALCITE-6389">[CALCITE-6389]
+   * RexBuilder.removeCastFromLiteral does not preserve semantics for some types of literal</a>. */
+  @Test void testRemoveCast() {
+    final RelDataTypeFactory typeFactory = new SqlTypeFactoryImpl(RelDataTypeSystem.DEFAULT);
+    RexBuilder builder = new RexBuilder(typeFactory);
+
+    // Can remove cast of an integer to an integer
+    BigDecimal value = new BigDecimal(10);
+    RelDataType toType = builder.typeFactory.createSqlType(SqlTypeName.INTEGER);
+    assertTrue(builder.canRemoveCastFromLiteral(toType, value, SqlTypeName.INTEGER));
+
+    // Can remove cast from integer to decimal
+    toType = builder.typeFactory.createSqlType(SqlTypeName.DECIMAL);
+    assertTrue(builder.canRemoveCastFromLiteral(toType, value, SqlTypeName.INTEGER));
+
+    // 250 is too large for a TINYINT
+    value = new BigDecimal(250);
+    toType = builder.typeFactory.createSqlType(SqlTypeName.TINYINT);
+    assertFalse(builder.canRemoveCastFromLiteral(toType, value, SqlTypeName.INTEGER));
+
+    // 50 isn't too large for a TINYINT
+    value = new BigDecimal(50);
+    toType = builder.typeFactory.createSqlType(SqlTypeName.TINYINT);
+    assertTrue(builder.canRemoveCastFromLiteral(toType, value, SqlTypeName.INTEGER));
+
+    // 120.25 cannot be represented with precision 2 and scale 2 without loss
+    value = new BigDecimal("120.25");
+    toType = builder.typeFactory.createSqlType(SqlTypeName.DECIMAL, 2, 2);
+    assertFalse(builder.canRemoveCastFromLiteral(toType, value, SqlTypeName.DECIMAL));
+
+    // 120.25 cannot be represented with precision 5 and scale 1 without rounding
+    value = new BigDecimal("120.25");
+    toType = builder.typeFactory.createSqlType(SqlTypeName.DECIMAL, 5, 1);
+    assertFalse(builder.canRemoveCastFromLiteral(toType, value, SqlTypeName.DECIMAL));
+
+    // longmax + 1 cannot be represented as a long
+    value = new BigDecimal(Long.MAX_VALUE).add(BigDecimal.ONE);
+    toType = builder.typeFactory.createSqlType(SqlTypeName.BIGINT);
+    assertFalse(builder.canRemoveCastFromLiteral(toType, value, SqlTypeName.DECIMAL));
+
+    // Cast to decimal of an INTERVAL '5' seconds cannot be removed
+    value = new BigDecimal("5");
+    toType = builder.typeFactory.createSqlType(SqlTypeName.DECIMAL, 5, 1);
+    assertFalse(builder.canRemoveCastFromLiteral(toType, value, SqlTypeName.INTERVAL_SECOND));
+
+    // Cast to decimal of an INTERVAL '5' minutes cannot be removed
+    value = new BigDecimal("5");
+    toType = builder.typeFactory.createSqlType(SqlTypeName.DECIMAL, 5, 1);
+    assertFalse(builder.canRemoveCastFromLiteral(toType, value, SqlTypeName.INTERVAL_MINUTE));
+  }
+
   @Test void testTimestampString() {
     final TimestampString ts = new TimestampString(1969, 7, 21, 2, 56, 15);
     assertThat(ts, hasToString("1969-07-21 02:56:15"));
@@ -438,6 +495,77 @@ class RexBuilderTest {
     assertThat(literal.getValue3() instanceof Long, is(true));
   }
 
+  /** Tests
+   * {@link RexBuilder#makeTimestampTzLiteral(TimestampWithTimeZoneString, int)}. */
+  @Test void testTimestampTzLiterals() {
+    final RelDataTypeFactory typeFactory =
+        new SqlTypeFactoryImpl(RelDataTypeSystem.DEFAULT);
+    final RelDataType timestampType =
+        typeFactory.createSqlType(SqlTypeName.TIMESTAMP_TZ);
+    final RelDataType timestampType3 =
+        typeFactory.createSqlType(SqlTypeName.TIMESTAMP_TZ, 3);
+    final RelDataType timestampType9 =
+        typeFactory.createSqlType(SqlTypeName.TIMESTAMP_TZ, 9);
+    final RelDataType timestampType18 =
+        typeFactory.createSqlType(SqlTypeName.TIMESTAMP_TZ, 18);
+    final RexBuilder builder = new RexBuilder(typeFactory);
+
+    // The new way
+    final TimestampWithTimeZoneString ts =
+        new TimestampWithTimeZoneString(1969, 7, 21, 2, 56, 15,
+            TimeZone.getTimeZone("PST").getID());
+    checkTimestampTz(builder.makeLiteral(ts, timestampType));
+
+    // Now with milliseconds
+    final TimestampWithTimeZoneString ts2 = ts.withMillis(56);
+    assertThat(ts2, hasToString("1969-07-21 02:56:15.056 PST"));
+    final RexLiteral literal2 =
+        builder.makeLiteral(ts2, timestampType3);
+    assertThat(literal2.getValue(), hasToString("1969-07-21 02:56:15.056 PST"));
+
+    // Now with nanoseconds
+    final TimestampWithTimeZoneString ts3 = ts.withNanos(56);
+    final RexLiteral literal3 =
+        builder.makeLiteral(ts3, timestampType9);
+    assertThat(literal3.getValueAs(TimestampWithTimeZoneString.class),
+        hasToString("1969-07-21 02:56:15 PST"));
+    final TimestampWithTimeZoneString ts3b = ts.withNanos(2345678);
+    final RexLiteral literal3b =
+        builder.makeLiteral(ts3b, timestampType9);
+    assertThat(literal3b.getValueAs(TimestampWithTimeZoneString.class),
+        hasToString("1969-07-21 02:56:15.002 PST"));
+
+    // Now with a very long fraction
+    final TimestampWithTimeZoneString ts4 = ts.withFraction("102030405060708090102");
+    final RexLiteral literal4 =
+        builder.makeLiteral(ts4, timestampType18);
+    assertThat(literal4.getValueAs(TimestampWithTimeZoneString.class),
+        hasToString("1969-07-21 02:56:15.102 PST"));
+
+    // toString
+    assertThat(ts2.round(1), hasToString("1969-07-21 02:56:15 PST"));
+    assertThat(ts2.round(2), hasToString("1969-07-21 02:56:15.05 PST"));
+    assertThat(ts2.round(3), hasToString("1969-07-21 02:56:15.056 PST"));
+    assertThat(ts2.round(4), hasToString("1969-07-21 02:56:15.056 PST"));
+
+    assertThat(ts2.toString(6), is("1969-07-21 02:56:15.056000 PST"));
+    assertThat(ts2.toString(1), is("1969-07-21 02:56:15.0 PST"));
+    assertThat(ts2.toString(0), is("1969-07-21 02:56:15 PST"));
+
+    assertThat(ts2.round(0), hasToString("1969-07-21 02:56:15 PST"));
+    assertThat(ts2.round(0).toString(0), is("1969-07-21 02:56:15 PST"));
+    assertThat(ts2.round(0).toString(1), is("1969-07-21 02:56:15.0 PST"));
+    assertThat(ts2.round(0).toString(2), is("1969-07-21 02:56:15.00 PST"));
+  }
+
+  private void checkTimestampTz(RexLiteral literal) {
+    assertThat(literal,
+        hasToString("1969-07-21 02:56:15 PST:TIMESTAMP_TZ(0)"));
+    assertThat(literal.getValue() instanceof TimestampWithTimeZoneString, is(true));
+    assertThat(literal.getValue2() instanceof Long, is(true));
+    assertThat(literal.getValue3() instanceof Long, is(true));
+  }
+
   /** Tests {@link RexBuilder#makeTimeLiteral(TimeString, int)}. */
   @Test void testTimeLiteral() {
     final RelDataTypeFactory typeFactory =
@@ -578,6 +706,65 @@ class RexBuilderTest {
     }
   }
 
+  /** Tests {@link RexBuilder#makeExactLiteral(BigDecimal, RelDataType)}. */
+  @Test void testDecimalWithRoundingMode() {
+    final RelDataTypeFactory typeFactory =
+        new SqlTypeFactoryImpl(RelDataTypeSystem.DEFAULT);
+    final RelDataType type = typeFactory.createSqlType(SqlTypeName.DECIMAL, 4, 2);
+    final RexBuilder builder = new RexBuilder(typeFactory);
+    RexLiteral rexLiteral = builder.makeExactLiteral(new BigDecimal("13.556"), type);
+    assertThat(rexLiteral.getValue() instanceof BigDecimal, is(true));
+    assertThat(rexLiteral.getValue(), hasToString("13.55"));
+    final RelDataTypeFactory typeFactoryHalfUp =
+        new SqlTypeFactoryImpl(new RelDataTypeSystemImpl() {
+          @Override public RoundingMode roundingMode() {
+            return RoundingMode.HALF_UP;
+          }
+        });
+    final RelDataType typeHalfUp =
+        typeFactoryHalfUp.createSqlType(SqlTypeName.DECIMAL, 4, 2);
+    final RexBuilder builderHalfUp = new RexBuilder(typeFactoryHalfUp);
+    RexLiteral rexLiteralHalfUp =
+        builderHalfUp.makeExactLiteral(new BigDecimal("13.556"), typeHalfUp);
+    assertThat(rexLiteralHalfUp.getValue() instanceof BigDecimal, is(true));
+    assertThat(rexLiteralHalfUp.getValue(), hasToString("13.56"));
+  }
+
+  @Test void testDecimalWithNegativeScaleRoundingHalfUp() {
+    final RelDataTypeFactory typeFactory =
+        new SqlTypeFactoryImpl(new RelDataTypeSystemImpl() {
+          @Override public int getMinScale(SqlTypeName typeName) {
+            switch (typeName) {
+            case DECIMAL:
+              return -2;
+            default:
+              return super.getMinScale(typeName);
+            }
+          }
+
+          @Override public RoundingMode roundingMode() {
+            return RoundingMode.HALF_UP;
+          }
+        });
+    final RelDataType type = typeFactory.createSqlType(SqlTypeName.DECIMAL, 3, -2);
+    final RexBuilder builder = new RexBuilder(typeFactory);
+    RexLiteral rexLiteral = builder.makeLiteral(new BigDecimal("12355"), type);
+    assertThat(rexLiteral.getValue() instanceof BigDecimal, is(true));
+    assertThat(rexLiteral.getValue(), hasToString("12400"));
+  }
+
+  @Test void testDecimalWithNegativeScaleRoundingDown() {
+    final RelDataTypeFactory typeFactory =
+        new SqlTypeFactoryImpl(
+            CustomTypeSystems.withMinScale(RelDataTypeSystem.DEFAULT,
+                typeName -> -2));
+    final RelDataType type = typeFactory.createSqlType(SqlTypeName.DECIMAL, 3, -2);
+    final RexBuilder builder = new RexBuilder(typeFactory);
+    RexLiteral rexLiteralHalfUp = builder.makeLiteral(new BigDecimal("12355"), type);
+    assertThat(rexLiteralHalfUp.getValue() instanceof BigDecimal, is(true));
+    assertThat(rexLiteralHalfUp.getValue(), hasToString("12300"));
+  }
+
   /** Tests {@link DateString} year range. */
   @Test void testDateStringYearError() {
     try {
@@ -705,25 +892,25 @@ class RexBuilderTest {
     final NlsString utf8 = new NlsString("foobar", "UTF8", SqlCollation.IMPLICIT);
 
     RexLiteral literal = builder.makePreciseStringLiteral("foobar");
-    assertEquals("'foobar'", literal.toString());
+    assertThat(literal, hasToString("'foobar'"));
     literal =
         builder.makePreciseStringLiteral(
             new ByteString(new byte[] { 'f', 'o', 'o', 'b', 'a', 'r'}),
             "UTF8", SqlCollation.IMPLICIT);
-    assertEquals("_UTF8'foobar'", literal.toString());
-    assertEquals("_UTF8'foobar':CHAR(6) CHARACTER SET \"UTF-8\"",
-        literal.computeDigest(RexDigestIncludeType.ALWAYS));
+    assertThat(literal, hasToString("_UTF8'foobar'"));
+    assertThat(literal.computeDigest(RexDigestIncludeType.ALWAYS),
+        is("_UTF8'foobar':CHAR(6) CHARACTER SET \"UTF-8\""));
     literal =
         builder.makePreciseStringLiteral(
             new ByteString("\u82f1\u56fd".getBytes(StandardCharsets.UTF_8)),
             "UTF8", SqlCollation.IMPLICIT);
-    assertEquals("_UTF8'\u82f1\u56fd'", literal.toString());
+    assertThat(literal, hasToString("_UTF8'\u82f1\u56fd'"));
     // Test again to check decode cache.
     literal =
         builder.makePreciseStringLiteral(
             new ByteString("\u82f1".getBytes(StandardCharsets.UTF_8)),
             "UTF8", SqlCollation.IMPLICIT);
-    assertEquals("_UTF8'\u82f1'", literal.toString());
+    assertThat(literal, hasToString("_UTF8'\u82f1'"));
     try {
       literal =
           builder.makePreciseStringLiteral(
@@ -734,9 +921,9 @@ class RexBuilderTest {
       assertThat(e.getMessage(), containsString("Failed to encode"));
     }
     literal = builder.makeLiteral(latin1, varchar);
-    assertEquals("_LATIN1'foobar'", literal.toString());
+    assertThat(literal, hasToString("_LATIN1'foobar'"));
     literal = builder.makeLiteral(utf8, varchar);
-    assertEquals("_UTF8'foobar'", literal.toString());
+    assertThat(literal, hasToString("_UTF8'foobar'"));
   }
 
   /** Tests {@link RexBuilder#makeExactLiteral(java.math.BigDecimal)}. */
@@ -762,7 +949,7 @@ class RexBuilderTest {
     final RelDataTypeFactory typeFactory =
             new SqlTypeFactoryImpl(RelDataTypeSystem.DEFAULT);
     final RexBuilder rexBuilder = new RexBuilder(typeFactory);
-    final RelDataType floatType = typeFactory.createSqlType(SqlTypeName.FLOAT);
+    final RelDataType floatType = typeFactory.createSqlType(SqlTypeName.REAL);
     RexNode left = rexBuilder.makeInputRef(floatType, 0);
     final RexNode literal1 = rexBuilder.makeLiteral(1.0f, floatType);
     final RexNode literal2 = rexBuilder.makeLiteral(2.0f, floatType);
@@ -827,7 +1014,11 @@ class RexBuilderTest {
         type2rexLiteral.apply(typeFactory.createSqlType(SqlTypeName.TIME_WITH_LOCAL_TIME_ZONE),
             relDataType -> new TimeString(0, 0, 0)),
         type2rexLiteral.apply(typeFactory.createSqlType(SqlTypeName.TIMESTAMP_WITH_LOCAL_TIME_ZONE),
-            relDataType -> new TimestampString(0, 1, 1, 0, 0, 0)));
+            relDataType -> new TimestampString(0, 1, 1, 0, 0, 0)),
+        type2rexLiteral.apply(typeFactory.createSqlType(SqlTypeName.TIME_TZ),
+            relDataType -> new TimeWithTimeZoneString(0, 0, 0, "GMT+00:00")),
+        type2rexLiteral.apply(typeFactory.createSqlType(SqlTypeName.TIMESTAMP_TZ),
+            relDataType -> new TimestampWithTimeZoneString(0, 1, 1, 0, 0, 0, "GMT+00:00")));
   }
 
   /** Test case for
@@ -846,7 +1037,7 @@ class RexBuilderTest {
 
     final RexNode sarg = ((RexCall) inCall).operands.get(1);
     RelDataType expected = typeFactory.createSqlType(SqlTypeName.DECIMAL, 6, 1);
-    assertEquals(sarg.getType(), expected);
+    assertThat(expected, is(sarg.getType()));
   }
 
   /** Test case for
@@ -865,7 +1056,7 @@ class RexBuilderTest {
 
     final RexNode sarg = ((RexCall) betweenCall).operands.get(1);
     RelDataType expected = typeFactory.createSqlType(SqlTypeName.DECIMAL, 6, 1);
-    assertEquals(sarg.getType(), expected);
+    assertThat(expected, is(sarg.getType()));
   }
 
   /** Tests {@link RexCopier#visitOver(RexOver)}. */
@@ -889,7 +1080,7 @@ class RexBuilderTest {
             RexWindowBounds.CURRENT_ROW,
             true, true, false, false, false);
     final RexNode copy = builder.copy(node);
-    assertTrue(copy instanceof RexOver);
+    assertThat(copy, instanceOf(RexOver.class));
 
     RexOver result = (RexOver) copy;
     assertThat(result.getType().getSqlTypeName(), is(SqlTypeName.VARCHAR));
@@ -897,8 +1088,8 @@ class RexBuilderTest {
     assertThat(result.getWindow(), is(node.getWindow()));
     assertThat(result.getAggOperator(), is(node.getAggOperator()));
     assertThat(result.getAggOperator(), is(node.getAggOperator()));
-    assertEquals(node.isDistinct(), result.isDistinct());
-    assertEquals(node.ignoreNulls(), result.ignoreNulls());
+    assertThat(result.isDistinct(), is(node.isDistinct()));
+    assertThat(result.ignoreNulls(), is(node.ignoreNulls()));
     for (int i = 0; i < node.getOperands().size(); i++) {
       assertThat(result.getOperands().get(i).getType().getSqlTypeName(),
           is(node.getOperands().get(i).getType().getSqlTypeName()));
@@ -920,7 +1111,7 @@ class RexBuilderTest {
     final RexCorrelVariable node =
         (RexCorrelVariable) builder.makeCorrel(type, new CorrelationId(0));
     final RexNode copy = builder.copy(node);
-    assertTrue(copy instanceof RexCorrelVariable);
+    assertThat(copy, instanceOf(RexCorrelVariable.class));
 
     final RexCorrelVariable result = (RexCorrelVariable) copy;
     assertThat(result.id, is(node.id));
@@ -940,7 +1131,7 @@ class RexBuilderTest {
 
     final RexLocalRef node = new RexLocalRef(0, type);
     final RexNode copy = builder.copy(node);
-    assertTrue(copy instanceof RexLocalRef);
+    assertThat(copy, instanceOf(RexLocalRef.class));
 
     final RexLocalRef result = (RexLocalRef) copy;
     assertThat(result.getIndex(), is(node.getIndex()));
@@ -960,7 +1151,7 @@ class RexBuilderTest {
 
     final RexDynamicParam node = builder.makeDynamicParam(type, 0);
     final RexNode copy = builder.copy(node);
-    assertTrue(copy instanceof RexDynamicParam);
+    assertThat(copy, instanceOf(RexDynamicParam.class));
 
     final RexDynamicParam result = (RexDynamicParam) copy;
     assertThat(result.getIndex(), is(node.getIndex()));
@@ -980,7 +1171,7 @@ class RexBuilderTest {
 
     final RexRangeRef node = builder.makeRangeReference(type, 1, true);
     final RexNode copy = builder.copy(node);
-    assertTrue(copy instanceof RexRangeRef);
+    assertThat(copy, instanceOf(RexRangeRef.class));
 
     final RexRangeRef result = (RexRangeRef) copy;
     assertThat(result.getOffset(), is(node.getOffset()));
