@@ -24,6 +24,7 @@ import org.apache.calcite.plan.RelOptPredicateList;
 import org.apache.calcite.rel.type.RelDataType;
 import org.apache.calcite.rel.type.RelDataTypeFactory;
 import org.apache.calcite.rel.type.RelDataTypeSystem;
+import org.apache.calcite.sql.fun.SqlInternalOperators;
 import org.apache.calcite.sql.fun.SqlStdOperatorTable;
 import org.apache.calcite.sql.type.SqlTypeName;
 
@@ -326,6 +327,14 @@ public abstract class RexProgramBuilderBase {
     return rexBuilder.makeCall(SqlStdOperatorTable.PLUS, n1, n2);
   }
 
+  protected RexNode m2v(RexNode n) {
+    return rexBuilder.makeCall(SqlInternalOperators.M2V, n);
+  }
+
+  protected RexNode v2m(RexNode n) {
+    return rexBuilder.makeCall(SqlInternalOperators.V2M, n);
+  }
+
   protected RexNode item(RexNode inputRef, RexNode literal) {
     return rexBuilder.makeCall(SqlStdOperatorTable.ITEM, inputRef, literal);
   }
