@@ -46,8 +46,9 @@ import java.util.function.Consumer;
 import java.util.function.UnaryOperator;
 
 import static org.hamcrest.CoreMatchers.equalTo;
+import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
-import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.hamcrest.Matchers.hasToString;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.fail;
 
@@ -366,6 +367,7 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
           "KEYWORD(EXTEND)",
           "KEYWORD(/*+)",
           "KEYWORD(AS)",
+          "KEYWORD(ASOF)",
           "KEYWORD(USING)",
           "KEYWORD(OUTER)",
           "KEYWORD(RIGHT)",
@@ -378,6 +380,7 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
           "KEYWORD(HAVING)",
           "KEYWORD(LEFT)",
           "KEYWORD(EXCEPT)",
+          "KEYWORD(MATCH_CONDITION)",
           "KEYWORD(MATCH_RECOGNIZE)",
           "KEYWORD(MINUS)",
           "KEYWORD(JOIN)",
@@ -1546,7 +1549,7 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
         }
         buf.append(token).append("\n");
       }
-      assertEquals(expected, buf.toString());
+      assertThat(buf, hasToString(expected));
     }
 
     protected void assertHint(List<String>... expectedLists) {
@@ -1568,8 +1571,7 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
           advisor.getCompletionHints(
               sap.sql,
               requireNonNull(sap.pos, "sap.pos"));
-      assertEquals(
-          expectedResults, convertCompletionHints(results));
+      assertThat(convertCompletionHints(results), is(expectedResults));
     }
 
     /**
@@ -1581,7 +1583,7 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
       SqlAdvisor advisor = factory.createAdvisor();
 
       String actual = advisor.simplifySql(sap.sql, sap.cursor);
-      assertEquals(expected, actual);
+      assertThat(actual, is(expected));
       return this;
     }
 
@@ -1618,11 +1620,10 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
       final String[] replaced = {null};
       List<SqlMoniker> results =
           advisor.getCompletionHints(sap.sql, sap.cursor, replaced);
-      assertEquals(expectedResults, convertCompletionHints(results),
-          () -> "Completion hints for " + sap);
+      assertThat("Completion hints for " + sap, convertCompletionHints(results),
+          is(expectedResults));
       if (expectedWord != null) {
-        assertEquals(expectedWord, replaced[0],
-            "replaced[0] for " + sap);
+        assertThat("replaced[0] for " + sap, replaced[0], is(expectedWord));
       } else {
         assertNotNull(replaced[0]);
       }
@@ -1643,8 +1644,8 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
         }
         missingReplacemenets.remove(id);
         String actualReplacement = advisor.getReplacement(result, word);
-        assertEquals(expectedReplacement, actualReplacement,
-            () -> sap + ", replacement of " + word + " with " + id);
+        assertThat(sap + ", replacement of " + word + " with " + id,
+            actualReplacement, is(expectedReplacement));
       }
       if (missingReplacemenets.isEmpty()) {
         return;
