@@ -25,6 +25,8 @@ import org.apache.calcite.plan.RelOptUtil;
 import org.apache.calcite.rel.rules.materialize.MaterializedViewRules;
 import org.apache.calcite.runtime.Hook;
 import org.apache.calcite.schema.SchemaPlus;
+import org.apache.calcite.sql.SqlDialect;
+import org.apache.calcite.sql.SqlDialect.DatabaseProduct;
 import org.apache.calcite.test.schemata.foodmart.FoodmartSchema;
 import org.apache.calcite.util.ImmutableBitSet;
 import org.apache.calcite.util.TestUtil;
@@ -49,17 +51,19 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.Consumer;
 
 import static org.apache.calcite.test.Matchers.containsStringLinux;
-import static org.apache.calcite.test.Matchers.within;
 
 import static org.hamcrest.CoreMatchers.anyOf;
 import static org.hamcrest.CoreMatchers.containsString;
 import static org.hamcrest.CoreMatchers.equalTo;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
+import static org.hamcrest.Matchers.closeTo;
 import static org.hamcrest.Matchers.hasSize;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
 import static org.junit.jupiter.api.Assumptions.assumeTrue;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * Unit test for lattices.
  */
@@ -201,10 +205,13 @@ class LatticeTest {
     modelWithLattice("EMPLOYEES", "select * from \"foodmart\".\"days\"")
         .doWithConnection(c -> {
           final SchemaPlus schema = c.getRootSchema();
-          final SchemaPlus adhoc = schema.getSubSchema("adhoc");
+          final SchemaPlus adhoc =
+              requireNonNull(schema.getSubSchema("adhoc"));
           assertThat(adhoc.getTableNames().contains("EMPLOYEES"), is(true));
+          final CalciteSchema adhocSchema =
+              requireNonNull(adhoc.unwrap(CalciteSchema.class));
           final Map.Entry<String, CalciteSchema.LatticeEntry> entry =
-              adhoc.unwrap(CalciteSchema.class).getLatticeMap().firstEntry();
+              adhocSchema.getLatticeMap().firstEntry();
           final Lattice lattice = entry.getValue().getLattice();
           final String sql = "SELECT \"days\".\"day\"\n"
               + "FROM \"foodmart\".\"days\" AS \"days\"\n"
@@ -222,6 +229,37 @@ class LatticeTest {
         });
   }
 
+  /** Test case for
+   * <a href="https://issues.apache.org/jira/browse/CALCITE-6603">[CALCITE-6603]
+   * Lattice SQL supports generation of specified dialects</a>. */
+  @Test void testLatticeSqlWithDialect() throws Exception {
+    final SqlDialect dialect = DatabaseProduct.SPARK.getDialect();
+    modelWithLattice("EMPLOYEES", "select * from \"foodmart\".\"days\"")
+        .doWithConnection(c -> {
+          final SchemaPlus schema = c.getRootSchema();
+          final SchemaPlus adhoc =
+              requireNonNull(schema.getSubSchema("adhoc"));
+          assertThat(adhoc.getTableNames().contains("EMPLOYEES"), is(true));
+          final CalciteSchema adhocSchema =
+              requireNonNull(adhoc.unwrap(CalciteSchema.class));
+          final Map.Entry<String, CalciteSchema.LatticeEntry> entry =
+              adhocSchema.getLatticeMap().firstEntry();
+          final Lattice lattice = entry.getValue().getLattice();
+          final String sql = "SELECT `days`.`day`\n"
+              + "FROM `foodmart`.`days` AS `days`\n"
+              + "GROUP BY `days`.`day`";
+          assertThat(
+              lattice.sql(ImmutableBitSet.of(0), true,
+                  ImmutableList.of(), dialect), is(sql));
+          final String sql2 = "SELECT `days`.`day`, `days`.`week_day`\n"
+              + "FROM `foodmart`.`days` AS `days`";
+          assertThat(
+              lattice.sql(ImmutableBitSet.of(0, 1), false,
+                  ImmutableList.of(), dialect),
+              is(sql2));
+        });
+  }
+
   /** Tests some of the properties of the {@link Lattice} data structure. */
   @Test void testLattice() throws Exception {
     modelWithLattice("star",
@@ -230,10 +268,13 @@ class LatticeTest {
             + "join \"foodmart\".\"time_by_day\" as t on t.\"time_id\" = s.\"time_id\"")
         .doWithConnection(c -> {
           final SchemaPlus schema = c.getRootSchema();
-          final SchemaPlus adhoc = schema.getSubSchema("adhoc");
+          final SchemaPlus adhoc =
+              requireNonNull(schema.getSubSchema("adhoc"));
           assertThat(adhoc.getTableNames().contains("EMPLOYEES"), is(true));
+          final CalciteSchema adhocSchema =
+              requireNonNull(adhoc.unwrap(CalciteSchema.class));
           final Map.Entry<String, CalciteSchema.LatticeEntry> entry =
-              adhoc.unwrap(CalciteSchema.class).getLatticeMap().firstEntry();
+              adhocSchema.getLatticeMap().firstEntry();
           final Lattice lattice = entry.getValue().getLattice();
           assertThat(lattice.firstColumn("S"), is(10));
           assertThat(lattice.firstColumn("P"), is(18));
@@ -976,12 +1017,12 @@ class LatticeTest {
 
   /** Unit test for {@link Lattice#getRowCount(double, List)}. */
   @Test void testColumnCount() {
-    assertThat(Lattice.getRowCount(10, 2, 3), within(5.03D, 0.01D));
-    assertThat(Lattice.getRowCount(10, 9, 8), within(9.4D, 0.01D));
-    assertThat(Lattice.getRowCount(100, 9, 8), within(54.2D, 0.1D));
-    assertThat(Lattice.getRowCount(1000, 9, 8), within(72D, 0.01D));
+    assertThat(Lattice.getRowCount(10, 2, 3), closeTo(5.03D, 0.01D));
+    assertThat(Lattice.getRowCount(10, 9, 8), closeTo(9.4D, 0.01D));
+    assertThat(Lattice.getRowCount(100, 9, 8), closeTo(54.2D, 0.1D));
+    assertThat(Lattice.getRowCount(1000, 9, 8), closeTo(72D, 0.01D));
     assertThat(Lattice.getRowCount(1000, 1, 1), is(1D));
-    assertThat(Lattice.getRowCount(1, 3, 5), within(1D, 0.01D));
-    assertThat(Lattice.getRowCount(1, 3, 5, 13, 4831), within(1D, 0.01D));
+    assertThat(Lattice.getRowCount(1, 3, 5), closeTo(1D, 0.01D));
+    assertThat(Lattice.getRowCount(1, 3, 5, 13, 4831), closeTo(1D, 0.01D));
   }
 }
