@@ -22,13 +22,28 @@ import org.apache.calcite.util.Sources;
 import org.apache.calcite.util.TestUnsafe;
 import org.apache.calcite.util.Util;
 
+import com.fasterxml.jackson.annotation.JsonCreator;
+import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
+import com.fasterxml.jackson.annotation.JsonProperty;
+import com.fasterxml.jackson.databind.JavaType;
+import com.fasterxml.jackson.databind.ObjectMapper;
+import com.fasterxml.jackson.dataformat.yaml.YAMLMapper;
+
+import org.checkerframework.checker.nullness.qual.Nullable;
 import org.junit.jupiter.api.Test;
 
+import java.io.BufferedReader;
 import java.io.File;
+import java.io.IOException;
 import java.io.PrintWriter;
 import java.io.StringWriter;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.ArrayList;
+import java.util.Comparator;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.function.BiFunction;
 import java.util.function.Consumer;
 import java.util.regex.Matcher;
@@ -40,14 +55,20 @@ import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.empty;
 import static org.hamcrest.Matchers.hasItem;
 import static org.hamcrest.Matchers.hasSize;
+import static org.junit.jupiter.api.Assertions.fail;
 import static org.junit.jupiter.api.Assumptions.assumeTrue;
 
+import static java.lang.Integer.parseInt;
+
 /** Various automated checks on the code and git history. */
 class LintTest {
   /** Pattern that matches "[CALCITE-12]" or "[CALCITE-1234]" followed by a
    * space. */
   private static final Pattern CALCITE_PATTERN =
       Pattern.compile("^(\\[CALCITE-[0-9]{1,4}][ ]).*");
+  private static final Path ROOT_PATH = Paths.get(System.getProperty("gradle.rootDir"));
+
+  private static final Map<String, String> TERMINOLOGY_MAP = new HashMap<>();
 
   @SuppressWarnings("Convert2MethodRef") // JDK 8 requires lambdas
   private Puffin.Program<GlobalState> makeProgram() {
@@ -60,7 +81,7 @@ class LintTest {
             line -> {
               final Matcher matcher = line.matcher(".* lint:skip ([0-9]+).*");
               if (matcher.matches()) {
-                int n = Integer.parseInt(matcher.group(1));
+                int n = parseInt(matcher.group(1));
                 line.state().skipToLine = line.fnr() + n;
               }
             })
@@ -330,6 +351,8 @@ class LintTest {
         hasSize(1));
     assertThat(f.apply("[CALCITE-4817] cannot start with lower-case", ""),
         hasItem("Message must start with upper-case letter"));
+    assertThat(f.apply("[MINOR] Lint", ""),
+        hasItem("starts with '[', and is not '[CALCITE-nnnn]'"));
 
     // If 'Lint:skip' occurs in the body, no checks are performed
     assertThat(
@@ -340,6 +363,21 @@ class LintTest {
         empty());
   }
 
+  static {
+    TERMINOLOGY_MAP.put("mysql", "MySQL");
+    TERMINOLOGY_MAP.put("mssql", "MSSQL");
+    TERMINOLOGY_MAP.put("Mysql", "MySQL");
+    TERMINOLOGY_MAP.put("postgresql", "PostgreSQL");
+    TERMINOLOGY_MAP.put("hive", "Hive");
+    TERMINOLOGY_MAP.put("spark", "Spark");
+    TERMINOLOGY_MAP.put("arrow", "Arrow");
+    TERMINOLOGY_MAP.put("presto", "Presto");
+    TERMINOLOGY_MAP.put("oracle", "Oracle");
+    TERMINOLOGY_MAP.put("bigquery", "BigQuery");
+    TERMINOLOGY_MAP.put("redshift", "Redshift");
+    TERMINOLOGY_MAP.put("snowflake", "Snowflake");
+  }
+
   private static void checkMessage(String subject, String body,
       Consumer<String> consumer) {
     if (body.contains("Lint:skip")) {
@@ -359,6 +397,9 @@ class LintTest {
             + "problem, not what you did");
       }
     }
+    if (subject2.startsWith("[")) {
+      consumer.accept("starts with '[', and is not '[CALCITE-nnnn]'");
+    }
     if (subject2.startsWith(" ")) {
       consumer.accept("starts with space");
     }
@@ -371,6 +412,76 @@ class LintTest {
     if (subject2.matches("[a-z].*")) {
       consumer.accept("Message must start with upper-case letter");
     }
+
+    // Check for keywords that should be capitalized
+    for (Map.Entry<String, String> entry : TERMINOLOGY_MAP.entrySet()) {
+      String keyword = entry.getKey();
+      String correctCapitalization = entry.getValue();
+      if (subject2.matches(".*\\b" + keyword + "\\b.*")) {
+        consumer.accept("Message must be capitalized as '" + correctCapitalization + "'");
+      }
+    }
+  }
+
+  /** Ensures that the {@code contributors.yml} file is sorted by name. */
+  @Test void testContributorsFileIsSorted() throws IOException {
+    final ObjectMapper mapper = new YAMLMapper();
+    final File contributorsFile = ROOT_PATH.resolve("site/_data/contributors.yml").toFile();
+    JavaType listType =
+        mapper.getTypeFactory()
+            .constructCollectionType(List.class, Contributor.class);
+    List<Contributor> contributors =
+        mapper.readValue(contributorsFile, listType);
+    Contributor contributor =
+        firstOutOfOrder(contributors,
+            Comparator.comparing(c -> c.name, String.CASE_INSENSITIVE_ORDER));
+    if (contributor != null) {
+      fail("contributor '" + contributor.name + "' is out of order");
+    }
+  }
+
+  /** Ensures that the {@code .mailmap} file is sorted. */
+  @Test void testMailmapFile() {
+    final File mailmapFile = ROOT_PATH.resolve(".mailmap").toFile();
+    final List<String> lines = new ArrayList<>();
+    forEachLineIn(mailmapFile, line -> {
+      if (!line.startsWith("#")) {
+        lines.add(line);
+      }
+    });
+    String line = firstOutOfOrder(lines, String.CASE_INSENSITIVE_ORDER);
+    if (line != null) {
+      fail("line '" + line + "' is out of order");
+    }
+  }
+
+  /** Performs an action for each line in a file. */
+  private static void forEachLineIn(File file, Consumer<String> consumer) {
+    try (BufferedReader r = Util.reader(file)) {
+      for (;;) {
+        String line = r.readLine();
+        if (line == null) {
+          break;
+        }
+        consumer.accept(line);
+      }
+    } catch (IOException e) {
+      throw Util.throwAsRuntime(e);
+    }
+  }
+
+  /** Returns the first element in a list that is out of order, or null if the
+   * list is sorted. */
+  private static <E> @Nullable E firstOutOfOrder(Iterable<E> elements,
+      Comparator<E> comparator) {
+    E previous = null;
+    for (E e : elements) {
+      if (previous != null && comparator.compare(previous, e) > 0) {
+        return e;
+      }
+      previous = e;
+    }
+    return null;
   }
 
   /** Warning that code is not as it should be. */
@@ -419,4 +530,14 @@ class LintTest {
       return javadocEndLine < javadocStartLine;
     }
   }
+
+  /** Contributor element in "contributors.yaml" file. */
+  @JsonIgnoreProperties(ignoreUnknown = true)
+  private static class Contributor {
+    final String name;
+
+    @JsonCreator Contributor(@JsonProperty("name") String name) {
+      this.name = name;
+    }
+  }
 }
