@@ -29,11 +29,14 @@ import org.apache.calcite.linq4j.tree.Expressions;
 import org.apache.calcite.linq4j.tree.ParameterExpression;
 import org.apache.calcite.linq4j.tree.Types;
 import org.apache.calcite.schema.SchemaPlus;
+import org.apache.calcite.schema.Statistic;
+import org.apache.calcite.schema.Table;
 import org.apache.calcite.schema.impl.AbstractSchema;
 import org.apache.calcite.schema.impl.TableMacroImpl;
 import org.apache.calcite.schema.impl.ViewTable;
 import org.apache.calcite.test.schemata.catchall.CatchallSchema;
 import org.apache.calcite.test.schemata.catchall.CatchallSchema.EveryType;
+import org.apache.calcite.test.schemata.hr.Dependent;
 import org.apache.calcite.test.schemata.hr.Employee;
 import org.apache.calcite.test.schemata.hr.HrSchema;
 import org.apache.calcite.util.Smalls;
@@ -60,11 +63,15 @@ import java.util.BitSet;
 import java.util.List;
 import java.util.Properties;
 
+import static org.apache.calcite.test.Matchers.isListOf;
+
 import static org.hamcrest.CoreMatchers.equalTo;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
+import static org.hamcrest.Matchers.arrayWithSize;
+import static org.hamcrest.Matchers.hasSize;
 import static org.hamcrest.Matchers.hasToString;
-import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
 
@@ -79,6 +86,15 @@ public class ReflectiveSchemaTest {
   private static final ReflectiveSchema CATCHALL =
       new ReflectiveSchema(new CatchallSchema());
 
+  /**
+   * Extension of HrSchema with a dummy field for testing a row count from a Collection.
+   */
+  public static class HrSchemaPlus extends HrSchema {
+    public final ImmutableList<Dependent> xlocations =
+        ImmutableList.of(new Dependent(10, "San Francisco"),
+            new Dependent(20, "San Diego"));
+  }
+
   /**
    * Test that uses a JDBC connection as a linq4j
    * {@link org.apache.calcite.linq4j.QueryProvider}.
@@ -129,10 +145,10 @@ public class ReflectiveSchemaTest {
                             "toUpperCase")),
                     e))
             .toList();
-    assertEquals(1, list.size());
-    assertEquals(2, list.get(0).length);
-    assertEquals(150, list.get(0)[0]);
-    assertEquals("SEBASTIAN", list.get(0)[1]);
+    assertThat(list, hasSize(1));
+    assertThat(list.get(0), arrayWithSize(2));
+    assertThat(list.get(0)[0], is(150));
+    assertThat(list.get(0)[1], is("SEBASTIAN"));
   }
 
   @Test void testQueryProviderSingleColumn() throws Exception {
@@ -157,7 +173,7 @@ public class ReflectiveSchemaTest {
                     Expressions.field(e, "empid"),
                     e))
             .toList();
-    assertEquals(Arrays.asList(100, 200, 150, 110), list);
+    assertThat(list, isListOf(100, 200, 150, 110));
   }
 
   /**
@@ -242,16 +258,16 @@ public class ReflectiveSchemaTest {
     // "hr_emps" -> "hr"."emps", 4 rows
     ResultSet resultSet =
         statement.executeQuery("select * from \"s\".\"hr_emps\"");
-    assertEquals(4, count(resultSet));
+    assertThat(count(resultSet), is(4));
 
     // "s_emps" -> "s"."emps", 3 rows
     resultSet =
         statement.executeQuery("select * from \"s\".\"s_emps\"");
-    assertEquals(3, count(resultSet));
+    assertThat(count(resultSet), is(3));
 
     // "null_emps" -> "s"."emps", 3
     resultSet = statement.executeQuery("select * from \"s\".\"null_emps\"");
-    assertEquals(3, count(resultSet));
+    assertThat(count(resultSet), is(3));
     statement.close();
   }
 
@@ -919,4 +935,27 @@ public class ReflectiveSchemaTest {
             "EXPR$0=0",
             "EXPR$0=null");
   }
+
+  /**
+   * Test that the row count statistic can be retrieved from a ReflectiveSchema.
+   *
+   * @see <a href="https://issues.apache.org/jira/browse/CALCITE-5649">[CALCITE-5649]</a>
+   */
+  @Test void testArrayFieldTableHasRowCount() {
+    ReflectiveSchema schema = new ReflectiveSchema(new HrSchema());
+    Table table = schema.getTable("emps");
+    assertNotNull(table);
+    Statistic statistic = table.getStatistic();
+    assertNotNull(statistic);
+    assertThat(statistic.getRowCount(), is(4D));
+  }
+
+  @Test void testCollectionFieldTableHasRowCount() {
+    ReflectiveSchema schema = new ReflectiveSchema(new HrSchemaPlus());
+    Table table = schema.getTable("xlocations");
+    assertNotNull(table);
+    Statistic statistic = table.getStatistic();
+    assertNotNull(statistic);
+    assertThat(statistic.getRowCount(), is(2D));
+  }
 }
