@@ -23,6 +23,7 @@ import org.apache.calcite.runtime.Utilities;
 
 import com.google.common.collect.ImmutableList;
 
+import org.hamcrest.Matcher;
 import org.junit.jupiter.api.Test;
 
 import java.math.BigDecimal;
@@ -44,6 +45,8 @@ import static org.apache.calcite.runtime.SqlFunctions.arraysOverlap;
 import static org.apache.calcite.runtime.SqlFunctions.charLength;
 import static org.apache.calcite.runtime.SqlFunctions.concat;
 import static org.apache.calcite.runtime.SqlFunctions.concatMulti;
+import static org.apache.calcite.runtime.SqlFunctions.concatMultiObjectWithSeparator;
+import static org.apache.calcite.runtime.SqlFunctions.concatMultiTypeWithSeparator;
 import static org.apache.calcite.runtime.SqlFunctions.concatMultiWithNull;
 import static org.apache.calcite.runtime.SqlFunctions.concatMultiWithSeparator;
 import static org.apache.calcite.runtime.SqlFunctions.concatWithNull;
@@ -57,11 +60,13 @@ import static org.apache.calcite.runtime.SqlFunctions.lesser;
 import static org.apache.calcite.runtime.SqlFunctions.lower;
 import static org.apache.calcite.runtime.SqlFunctions.ltrim;
 import static org.apache.calcite.runtime.SqlFunctions.md5;
+import static org.apache.calcite.runtime.SqlFunctions.overlay;
 import static org.apache.calcite.runtime.SqlFunctions.position;
 import static org.apache.calcite.runtime.SqlFunctions.rtrim;
 import static org.apache.calcite.runtime.SqlFunctions.sha1;
 import static org.apache.calcite.runtime.SqlFunctions.sha256;
 import static org.apache.calcite.runtime.SqlFunctions.sha512;
+import static org.apache.calcite.runtime.SqlFunctions.substring;
 import static org.apache.calcite.runtime.SqlFunctions.toBase64;
 import static org.apache.calcite.runtime.SqlFunctions.toInt;
 import static org.apache.calcite.runtime.SqlFunctions.toIntOptional;
@@ -69,12 +74,13 @@ import static org.apache.calcite.runtime.SqlFunctions.toLong;
 import static org.apache.calcite.runtime.SqlFunctions.toLongOptional;
 import static org.apache.calcite.runtime.SqlFunctions.trim;
 import static org.apache.calcite.runtime.SqlFunctions.upper;
-import static org.apache.calcite.test.Matchers.within;
+import static org.apache.calcite.test.Matchers.isListOf;
 
 import static org.hamcrest.CoreMatchers.equalTo;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.CoreMatchers.nullValue;
 import static org.hamcrest.MatcherAssert.assertThat;
+import static org.hamcrest.Matchers.closeTo;
 import static org.hamcrest.Matchers.hasToString;
 import static org.junit.jupiter.api.Assertions.assertSame;
 import static org.junit.jupiter.api.Assertions.fail;
@@ -132,22 +138,22 @@ class SqlFunctionsTest {
 
   @Test void testToString() {
     assertThat(SqlFunctions.toString(0f), is("0E0"));
-    assertThat(SqlFunctions.toString(1f), is("1"));
+    assertThat(SqlFunctions.toString(1f), is("1.0"));
     assertThat(SqlFunctions.toString(1.5f), is("1.5"));
     assertThat(SqlFunctions.toString(-1.5f), is("-1.5"));
     assertThat(SqlFunctions.toString(1.5e8f), is("1.5E8"));
     assertThat(SqlFunctions.toString(-0.0625f), is("-0.0625"));
     assertThat(SqlFunctions.toString(0.0625f), is("0.0625"));
-    assertThat(SqlFunctions.toString(-5e-12f), is("-5E-12"));
+    assertThat(SqlFunctions.toString(-5e-12f), is("-5.0E-12"));
 
     assertThat(SqlFunctions.toString(0d), is("0E0"));
-    assertThat(SqlFunctions.toString(1d), is("1"));
+    assertThat(SqlFunctions.toString(1d), is("1.0"));
     assertThat(SqlFunctions.toString(1.5d), is("1.5"));
     assertThat(SqlFunctions.toString(-1.5d), is("-1.5"));
     assertThat(SqlFunctions.toString(1.5e8d), is("1.5E8"));
     assertThat(SqlFunctions.toString(-0.0625d), is("-0.0625"));
     assertThat(SqlFunctions.toString(0.0625d), is("0.0625"));
-    assertThat(SqlFunctions.toString(-5e-12d), is("-5E-12"));
+    assertThat(SqlFunctions.toString(-5e-12d), is("-5.0E-12"));
 
     assertThat(SqlFunctions.toString(new BigDecimal("0")), is("0"));
     assertThat(SqlFunctions.toString(new BigDecimal("1")), is("1"));
@@ -169,6 +175,61 @@ class SqlFunctionsTest {
     assertThat(concat(null, "b"), is("nullb"));
   }
 
+  /** Test case for
+   * <a href="https://issues.apache.org/jira/browse/CALCITE-6433">[CALCITE-6433]
+   * SUBSTRING can return incorrect empty result for some parameters</a>. */
+  @Test void testSubString() {
+    // str vs single param
+    assertThat(substring("string", -1), is("string"));
+    assertThat(substring("string", -1L), is("string"));
+    assertThat(substring("string", 2), is("tring"));
+    assertThat(substring("string", 2L), is("tring"));
+    assertThat(substring("string", Integer.MIN_VALUE), is("string"));
+    assertThat(substring("string", Long.MIN_VALUE), is("string"));
+    assertThat(substring("string", Integer.MIN_VALUE + 10), is("string"));
+    assertThat(substring("string", Integer.MAX_VALUE), is(""));
+    assertThat(substring("string", Long.MAX_VALUE), is(""));
+    assertThat(substring("string", Integer.MAX_VALUE - 10), is(""));
+    assertThat(substring("string", Integer.MIN_VALUE - 10L), is("string"));
+    assertThat(substring("string", Integer.MAX_VALUE + 10L), is(""));
+
+    // str vs multi params
+    assertThat(substring("string", -1, 1), is(""));
+    assertThat(substring("string", -1, 1L), is(""));
+    assertThat(substring("string", -1L, 1), is(""));
+    assertThat(substring("string", -1L, 1L), is(""));
+
+    assertThat(substring("string", 1, 2), is("st"));
+    assertThat(substring("string", 1, 2L), is("st"));
+    assertThat(substring("string", 1L, 2), is("st"));
+    assertThat(substring("string", 1L, 2L), is("st"));
+
+    assertThat(substring("string", -1, 2), is(""));
+    assertThat(substring("string", -1L, 2), is(""));
+    assertThat(substring("string", -1, 2L), is(""));
+    assertThat(substring("string", -1L, 2L), is(""));
+
+    assertThat(substring("string", -1, 3), is("s"));
+    assertThat(substring("string", -1L, 3), is("s"));
+    assertThat(substring("string", -1, 3L), is("s"));
+    assertThat(substring("string", -1L, 3L), is("s"));
+
+    assertThat(substring("string", -10, 12), is("s"));
+    assertThat(substring("string", -10L, 12), is("s"));
+    assertThat(substring("string", -10, 12L), is("s"));
+    assertThat(substring("string", -10L, 12L), is("s"));
+
+    assertThat(substring("string", -1, Integer.MAX_VALUE), is("string"));
+    assertThat(substring("string", -1L, Integer.MAX_VALUE), is("string"));
+    assertThat(substring("string", -1, Long.MAX_VALUE), is("string"));
+
+    assertThat(substring("string", Integer.MIN_VALUE, Integer.MAX_VALUE), is(""));
+    assertThat(substring("string", Integer.MIN_VALUE, Integer.MAX_VALUE + 10L), is("string"));
+    assertThat(substring("string", Long.MIN_VALUE, Integer.MAX_VALUE), is(""));
+    assertThat(substring("string", Integer.MIN_VALUE, Long.MAX_VALUE), is("string"));
+    assertThat(substring("string", Integer.MIN_VALUE - 10L, Long.MAX_VALUE), is("string"));
+  }
+
   @Test void testConcatWithNull() {
     assertThat(concatWithNull("a b", "cd"), is("a bcd"));
     // Null value could be passed in. If we pass one null value,
@@ -211,6 +272,53 @@ class SqlFunctionsTest {
     assertThat(concatMultiWithSeparator(null, null, null), is(""));
   }
 
+  /** Test case for
+   * <a href="https://issues.apache.org/jira/browse/CALCITE-6446">[CALCITE-6446]
+   * Add CONCAT_WS function (enabled in Spark library)</a>. */
+  @Test void testConcatMultiTypeWithSeparator() {
+    // string type
+    assertThat(concatMultiTypeWithSeparator("a"), is(""));
+    assertThat(concatMultiTypeWithSeparator(",", "a"), is("a"));
+    assertThat(concatMultiTypeWithSeparator(",", "a b", "cd"), is("a b,cd"));
+    assertThat(concatMultiTypeWithSeparator(",", "a b", null, "cd", null, "e"), is("a b,cd,e"));
+    assertThat(concatMultiTypeWithSeparator(",", "", ""), is(","));
+    assertThat(concatMultiTypeWithSeparator("", null, null), is(""));
+    // array type
+    assertThat(concatMultiTypeWithSeparator(",", Arrays.asList()), is(""));
+    assertThat(concatMultiTypeWithSeparator(",", Arrays.asList("a")), is("a"));
+    assertThat(concatMultiTypeWithSeparator(",", Arrays.asList("a", "b")), is("a,b"));
+    assertThat(concatMultiTypeWithSeparator(",", Arrays.asList("a", null, "b")), is("a,b"));
+    assertThat(concatMultiTypeWithSeparator(",", Arrays.asList(null, "b")), is("b"));
+    assertThat(concatMultiTypeWithSeparator(",", Arrays.asList(null, null)), is(""));
+    assertThat(
+        concatMultiTypeWithSeparator(",",
+            Arrays.asList("11", "11"), Arrays.asList("12", "12")), is("11,11,12,12"));
+    // multi type
+    assertThat(concatMultiTypeWithSeparator(",", "11", "11", Arrays.asList("12", "12")),
+        is("11,11,12,12"));
+    assertThat(concatMultiTypeWithSeparator(",", null, "11", Arrays.asList("12", "12")),
+        is("11,12,12"));
+    assertThat(concatMultiTypeWithSeparator(",", "11", null, Arrays.asList("12", "12")),
+        is("11,12,12"));
+    assertThat(
+        concatMultiTypeWithSeparator(",", "11", "11", Arrays.asList("12", "12"),
+            Arrays.asList("13", null, "13")),
+        is("11,11,12,12,13,13"));
+  }
+
+  /** Test case for
+   * <a href="https://issues.apache.org/jira/browse/CALCITE-6450">[CALCITE-6450]
+   * Postgres CONCAT_WS function </a>. */
+  @Test void testConcatMultiObjectWithSeparator() {
+    assertThat(concatMultiObjectWithSeparator("a"), is(""));
+    assertThat(concatMultiObjectWithSeparator(",", "a b", "cd"), is("a b,cd"));
+    assertThat(concatMultiObjectWithSeparator(",", "a", 1, Arrays.asList("b", "c")),
+        is("a,1,[b, c]"));
+    assertThat(concatMultiObjectWithSeparator(",", "a", 1, Arrays.asList("b", "c"), null),
+        is("a,1,[b, c]"));
+    assertThat(concatMultiObjectWithSeparator("abc", null, null), is(""));
+  }
+
   @Test void testPosixRegex() {
     final SqlFunctions.PosixRegexFunction f =
         new SqlFunctions.PosixRegexFunction();
@@ -446,6 +554,7 @@ class SqlFunctionsTest {
 
   @Test void testRegexpReplace() {
     final SqlFunctions.RegexFunction f = new SqlFunctions.RegexFunction();
+    assertThat(f.regexpReplace("abc", "b"), is("ac"));
     assertThat(f.regexpReplace("a b c", "b", "X"), is("a X c"));
     assertThat(f.regexpReplace("abc def ghi", "[g-z]+", "X"), is("abc def X"));
     assertThat(f.regexpReplace("abc def ghi", "[a-z]+", "X"), is("X X X"));
@@ -464,6 +573,11 @@ class SqlFunctionsTest {
         is("abc def GHI"));
     assertThat(f.regexpReplace("abc def GHI", "[a-z]+", "X", 1, 3, "i"),
         is("abc def X"));
+    assertThat(f.regexpReplacePg("abc def GHI", "[a-z]+", "X"), is("X def GHI"));
+    assertThat(f.regexpReplacePg("abc def GHI", "[a-z]+", "X", "g"),
+        is("X X GHI"));
+    assertThat(f.regexpReplacePg("ABC def GHI", "[a-z]+", "X", "i"),
+        is("X def GHI"));
 
     try {
       f.regexpReplace("abc def ghi", "[a-z]+", "X", 0);
@@ -661,6 +775,15 @@ class SqlFunctionsTest {
     assertThat(trimSpacesBoth("x"), is("x"));
   }
 
+  /** Test for {@link SqlFunctions#overlay}. */
+  @Test void testOverlay() {
+    assertThat(overlay("HelloWorld", "Java", 6), is("HelloJavad"));
+    assertThat(overlay("Hello World", "World", 1), is("World World"));
+    assertThat(overlay("HelloWorld", "Java", 6, 5), is("HelloJava"));
+    assertThat(overlay("HelloWorld", "Java", 6, 0), is("HelloJavaWorld"));
+  }
+
+
   static String trimSpacesBoth(String s) {
     return trim(true, true, " ", s);
   }
@@ -749,129 +872,129 @@ class SqlFunctionsTest {
   }
 
   @Test void testSTruncateDouble() {
-    assertThat(SqlFunctions.struncate(12.345d, 3), within(12.345d, 0.001));
-    assertThat(SqlFunctions.struncate(12.345d, 2), within(12.340d, 0.001));
-    assertThat(SqlFunctions.struncate(12.345d, 1), within(12.300d, 0.001));
-    assertThat(SqlFunctions.struncate(12.999d, 0), within(12.000d, 0.001));
+    assertThat(SqlFunctions.struncate(12.345d, 3), closeTo(12.345d, 0.001));
+    assertThat(SqlFunctions.struncate(12.345d, 2), closeTo(12.340d, 0.001));
+    assertThat(SqlFunctions.struncate(12.345d, 1), closeTo(12.300d, 0.001));
+    assertThat(SqlFunctions.struncate(12.999d, 0), closeTo(12.000d, 0.001));
 
-    assertThat(SqlFunctions.struncate(-12.345d, 3), within(-12.345d, 0.001));
-    assertThat(SqlFunctions.struncate(-12.345d, 2), within(-12.340d, 0.001));
-    assertThat(SqlFunctions.struncate(-12.345d, 1), within(-12.300d, 0.001));
-    assertThat(SqlFunctions.struncate(-12.999d, 0), within(-12.000d, 0.001));
+    assertThat(SqlFunctions.struncate(-12.345d, 3), closeTo(-12.345d, 0.001));
+    assertThat(SqlFunctions.struncate(-12.345d, 2), closeTo(-12.340d, 0.001));
+    assertThat(SqlFunctions.struncate(-12.345d, 1), closeTo(-12.300d, 0.001));
+    assertThat(SqlFunctions.struncate(-12.999d, 0), closeTo(-12.000d, 0.001));
 
-    assertThat(SqlFunctions.struncate(12345d, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.struncate(12000d, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.struncate(12001d, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.struncate(12000d, -4), within(10000d, 0.001));
-    assertThat(SqlFunctions.struncate(12000d, -5), within(0d, 0.001));
-    assertThat(SqlFunctions.struncate(11999d, -3), within(11000d, 0.001));
+    assertThat(SqlFunctions.struncate(12345d, -3), closeTo(12000d, 0.001));
+    assertThat(SqlFunctions.struncate(12000d, -3), closeTo(12000d, 0.001));
+    assertThat(SqlFunctions.struncate(12001d, -3), closeTo(12000d, 0.001));
+    assertThat(SqlFunctions.struncate(12000d, -4), closeTo(10000d, 0.001));
+    assertThat(SqlFunctions.struncate(12000d, -5), closeTo(0d, 0.001));
+    assertThat(SqlFunctions.struncate(11999d, -3), closeTo(11000d, 0.001));
 
-    assertThat(SqlFunctions.struncate(-12345d, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.struncate(-12000d, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.struncate(-11999d, -3), within(-11000d, 0.001));
-    assertThat(SqlFunctions.struncate(-12000d, -4), within(-10000d, 0.001));
-    assertThat(SqlFunctions.struncate(-12000d, -5), within(0d, 0.001));
+    assertThat(SqlFunctions.struncate(-12345d, -3), closeTo(-12000d, 0.001));
+    assertThat(SqlFunctions.struncate(-12000d, -3), closeTo(-12000d, 0.001));
+    assertThat(SqlFunctions.struncate(-11999d, -3), closeTo(-11000d, 0.001));
+    assertThat(SqlFunctions.struncate(-12000d, -4), closeTo(-10000d, 0.001));
+    assertThat(SqlFunctions.struncate(-12000d, -5), closeTo(0d, 0.001));
   }
 
   @Test void testSTruncateLong() {
-    assertThat(SqlFunctions.struncate(12345L, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.struncate(12000L, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.struncate(12001L, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.struncate(12000L, -4), within(10000d, 0.001));
-    assertThat(SqlFunctions.struncate(12000L, -5), within(0d, 0.001));
-    assertThat(SqlFunctions.struncate(11999L, -3), within(11000d, 0.001));
+    assertThat(SqlFunctions.struncate(12345L, -3), is(12000L));
+    assertThat(SqlFunctions.struncate(12000L, -3), is(12000L));
+    assertThat(SqlFunctions.struncate(12001L, -3), is(12000L));
+    assertThat(SqlFunctions.struncate(12000L, -4), is(10000L));
+    assertThat(SqlFunctions.struncate(12000L, -5), is(0L));
+    assertThat(SqlFunctions.struncate(11999L, -3), is(11000L));
 
-    assertThat(SqlFunctions.struncate(-12345L, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.struncate(-12000L, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.struncate(-11999L, -3), within(-11000d, 0.001));
-    assertThat(SqlFunctions.struncate(-12000L, -4), within(-10000d, 0.001));
-    assertThat(SqlFunctions.struncate(-12000L, -5), within(0d, 0.001));
+    assertThat(SqlFunctions.struncate(-12345L, -3), is(-12000L));
+    assertThat(SqlFunctions.struncate(-12000L, -3), is(-12000L));
+    assertThat(SqlFunctions.struncate(-11999L, -3), is(-11000L));
+    assertThat(SqlFunctions.struncate(-12000L, -4), is(-10000L));
+    assertThat(SqlFunctions.struncate(-12000L, -5), is(0L));
   }
 
   @Test void testSTruncateInt() {
-    assertThat(SqlFunctions.struncate(12345, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.struncate(12000, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.struncate(12001, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.struncate(12000, -4), within(10000d, 0.001));
-    assertThat(SqlFunctions.struncate(12000, -5), within(0d, 0.001));
-    assertThat(SqlFunctions.struncate(11999, -3), within(11000d, 0.001));
+    assertThat(SqlFunctions.struncate(12345, -3), is(12000));
+    assertThat(SqlFunctions.struncate(12000, -3), is(12000));
+    assertThat(SqlFunctions.struncate(12001, -3), is(12000));
+    assertThat(SqlFunctions.struncate(12000, -4), is(10000));
+    assertThat(SqlFunctions.struncate(12000, -5), is(0));
+    assertThat(SqlFunctions.struncate(11999, -3), is(11000));
 
-    assertThat(SqlFunctions.struncate(-12345, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.struncate(-12000, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.struncate(-11999, -3), within(-11000d, 0.001));
-    assertThat(SqlFunctions.struncate(-12000, -4), within(-10000d, 0.001));
-    assertThat(SqlFunctions.struncate(-12000, -5), within(0d, 0.001));
+    assertThat(SqlFunctions.struncate(-12345, -3), is(-12000));
+    assertThat(SqlFunctions.struncate(-12000, -3), is(-12000));
+    assertThat(SqlFunctions.struncate(-11999, -3), is(-11000));
+    assertThat(SqlFunctions.struncate(-12000, -4), is(-10000));
+    assertThat(SqlFunctions.struncate(-12000, -5), is(0));
   }
 
   @Test void testSRoundDouble() {
-    assertThat(SqlFunctions.sround(12.345d, 3), within(12.345d, 0.001));
-    assertThat(SqlFunctions.sround(12.345d, 2), within(12.350d, 0.001));
-    assertThat(SqlFunctions.sround(12.345d, 1), within(12.300d, 0.001));
-    assertThat(SqlFunctions.sround(12.999d, 2), within(13.000d, 0.001));
-    assertThat(SqlFunctions.sround(12.999d, 1), within(13.000d, 0.001));
-    assertThat(SqlFunctions.sround(12.999d, 0), within(13.000d, 0.001));
+    assertThat(SqlFunctions.sround(12.345d, 3), closeTo(12.345d, 0.001));
+    assertThat(SqlFunctions.sround(12.345d, 2), closeTo(12.350d, 0.001));
+    assertThat(SqlFunctions.sround(12.345d, 1), closeTo(12.300d, 0.001));
+    assertThat(SqlFunctions.sround(12.999d, 2), closeTo(13.000d, 0.001));
+    assertThat(SqlFunctions.sround(12.999d, 1), closeTo(13.000d, 0.001));
+    assertThat(SqlFunctions.sround(12.999d, 0), closeTo(13.000d, 0.001));
 
-    assertThat(SqlFunctions.sround(-12.345d, 3), within(-12.345d, 0.001));
-    assertThat(SqlFunctions.sround(-12.345d, 2), within(-12.350d, 0.001));
-    assertThat(SqlFunctions.sround(-12.345d, 1), within(-12.300d, 0.001));
-    assertThat(SqlFunctions.sround(-12.999d, 2), within(-13.000d, 0.001));
-    assertThat(SqlFunctions.sround(-12.999d, 1), within(-13.000d, 0.001));
-    assertThat(SqlFunctions.sround(-12.999d, 0), within(-13.000d, 0.001));
+    assertThat(SqlFunctions.sround(-12.345d, 3), closeTo(-12.345d, 0.001));
+    assertThat(SqlFunctions.sround(-12.345d, 2), closeTo(-12.350d, 0.001));
+    assertThat(SqlFunctions.sround(-12.345d, 1), closeTo(-12.300d, 0.001));
+    assertThat(SqlFunctions.sround(-12.999d, 2), closeTo(-13.000d, 0.001));
+    assertThat(SqlFunctions.sround(-12.999d, 1), closeTo(-13.000d, 0.001));
+    assertThat(SqlFunctions.sround(-12.999d, 0), closeTo(-13.000d, 0.001));
 
-    assertThat(SqlFunctions.sround(12345d, -1), within(12350d, 0.001));
-    assertThat(SqlFunctions.sround(12345d, -2), within(12300d, 0.001));
-    assertThat(SqlFunctions.sround(12345d, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.sround(12000d, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.sround(12001d, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.sround(12000d, -4), within(10000d, 0.001));
-    assertThat(SqlFunctions.sround(12000d, -5), within(0d, 0.001));
-    assertThat(SqlFunctions.sround(11999d, -3), within(12000d, 0.001));
+    assertThat(SqlFunctions.sround(12345d, -1), closeTo(12350d, 0.001));
+    assertThat(SqlFunctions.sround(12345d, -2), closeTo(12300d, 0.001));
+    assertThat(SqlFunctions.sround(12345d, -3), closeTo(12000d, 0.001));
+    assertThat(SqlFunctions.sround(12000d, -3), closeTo(12000d, 0.001));
+    assertThat(SqlFunctions.sround(12001d, -3), closeTo(12000d, 0.001));
+    assertThat(SqlFunctions.sround(12000d, -4), closeTo(10000d, 0.001));
+    assertThat(SqlFunctions.sround(12000d, -5), closeTo(0d, 0.001));
+    assertThat(SqlFunctions.sround(11999d, -3), closeTo(12000d, 0.001));
 
-    assertThat(SqlFunctions.sround(-12345d, -1), within(-12350d, 0.001));
-    assertThat(SqlFunctions.sround(-12345d, -2), within(-12300d, 0.001));
-    assertThat(SqlFunctions.sround(-12345d, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.sround(-12000d, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.sround(-11999d, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.sround(-12000d, -4), within(-10000d, 0.001));
-    assertThat(SqlFunctions.sround(-12000d, -5), within(0d, 0.001));
+    assertThat(SqlFunctions.sround(-12345d, -1), closeTo(-12350d, 0.001));
+    assertThat(SqlFunctions.sround(-12345d, -2), closeTo(-12300d, 0.001));
+    assertThat(SqlFunctions.sround(-12345d, -3), closeTo(-12000d, 0.001));
+    assertThat(SqlFunctions.sround(-12000d, -3), closeTo(-12000d, 0.001));
+    assertThat(SqlFunctions.sround(-11999d, -3), closeTo(-12000d, 0.001));
+    assertThat(SqlFunctions.sround(-12000d, -4), closeTo(-10000d, 0.001));
+    assertThat(SqlFunctions.sround(-12000d, -5), closeTo(0d, 0.001));
   }
 
   @Test void testSRoundLong() {
-    assertThat(SqlFunctions.sround(12345L, -1), within(12350d, 0.001));
-    assertThat(SqlFunctions.sround(12345L, -2), within(12300d, 0.001));
-    assertThat(SqlFunctions.sround(12345L, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.sround(12000L, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.sround(12001L, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.sround(12000L, -4), within(10000d, 0.001));
-    assertThat(SqlFunctions.sround(12000L, -5), within(0d, 0.001));
-    assertThat(SqlFunctions.sround(11999L, -3), within(12000d, 0.001));
+    assertThat(SqlFunctions.sround(12345L, -1), is(12350L));
+    assertThat(SqlFunctions.sround(12345L, -2), is(12300L));
+    assertThat(SqlFunctions.sround(12345L, -3), is(12000L));
+    assertThat(SqlFunctions.sround(12000L, -3), is(12000L));
+    assertThat(SqlFunctions.sround(12001L, -3), is(12000L));
+    assertThat(SqlFunctions.sround(12000L, -4), is(10000L));
+    assertThat(SqlFunctions.sround(12000L, -5), is(0L));
+    assertThat(SqlFunctions.sround(11999L, -3), is(12000L));
 
-    assertThat(SqlFunctions.sround(-12345L, -1), within(-12350d, 0.001));
-    assertThat(SqlFunctions.sround(-12345L, -2), within(-12300d, 0.001));
-    assertThat(SqlFunctions.sround(-12345L, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.sround(-12000L, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.sround(-11999L, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.sround(-12000L, -4), within(-10000d, 0.001));
-    assertThat(SqlFunctions.sround(-12000L, -5), within(0d, 0.001));
+    assertThat(SqlFunctions.sround(-12345L, -1), is(-12350L));
+    assertThat(SqlFunctions.sround(-12345L, -2), is(-12300L));
+    assertThat(SqlFunctions.sround(-12345L, -3), is(-12000L));
+    assertThat(SqlFunctions.sround(-12000L, -3), is(-12000L));
+    assertThat(SqlFunctions.sround(-11999L, -3), is(-12000L));
+    assertThat(SqlFunctions.sround(-12000L, -4), is(-10000L));
+    assertThat(SqlFunctions.sround(-12000L, -5), is(0L));
   }
 
   @Test void testSRoundInt() {
-    assertThat(SqlFunctions.sround(12345, -1), within(12350d, 0.001));
-    assertThat(SqlFunctions.sround(12345, -2), within(12300d, 0.001));
-    assertThat(SqlFunctions.sround(12345, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.sround(12000, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.sround(12001, -3), within(12000d, 0.001));
-    assertThat(SqlFunctions.sround(12000, -4), within(10000d, 0.001));
-    assertThat(SqlFunctions.sround(12000, -5), within(0d, 0.001));
-    assertThat(SqlFunctions.sround(11999, -3), within(12000d, 0.001));
+    assertThat(SqlFunctions.sround(12345, -1), is(12350));
+    assertThat(SqlFunctions.sround(12345, -2), is(12300));
+    assertThat(SqlFunctions.sround(12345, -3), is(12000));
+    assertThat(SqlFunctions.sround(12000, -3), is(12000));
+    assertThat(SqlFunctions.sround(12001, -3), is(12000));
+    assertThat(SqlFunctions.sround(12000, -4), is(10000));
+    assertThat(SqlFunctions.sround(12000, -5), is(0));
+    assertThat(SqlFunctions.sround(11999, -3), is(12000));
 
-    assertThat(SqlFunctions.sround(-12345, -1), within(-12350d, 0.001));
-    assertThat(SqlFunctions.sround(-12345, -2), within(-12300d, 0.001));
-    assertThat(SqlFunctions.sround(-12345, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.sround(-12000, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.sround(-11999, -3), within(-12000d, 0.001));
-    assertThat(SqlFunctions.sround(-12000, -4), within(-10000d, 0.001));
-    assertThat(SqlFunctions.sround(-12000, -5), within(0d, 0.001));
+    assertThat(SqlFunctions.sround(-12345, -1), is(-12350));
+    assertThat(SqlFunctions.sround(-12345, -2), is(-12300));
+    assertThat(SqlFunctions.sround(-12345, -3), is(-12000));
+    assertThat(SqlFunctions.sround(-12000, -3), is(-12000));
+    assertThat(SqlFunctions.sround(-11999, -3), is(-12000));
+    assertThat(SqlFunctions.sround(-12000, -4), is(-10000));
+    assertThat(SqlFunctions.sround(-12000, -5), is(0));
   }
 
   @Test void testSplit() {
@@ -1296,51 +1419,53 @@ class SqlFunctionsTest {
     final List<String> addc = Arrays.asList("a", "d", "c", "d", "c");
     final List<String> z = Collections.emptyList();
     assertThat(SqlFunctions.multisetExceptAll(abacee, addc),
-        is(Arrays.asList("b", "a", "e", "e")));
+        isListOf("b", "a", "e", "e"));
     assertThat(SqlFunctions.multisetExceptAll(abacee, z), is(abacee));
     assertThat(SqlFunctions.multisetExceptAll(z, z), is(z));
     assertThat(SqlFunctions.multisetExceptAll(z, addc), is(z));
 
     assertThat(SqlFunctions.multisetExceptDistinct(abacee, addc),
-        is(Arrays.asList("b", "e")));
+        isListOf("b", "e"));
     assertThat(SqlFunctions.multisetExceptDistinct(abacee, z),
-        is(Arrays.asList("a", "b", "c", "e")));
+        isListOf("a", "b", "c", "e"));
     assertThat(SqlFunctions.multisetExceptDistinct(z, z), is(z));
     assertThat(SqlFunctions.multisetExceptDistinct(z, addc), is(z));
 
+    Matcher<Object> result;
+    result = isListOf("a", "c");
     assertThat(SqlFunctions.multisetIntersectAll(abacee, addc),
-        is(Arrays.asList("a", "c")));
+        result);
     assertThat(SqlFunctions.multisetIntersectAll(abacee, adaa),
-        is(Arrays.asList("a", "a")));
+        isListOf("a", "a"));
     assertThat(SqlFunctions.multisetIntersectAll(adaa, abacee),
-        is(Arrays.asList("a", "a")));
+        isListOf("a", "a"));
     assertThat(SqlFunctions.multisetIntersectAll(abacee, z), is(z));
     assertThat(SqlFunctions.multisetIntersectAll(z, z), is(z));
     assertThat(SqlFunctions.multisetIntersectAll(z, addc), is(z));
 
     assertThat(SqlFunctions.multisetIntersectDistinct(abacee, addc),
-        is(Arrays.asList("a", "c")));
+        isListOf("a", "c"));
     assertThat(SqlFunctions.multisetIntersectDistinct(abacee, adaa),
-        is(Collections.singletonList("a")));
+        isListOf("a"));
     assertThat(SqlFunctions.multisetIntersectDistinct(adaa, abacee),
-        is(Collections.singletonList("a")));
+        isListOf("a"));
     assertThat(SqlFunctions.multisetIntersectDistinct(abacee, z), is(z));
     assertThat(SqlFunctions.multisetIntersectDistinct(z, z), is(z));
     assertThat(SqlFunctions.multisetIntersectDistinct(z, addc), is(z));
 
     assertThat(SqlFunctions.multisetUnionAll(abacee, addc),
-        is(Arrays.asList("a", "b", "a", "c", "e", "e", "a", "d", "c", "d", "c")));
+        isListOf("a", "b", "a", "c", "e", "e", "a", "d", "c", "d", "c"));
     assertThat(SqlFunctions.multisetUnionAll(abacee, z), is(abacee));
     assertThat(SqlFunctions.multisetUnionAll(z, z), is(z));
     assertThat(SqlFunctions.multisetUnionAll(z, addc), is(addc));
 
     assertThat(SqlFunctions.multisetUnionDistinct(abacee, addc),
-        is(Arrays.asList("a", "b", "c", "d", "e")));
+        isListOf("a", "b", "c", "d", "e"));
     assertThat(SqlFunctions.multisetUnionDistinct(abacee, z),
-        is(Arrays.asList("a", "b", "c", "e")));
+        isListOf("a", "b", "c", "e"));
     assertThat(SqlFunctions.multisetUnionDistinct(z, z), is(z));
     assertThat(SqlFunctions.multisetUnionDistinct(z, addc),
-        is(Arrays.asList("a", "c", "d")));
+        isListOf("a", "c", "d"));
   }
 
   @Test void testMd5() {
@@ -1759,6 +1884,24 @@ class SqlFunctionsTest {
         is("1500-04-30 12:00:00.123"));
   }
 
+  @Test void testToDate() {
+    String pattern1 = "YYYY-MM-DD";
+
+    final SqlFunctions.DateFormatFunction f =
+        new SqlFunctions.DateFormatFunction();
+
+    assertThat(f.toDate("2001-10-06", pattern1), is(11601));
+  }
+
+  @Test void testToTimestamp() {
+    String pattern1 = "HH24:MI:SS YYYY-MM-DD";
+
+    final SqlFunctions.DateFormatFunction f =
+        new SqlFunctions.DateFormatFunction();
+
+    assertThat(f.toTimestamp("18:43:36 2001-10-06", pattern1), is(1002393816000L));
+  }
+
   /**
    * Tests that a Unix timestamp converts to a SQL timestamp in the local time
    * zone.
