@@ -45,7 +45,6 @@ import static org.hamcrest.CoreMatchers.sameInstance;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.hasSize;
 import static org.hamcrest.Matchers.hasToString;
-import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotSame;
 import static org.junit.jupiter.api.Assertions.assertSame;
@@ -123,7 +122,7 @@ class ImmutableBitSetTest {
       }
       buf.append(i);
     }
-    assertEquals(expected, buf.toString());
+    assertThat(buf, hasToString(expected));
   }
 
   /**
@@ -173,16 +172,15 @@ class ImmutableBitSetTest {
    * Tests the method {@link BitSets#range(int, int)}.
    */
   @Test void testRange() {
-    assertEquals(ImmutableBitSet.range(0, 4).toList(),
-        Arrays.asList(0, 1, 2, 3));
-    assertEquals(ImmutableBitSet.range(1, 4).toList(),
-        Arrays.asList(1, 2, 3));
-    assertEquals(ImmutableBitSet.range(4).toList(),
-        Arrays.asList(0, 1, 2, 3));
-    assertEquals(ImmutableBitSet.range(0).toList(),
-        Collections.<Integer>emptyList());
-    assertEquals(ImmutableBitSet.range(2, 2).toList(),
-        Collections.<Integer>emptyList());
+    final List<Integer> list0123 = Arrays.asList(0, 1, 2, 3);
+    final List<Integer> list123 = Arrays.asList(1, 2, 3);
+    final List<Integer> listEmpty = Collections.emptyList();
+
+    assertThat(ImmutableBitSet.range(0, 4).toList(), is(list0123));
+    assertThat(ImmutableBitSet.range(1, 4).toList(), is(list123));
+    assertThat(ImmutableBitSet.range(4).toList(), is(list0123));
+    assertThat(ImmutableBitSet.range(0).toList(), is(listEmpty));
+    assertThat(ImmutableBitSet.range(2, 2).toList(), is(listEmpty));
 
     assertThat(ImmutableBitSet.range(63, 66),
         hasToString("{63, 64, 65}"));
@@ -224,10 +222,10 @@ class ImmutableBitSetTest {
         if (c == 0) {
           assertTrue(i == j || i == 3 && j == 4 || i == 4 && j == 3);
         } else {
-          assertEquals(c, Utilities.compare(i, j));
+          assertThat(Utilities.compare(i, j), is(c));
         }
-        assertEquals(c == 0, set0.equals(set1));
-        assertEquals(c == 0, set1.equals(set0));
+        assertThat(set0.equals(set1), is(c == 0));
+        assertThat(set1.equals(set0), is(c == 0));
       }
     }
   }
@@ -414,8 +412,8 @@ class ImmutableBitSetTest {
     final ImmutableBitSet fives3 =
         ImmutableBitSet.builder().addAll(fives).clear(2).set(10).build();
     assertNotSame(fives3, fives);
-    assertEquals(fives3, fives);
-    assertEquals(fives3, fives2);
+    assertThat(fives, is(fives3));
+    assertThat(fives2, is(fives3));
   }
 
   @Test void testIndexOf() {
