@@ -39,11 +39,12 @@ import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Locale;
-import java.util.Objects;
 
 import static org.apache.calcite.util.DateTimeStringUtils.ISO_DATETIME_FRACTIONAL_SECOND_FORMAT;
 import static org.apache.calcite.util.DateTimeStringUtils.getDateFormatter;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * Filter element of a Druid "groupBy" or "topN" query.
  */
@@ -461,7 +462,7 @@ abstract class DruidJsonFilter implements DruidJson {
 
     JsonExpressionFilter(String expression) {
       super(Type.EXPRESSION);
-      this.expression = Objects.requireNonNull(expression, "expression");
+      this.expression = requireNonNull(expression, "expression");
     }
 
     @Override public void write(JsonGenerator generator) throws IOException {
@@ -521,11 +522,11 @@ abstract class DruidJsonFilter implements DruidJson {
   protected static class JsonBound extends DruidJsonFilter {
     private final String dimension;
 
-    private final String lower;
+    private final @Nullable String lower;
 
     private final boolean lowerStrict;
 
-    private final String upper;
+    private final @Nullable String upper;
 
     private final boolean upperStrict;
 
@@ -533,8 +534,8 @@ abstract class DruidJsonFilter implements DruidJson {
 
     private final ExtractionFunction extractionFunction;
 
-    protected JsonBound(String dimension, String lower,
-        boolean lowerStrict, String upper, boolean upperStrict,
+    protected JsonBound(String dimension, @Nullable String lower,
+        boolean lowerStrict, @Nullable String upper, boolean upperStrict,
         boolean alphaNumeric, ExtractionFunction extractionFunction) {
       super(Type.BOUND);
       this.dimension = dimension;
@@ -628,7 +629,7 @@ abstract class DruidJsonFilter implements DruidJson {
 
   public static DruidJsonFilter getSelectorFilter(String column, String value,
       ExtractionFunction extractionFunction) {
-    Objects.requireNonNull(column, "column");
+    requireNonNull(column, "column");
     return new JsonSelector(column, value, extractionFunction);
   }
 
