@@ -43,8 +43,8 @@ import static org.apache.calcite.linq4j.test.BlockBuilderBase.bool;
 import static org.apache.calcite.linq4j.test.BlockBuilderBase.optimize;
 
 import static org.hamcrest.CoreMatchers.equalTo;
+import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
-import static org.junit.jupiter.api.Assertions.assertEquals;
 
 /**
  * Unit test for {@link org.apache.calcite.linq4j.tree.BlockBuilder}
@@ -52,146 +52,160 @@ import static org.junit.jupiter.api.Assertions.assertEquals;
  */
 class OptimizerTest {
   @Test void testOptimizeComparison() {
-    assertEquals("{\n  return true;\n}\n",
-        optimize(Expressions.equal(ONE, ONE)));
+    assertThat(optimize(Expressions.equal(ONE, ONE)),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testOptimizeTernaryAlwaysTrue() {
     // true ? 1 : 2
-    assertEquals("{\n  return 1;\n}\n",
-        optimize(Expressions.condition(TRUE, ONE, TWO)));
+    assertThat(optimize(Expressions.condition(TRUE, ONE, TWO)),
+        is("{\n  return 1;\n}\n"));
   }
 
   @Test void testOptimizeTernaryAlwaysFalse() {
     // false ? 1 : 2
-    assertEquals("{\n  return 2;\n}\n",
-        optimize(Expressions.condition(FALSE, ONE, TWO)));
+    assertThat(optimize(Expressions.condition(FALSE, ONE, TWO)),
+        is("{\n  return 2;\n}\n"));
   }
 
   @Test void testOptimizeTernaryAlwaysSame() {
     // bool ? 1 : 1
-    assertEquals("{\n  return 1;\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(
-                Expressions.parameter(boolean.class, "bool"), ONE, ONE)));
+                Expressions.parameter(boolean.class, "bool"), ONE, ONE)),
+        is("{\n  return 1;\n}\n"));
   }
 
   @Test void testNonOptimizableTernary() {
     // bool ? 1 : 2
-    assertEquals("{\n  return bool ? 1 : 2;\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(
-                Expressions.parameter(boolean.class, "bool"), ONE, TWO)));
+                Expressions.parameter(boolean.class, "bool"), ONE, TWO)),
+        is("{\n  return bool ? 1 : 2;\n}\n"));
   }
 
   @Test void testOptimizeTernaryRotateNot() {
     // !bool ? 1 : 2
-    assertEquals("{\n  return bool ? 2 : 1;\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(
                 Expressions.not(Expressions.parameter(boolean.class, "bool")),
-                ONE, TWO)));
+                ONE, TWO)),
+        is("{\n  return bool ? 2 : 1;\n}\n"));
   }
 
   @Test void testOptimizeTernaryRotateEqualFalse() {
     // bool == false ? 1 : 2
-    assertEquals("{\n  return bool ? 2 : 1;\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(
                 Expressions.equal(Expressions.parameter(boolean.class, "bool"),
                     FALSE),
-                ONE, TWO)));
+                ONE, TWO)),
+        is("{\n  return bool ? 2 : 1;\n}\n"));
   }
 
   @Test void testOptimizeTernaryAtrueB() {
     // a ? true : b  === a || b
-    assertEquals("{\n  return a || b;\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(
                 Expressions.parameter(boolean.class, "a"),
-                TRUE, Expressions.parameter(boolean.class, "b"))));
+                TRUE, Expressions.parameter(boolean.class, "b"))),
+        is("{\n  return a || b;\n}\n"));
   }
 
   @Test void testOptimizeTernaryAtrueNull() {
     // a ? Boolean.TRUE : null  === a ? Boolean.TRUE : (Boolean) null
-    assertEquals("{\n  return a ? Boolean.TRUE : null;\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(
                 Expressions.parameter(boolean.class, "a"),
-                TRUE_B, Expressions.constant(null, Boolean.class))));
+                TRUE_B, Expressions.constant(null, Boolean.class))),
+        is("{\n  return a ? Boolean.TRUE : null;\n}\n"));
   }
 
   @Test void testOptimizeTernaryAtrueBoxed() {
     // a ? Boolean.TRUE : Boolean.valueOf(b)  === a || b
-    assertEquals("{\n  return a || Boolean.valueOf(b);\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(Expressions.parameter(boolean.class, "a"),
                 TRUE_B,
                 Expressions.call(Boolean.class, "valueOf",
-                    Expressions.parameter(boolean.class, "b")))));
+                    Expressions.parameter(boolean.class, "b")))),
+        is("{\n  return a || Boolean.valueOf(b);\n}\n"));
   }
 
   @Test void testOptimizeTernaryABtrue() {
     // a ? b : true  === !a || b
-    assertEquals("{\n  return !a || b;\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(
                 Expressions.parameter(boolean.class, "a"),
-                Expressions.parameter(boolean.class, "b"), TRUE)));
+                Expressions.parameter(boolean.class, "b"), TRUE)),
+        is("{\n  return !a || b;\n}\n"));
   }
 
   @Test void testOptimizeTernaryAfalseB() {
     // a ? false : b === !a && b
-    assertEquals("{\n  return !a && b;\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(
                 Expressions.parameter(boolean.class, "a"),
-                FALSE, Expressions.parameter(boolean.class, "b"))));
+                FALSE, Expressions.parameter(boolean.class, "b"))),
+        is("{\n  return !a && b;\n}\n"));
   }
 
   @Test void testOptimizeTernaryABfalse() {
     // a ? b : false === a && b
-    assertEquals("{\n  return a && b;\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(Expressions.parameter(boolean.class, "a"),
-                Expressions.parameter(boolean.class, "b"), FALSE)));
+                Expressions.parameter(boolean.class, "b"), FALSE)),
+        is("{\n  return a && b;\n}\n"));
   }
 
   @Test void testOptimizeTernaryInEqualABCeqB() {
     // (v ? (Integer) null : inp0_) == null
-    assertEquals("{\n  return v || inp0_ == null;\n}\n",
+    assertThat(
         optimize(
             Expressions.equal(
                 Expressions.condition(Expressions.parameter(boolean.class, "v"),
                     NULL_INTEGER,
                     Expressions.parameter(Integer.class, "inp0_")),
-            NULL)));
+            NULL)),
+        is("{\n  return v || inp0_ == null;\n}\n"));
   }
 
   @Test void testOptimizeTernaryNullCasting1() {
-    assertEquals("{\n  return (v ? Long.valueOf(1L) : null) == Long.valueOf(2L);\n}\n",
+    assertThat(
         optimize(
             Expressions.equal(
                 Expressions.condition(Expressions.parameter(boolean.class, "v"),
                     new ConstantExpression(Long.class, 1L),
                     new ConstantExpression(Long.class, null)),
-                new ConstantExpression(Long.class, 2L))));
+                new ConstantExpression(Long.class, 2L))),
+        is("{\n  return (v ? Long.valueOf(1L) : null) == Long.valueOf(2L);\n}\n"));
 
-    assertEquals("{\n  return (v ? null : Long.valueOf(1L)) == Long.valueOf(2L);\n}\n",
+    assertThat(
         optimize(
             Expressions.equal(
                 Expressions.condition(Expressions.parameter(boolean.class, "v"),
                     new ConstantExpression(Long.class, null),
                     new ConstantExpression(Long.class, 1L)),
-                new ConstantExpression(Long.class, 2L))));
+                new ConstantExpression(Long.class, 2L))),
+        is("{\n  return (v ? null : Long.valueOf(1L)) == Long.valueOf(2L);\n}\n"));
 
-    assertEquals("{\n  return (v ? null : Long.valueOf(1L)) == Long.valueOf(2L);\n}\n",
+    assertThat(
         optimize(
             Expressions.equal(
                 Expressions.condition(Expressions.parameter(boolean.class, "v"),
                     new ConstantExpression(Object.class, null),
                     new ConstantExpression(Long.class, 1L)),
-                new ConstantExpression(Long.class, 2L))));
+                new ConstantExpression(Long.class, 2L))),
+        is("{\n  return (v ? null : Long.valueOf(1L)) == Long.valueOf(2L);\n}\n"));
   }
 
   @Test void testOptimizeTernaryNullCasting2() {
@@ -199,15 +213,17 @@ class OptimizerTest {
     ParameterExpression v = Expressions.parameter(Boolean.class, "v");
 
     BlockStatement bl =
-        Expressions.block(Expressions.declare(0, v, new ConstantExpression(Boolean.class, false)),
+        Expressions.block(
+            Expressions.declare(0, v,
+                new ConstantExpression(Boolean.class, false)),
         Expressions.declare(0, o,
             Expressions.condition(v,
                 new ConstantExpression(Object.class, null),
                 new ConstantExpression(Boolean.class, true))));
 
-    assertEquals("{\n  Boolean v = Boolean.valueOf(false);\n"
-            + "  Boolean o = v ? null : Boolean.valueOf(true);\n}\n",
-        optimize(bl));
+    assertThat(optimize(bl),
+        is("{\n  Boolean v = Boolean.valueOf(false);\n"
+            + "  Boolean o = v ? null : Boolean.valueOf(true);\n}\n"));
 
     bl =
         Expressions.block(
@@ -216,8 +232,8 @@ class OptimizerTest {
                 new ConstantExpression(Boolean.class, true),
                 new ConstantExpression(Boolean.class, null))));
 
-    assertEquals("{\n  Boolean o = Boolean.valueOf(true) || (Boolean) null;\n}\n",
-        optimize(bl));
+    assertThat(optimize(bl),
+        is("{\n  Boolean o = Boolean.valueOf(true) || (Boolean) null;\n}\n"));
 
     bl =
         Expressions.block(
@@ -226,8 +242,8 @@ class OptimizerTest {
                 new ConstantExpression(Boolean.class, null),
                 new ConstantExpression(Boolean.class, true))));
 
-    assertEquals("{\n  Boolean o = (Boolean) null || Boolean.valueOf(true);\n}\n",
-        optimize(bl));
+    assertThat(optimize(bl),
+        is("{\n  Boolean o = (Boolean) null || Boolean.valueOf(true);\n}\n"));
   }
 
   @Test void testOptimizeBinaryNullCasting1() {
@@ -239,14 +255,15 @@ class OptimizerTest {
     ParameterExpression y = Expressions.parameter(Long.class, "y");
     BinaryExpression exp0 = Expressions.greaterThan(y, innerExp);
     ConditionalStatement finalExp =
-        Expressions.ifThenElse(exp0, Expressions.assign(x, one), Expressions.assign(x, second));
+        Expressions.ifThenElse(exp0, Expressions.assign(x, one),
+            Expressions.assign(x, second));
 
-    assertEquals("{\n  if (y > Long.valueOf(2L)) {\n"
+    assertThat(optimize(finalExp),
+        is("{\n  if (y > Long.valueOf(2L)) {\n"
             + "    return x = \"one\";\n"
             + "  } else {\n"
             + "    return x = null;\n"
-            + "  }\n}\n",
-        optimize(finalExp));
+            + "  }\n}\n"));
   }
 
   @Test void testOptimizeBinaryNullCasting2() {
@@ -258,306 +275,326 @@ class OptimizerTest {
         Expressions.assign(
             y, Expressions.orElse(x,
             new ConstantExpression(Boolean.class, null)));
-    assertEquals("{\n  return y = x || (Boolean) null;\n}\n", optimize(yt));
+    assertThat(optimize(yt),
+        is("{\n  return y = x || (Boolean) null;\n}\n"));
   }
 
   @Test void testOptimizeTernaryInEqualABCeqC() {
     // (v ? inp0_ : (Integer) null) == null
-    assertEquals("{\n  return !v || inp0_ == null;\n}\n",
+    assertThat(
         optimize(
             Expressions.equal(
                 Expressions.condition(Expressions.parameter(boolean.class, "v"),
                     Expressions.parameter(Integer.class, "inp0_"),
                     NULL_INTEGER),
-            NULL)));
+            NULL)),
+        is("{\n  return !v || inp0_ == null;\n}\n"));
   }
 
   @Test void testOptimizeTernaryAeqBBA() {
     // a == b ? b : a
     ParameterExpression a = Expressions.parameter(boolean.class, "a");
     ParameterExpression b = Expressions.parameter(boolean.class, "b");
-    assertEquals("{\n  return a;\n}\n",
-        optimize(Expressions.condition(Expressions.equal(a, b), b, a)));
+    assertThat(optimize(Expressions.condition(Expressions.equal(a, b), b, a)),
+        is("{\n  return a;\n}\n"));
   }
 
   @Test void testOptimizeTernaryAeqBAB() {
     // a == b ? a : b
     ParameterExpression a = Expressions.parameter(boolean.class, "a");
     ParameterExpression b = Expressions.parameter(boolean.class, "b");
-    assertEquals("{\n  return b;\n}\n",
-        optimize(Expressions.condition(Expressions.equal(a, b), a, b)));
+    assertThat(optimize(Expressions.condition(Expressions.equal(a, b), a, b)),
+        is("{\n  return b;\n}\n"));
   }
 
   @Test void testOptimizeTernaryInEqualABCneqB() {
     // (v ? (Integer) null : inp0_) != null
-    assertEquals("{\n  return !(v || inp0_ == null);\n}\n",
+    assertThat(
         optimize(
             Expressions.notEqual(
                 Expressions.condition(Expressions.parameter(boolean.class, "v"),
                     NULL_INTEGER,
                     Expressions.parameter(Integer.class, "inp0_")),
-            NULL)));
+            NULL)),
+        is("{\n  return !(v || inp0_ == null);\n}\n"));
   }
 
   @Test void testOptimizeTernaryInEqualABCneqC() {
     // (v ? inp0_ : (Integer) null) != null
-    assertEquals("{\n  return !(!v || inp0_ == null);\n}\n",
+    assertThat(
         optimize(
             Expressions.notEqual(
                 Expressions.condition(Expressions.parameter(boolean.class, "v"),
                     Expressions.parameter(Integer.class, "inp0_"),
                     NULL_INTEGER),
-            NULL)));
+            NULL)),
+        is("{\n  return !(!v || inp0_ == null);\n}\n"));
   }
 
   @Test void testOptimizeTernaryAneqBBA() {
     // a != b ? b : a
     ParameterExpression a = Expressions.parameter(boolean.class, "a");
     ParameterExpression b = Expressions.parameter(boolean.class, "b");
-    assertEquals("{\n  return b;\n}\n",
-        optimize(Expressions.condition(Expressions.notEqual(a, b), b, a)));
+    assertThat(
+        optimize(Expressions.condition(Expressions.notEqual(a, b), b, a)),
+        is("{\n  return b;\n}\n"));
   }
 
   @Test void testOptimizeTernaryAneqBAB() {
     // a != b ? a : b
     ParameterExpression a = Expressions.parameter(boolean.class, "a");
     ParameterExpression b = Expressions.parameter(boolean.class, "b");
-    assertEquals("{\n  return a;\n}\n",
-        optimize(Expressions.condition(Expressions.notEqual(a, b), a, b)));
+    assertThat(
+        optimize(Expressions.condition(Expressions.notEqual(a, b), a, b)),
+        is("{\n  return a;\n}\n"));
   }
 
   @Test void testAndAlsoTrueBool() {
     // true && bool
-    assertEquals("{\n  return bool;\n}\n",
+    assertThat(
         optimize(
             Expressions.andAlso(TRUE,
-                Expressions.parameter(boolean.class, "bool"))));
+                Expressions.parameter(boolean.class, "bool"))),
+        is("{\n  return bool;\n}\n"));
   }
 
   @Test void testAndAlsoBoolTrue() {
     // bool && true
-    assertEquals("{\n  return bool;\n}\n",
+    assertThat(
         optimize(
             Expressions.andAlso(
-                Expressions.parameter(boolean.class, "bool"), TRUE)));
+                Expressions.parameter(boolean.class, "bool"), TRUE)),
+        is("{\n  return bool;\n}\n"));
   }
 
   @Test void testAndAlsoFalseBool() {
     // false && bool
-    assertEquals("{\n  return false;\n}\n",
+    assertThat(
         optimize(
             Expressions.andAlso(FALSE,
-                Expressions.parameter(boolean.class, "bool"))));
+                Expressions.parameter(boolean.class, "bool"))),
+        is("{\n  return false;\n}\n"));
   }
 
   @Test void testAndAlsoNullBool() {
     // null && bool
-    assertEquals("{\n  return null && bool;\n}\n",
+    assertThat(
         optimize(
             Expressions.andAlso(NULL,
-                Expressions.parameter(boolean.class, "bool"))));
+                Expressions.parameter(boolean.class, "bool"))),
+        is("{\n  return null && bool;\n}\n"));
   }
 
   @Test void testAndAlsoXY() {
     // x && y
-    assertEquals("{\n  return x && y;\n}\n",
+    assertThat(
         optimize(
             Expressions.andAlso(
                 Expressions.parameter(boolean.class, "x"),
-                Expressions.parameter(boolean.class, "y"))));
+                Expressions.parameter(boolean.class, "y"))),
+        is("{\n  return x && y;\n}\n"));
   }
 
   @Test void testAndAlsoXX() {
     // x && x
     ParameterExpression x = Expressions.parameter(boolean.class, "x");
-    assertEquals("{\n  return x;\n}\n",
-        optimize(Expressions.andAlso(x, x)));
+    assertThat(optimize(Expressions.andAlso(x, x)),
+        is("{\n  return x;\n}\n"));
   }
 
   @Test void testOrElseTrueBool() {
     // true || bool
-    assertEquals("{\n  return true;\n}\n",
+    assertThat(
         optimize(
             Expressions.orElse(TRUE,
-                Expressions.parameter(boolean.class, "bool"))));
+                Expressions.parameter(boolean.class, "bool"))),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testOrElseFalseBool() {
     // false || bool
-    assertEquals("{\n  return bool;\n}\n",
+    assertThat(
         optimize(
             Expressions.orElse(FALSE,
-                Expressions.parameter(boolean.class, "bool"))));
+                Expressions.parameter(boolean.class, "bool"))),
+        is("{\n  return bool;\n}\n"));
   }
 
   @Test void testOrElseNullBool() {
     // null || bool
-    assertEquals("{\n  return null || bool;\n}\n",
+    assertThat(
         optimize(
             Expressions.orElse(NULL,
-                Expressions.parameter(boolean.class, "bool"))));
+                Expressions.parameter(boolean.class, "bool"))),
+        is("{\n  return null || bool;\n}\n"));
   }
 
   @Test void testOrElseXY() {
     // x || y
-    assertEquals("{\n  return x || y;\n}\n",
+    assertThat(
         optimize(
             Expressions.orElse(
                 Expressions.parameter(boolean.class, "x"),
-                Expressions.parameter(boolean.class, "y"))));
+                Expressions.parameter(boolean.class, "y"))),
+        is("{\n  return x || y;\n}\n"));
   }
 
   @Test void testOrElseXX() {
     // x || x
     ParameterExpression x = Expressions.parameter(boolean.class, "x");
-    assertEquals("{\n  return x;\n}\n", optimize(Expressions.orElse(x, x)));
+    assertThat(optimize(Expressions.orElse(x, x)),
+        is("{\n  return x;\n}\n"));
   }
 
   @Test void testEqualSameConst() {
     // 1 == 1
-    assertEquals("{\n  return true;\n}\n",
-        optimize(Expressions.equal(ONE, Expressions.constant(1))));
+    assertThat(optimize(Expressions.equal(ONE, Expressions.constant(1))),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testEqualDifferentConst() {
     // 1 == 2
-    assertEquals("{\n  return false;\n}\n",
-        optimize(Expressions.equal(ONE, TWO)));
+    assertThat(optimize(Expressions.equal(ONE, TWO)),
+        is("{\n  return false;\n}\n"));
   }
 
   @Test void testEqualSameExpr() {
     // x == x
     ParameterExpression x = Expressions.parameter(int.class, "x");
-    assertEquals("{\n  return true;\n}\n", optimize(Expressions.equal(x, x)));
+    assertThat(optimize(Expressions.equal(x, x)),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testEqualDifferentExpr() {
     // x == y
     ParameterExpression x = Expressions.parameter(int.class, "x");
     ParameterExpression y = Expressions.parameter(int.class, "y");
-    assertEquals("{\n  return x == y;\n}\n", optimize(Expressions.equal(x, y)));
+    assertThat(optimize(Expressions.equal(x, y)),
+        is("{\n  return x == y;\n}\n"));
   }
 
   @Test void testEqualPrimitiveNull() {
     // (int) x == null
     ParameterExpression x = Expressions.parameter(int.class, "x");
-    assertEquals("{\n  return false;\n}\n",
-        optimize(Expressions.equal(x, NULL)));
+    assertThat(optimize(Expressions.equal(x, NULL)),
+        is("{\n  return false;\n}\n"));
   }
 
   @Test void testEqualObjectNull() {
     // (Integer) x == null
     ParameterExpression x = Expressions.parameter(Integer.class, "x");
-    assertEquals("{\n  return x == null;\n}\n",
-        optimize(Expressions.equal(x, NULL)));
+    assertThat(optimize(Expressions.equal(x, NULL)),
+        is("{\n  return x == null;\n}\n"));
   }
 
   @Test void testEqualStringNull() {
     // "Y" == null
-    assertEquals("{\n  return false;\n}\n",
-        optimize(Expressions.equal(Expressions.constant("Y"), NULL)));
+    assertThat(optimize(Expressions.equal(Expressions.constant("Y"), NULL)),
+        is("{\n  return false;\n}\n"));
   }
 
   @Test void testEqualTypedNullUntypedNull() {
     // (Integer) null == null
-    assertEquals("{\n  return true;\n}\n",
-        optimize(Expressions.equal(NULL_INTEGER, NULL)));
+    assertThat(optimize(Expressions.equal(NULL_INTEGER, NULL)),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testEqualUnypedNullTypedNull() {
     // null == (Integer) null
-    assertEquals("{\n  return true;\n}\n",
-        optimize(Expressions.equal(NULL, NULL_INTEGER)));
+    assertThat(optimize(Expressions.equal(NULL, NULL_INTEGER)),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testEqualBoolTrue() {
     // x == true
     ParameterExpression x = Expressions.parameter(boolean.class, "x");
-    assertEquals("{\n  return x;\n}\n", optimize(Expressions.equal(x, TRUE)));
+    assertThat(optimize(Expressions.equal(x, TRUE)),
+        is("{\n  return x;\n}\n"));
   }
 
   @Test void testEqualBoolFalse() {
     // x == false
     ParameterExpression x = Expressions.parameter(boolean.class, "x");
-    assertEquals("{\n  return !x;\n}\n", optimize(Expressions.equal(x, FALSE)));
+    assertThat(optimize(Expressions.equal(x, FALSE)),
+        is("{\n  return !x;\n}\n"));
   }
 
   @Test void testNotEqualSameConst() {
     // 1 != 1
-    assertEquals("{\n  return false;\n}\n",
-        optimize(Expressions.notEqual(ONE, Expressions.constant(1))));
+    assertThat(optimize(Expressions.notEqual(ONE, Expressions.constant(1))),
+        is("{\n  return false;\n}\n"));
   }
 
   @Test void testNotEqualDifferentConst() {
     // 1 != 2
-    assertEquals("{\n  return true;\n}\n",
-        optimize(Expressions.notEqual(ONE, TWO)));
+    assertThat(optimize(Expressions.notEqual(ONE, TWO)),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testNotEqualSameExpr() {
     // x != x
     ParameterExpression x = Expressions.parameter(int.class, "x");
-    assertEquals("{\n  return false;\n}\n",
-        optimize(Expressions.notEqual(x, x)));
+    assertThat(optimize(Expressions.notEqual(x, x)),
+        is("{\n  return false;\n}\n"));
   }
 
   @Test void testNotEqualDifferentExpr() {
     // x != y
     ParameterExpression x = Expressions.parameter(int.class, "x");
     ParameterExpression y = Expressions.parameter(int.class, "y");
-    assertEquals("{\n  return x != y;\n}\n",
-        optimize(Expressions.notEqual(x, y)));
+    assertThat(optimize(Expressions.notEqual(x, y)),
+        is("{\n  return x != y;\n}\n"));
   }
 
   @Test void testNotEqualPrimitiveNull() {
     // (int) x == null
     ParameterExpression x = Expressions.parameter(int.class, "x");
-    assertEquals("{\n  return true;\n}\n",
-        optimize(Expressions.notEqual(x, NULL)));
+    assertThat(optimize(Expressions.notEqual(x, NULL)),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testNotEqualObjectNull() {
     // (Integer) x == null
     ParameterExpression x = Expressions.parameter(Integer.class, "x");
-    assertEquals("{\n  return x != null;\n}\n",
-        optimize(Expressions.notEqual(x, NULL)));
+    assertThat(optimize(Expressions.notEqual(x, NULL)),
+        is("{\n  return x != null;\n}\n"));
   }
 
   @Test void testNotEqualStringNull() {
     // "Y" != null
-    assertEquals("{\n  return true;\n}\n",
-        optimize(Expressions.notEqual(Expressions.constant("Y"), NULL)));
+    assertThat(optimize(Expressions.notEqual(Expressions.constant("Y"), NULL)),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testNotEqualTypedNullUntypedNull() {
     // (Integer) null != null
-    assertEquals("{\n  return false;\n}\n",
-        optimize(Expressions.notEqual(NULL_INTEGER, NULL)));
+    assertThat(optimize(Expressions.notEqual(NULL_INTEGER, NULL)),
+        is("{\n  return false;\n}\n"));
   }
 
   @Test void testNotEqualUnypedNullTypedNull() {
     // null != (Integer) null
-    assertEquals("{\n  return false;\n}\n",
-        optimize(Expressions.notEqual(NULL, NULL_INTEGER)));
+    assertThat(optimize(Expressions.notEqual(NULL, NULL_INTEGER)),
+        is("{\n  return false;\n}\n"));
   }
 
   @Test void testNotEqualBoolTrue() {
     // x != true
     ParameterExpression x = Expressions.parameter(boolean.class, "x");
-    assertEquals("{\n  return !x;\n}\n",
-        optimize(Expressions.notEqual(x, TRUE)));
+    assertThat(optimize(Expressions.notEqual(x, TRUE)),
+        is("{\n  return !x;\n}\n"));
   }
 
   @Test void testNotEqualBoolFalse() {
     // x != false
     ParameterExpression x = Expressions.parameter(boolean.class, "x");
-    assertEquals("{\n  return x;\n}\n",
-        optimize(Expressions.notEqual(x, FALSE)));
+    assertThat(optimize(Expressions.notEqual(x, FALSE)),
+        is("{\n  return x;\n}\n"));
   }
 
   @Test void testMultipleFolding() {
     // (1 == 2 ? 3 : 4) != (5 != 6 ? 4 : 8) ? 9 : 10
-    assertEquals("{\n  return 10;\n}\n",
+    assertThat(
         optimize(
             Expressions.condition(
                 Expressions.notEqual(
@@ -568,122 +605,120 @@ class OptimizerTest {
                             Expressions.constant(5), Expressions.constant(6)),
                         Expressions.constant(4), Expressions.constant(8))),
                 Expressions.constant(9),
-                Expressions.constant(10))));
+                Expressions.constant(10))),
+        is("{\n  return 10;\n}\n"));
   }
 
   @Test void testConditionalIfTrue() {
     // if (true) {return 1}
-    assertEquals("{\n  return 1;\n}\n",
-        optimize(Expressions.ifThen(TRUE, Expressions.return_(null, ONE))));
+    assertThat(
+        optimize(Expressions.ifThen(TRUE, Expressions.return_(null, ONE))),
+        is("{\n  return 1;\n}\n"));
   }
 
   @Test void testConditionalIfTrueElse() {
     // if (true) {return 1} else {return 2}
-    assertEquals("{\n  return 1;\n}\n",
+    assertThat(
         optimize(
             Expressions.ifThenElse(TRUE,
                 Expressions.return_(null, ONE),
-                Expressions.return_(null, TWO))));
+                Expressions.return_(null, TWO))),
+        is("{\n  return 1;\n}\n"));
   }
 
   @Test void testConditionalIfFalse() {
     // if (false) {return 1}
-    assertEquals("{}",
-        optimize(Expressions.ifThen(FALSE, Expressions.return_(null, ONE))));
+    assertThat(
+        optimize(Expressions.ifThen(FALSE, Expressions.return_(null, ONE))),
+        is("{}"));
   }
 
   @Test void testConditionalIfFalseElse() {
     // if (false) {return 1} else {return 2}
-    assertEquals("{\n  return 2;\n}\n",
+    assertThat(
         optimize(
             Expressions.ifThenElse(FALSE,
                 Expressions.return_(null, ONE),
-                Expressions.return_(null, TWO))));
+                Expressions.return_(null, TWO))),
+        is("{\n  return 2;\n}\n"));
   }
 
   @Test void testConditionalIfBoolTrue() {
     // if (bool) {return 1} else if (true) {return 2}
     Expression bool = Expressions.parameter(boolean.class, "bool");
-    assertEquals(
-        "{\n"
+    assertThat(
+        optimize(
+            Expressions.ifThenElse(bool,
+                Expressions.return_(null, ONE),
+                TRUE,
+                Expressions.return_(null, TWO))),
+        is("{\n"
             + "  if (bool) {\n"
             + "    return 1;\n"
             + "  } else {\n"
             + "    return 2;\n"
             + "  }\n"
-            + "}\n",
-        optimize(
-            Expressions.ifThenElse(bool,
-                Expressions.return_(null, ONE),
-                TRUE,
-                Expressions.return_(null, TWO))));
+            + "}\n"));
   }
 
   @Test void testConditionalIfBoolTrueElse() {
     // if (bool) {return 1} else if (true) {return 2} else {return 3}
     Expression bool = Expressions.parameter(boolean.class, "bool");
-    assertEquals(
-        "{\n"
-            + "  if (bool) {\n"
-            + "    return 1;\n"
-            + "  } else {\n"
-            + "    return 2;\n"
-            + "  }\n"
-            + "}\n",
+    assertThat(
         optimize(
             Expressions.ifThenElse(bool,
                 Expressions.return_(null, ONE),
                 TRUE,
                 Expressions.return_(null, TWO),
-                Expressions.return_(null, THREE))));
+                Expressions.return_(null, THREE))),
+        is("{\n"
+            + "  if (bool) {\n"
+            + "    return 1;\n"
+            + "  } else {\n"
+            + "    return 2;\n"
+            + "  }\n"
+            + "}\n"));
   }
 
   @Test void testConditionalIfBoolFalse() {
     // if (bool) {return 1} else if (false) {return 2}
     Expression bool = Expressions.parameter(boolean.class, "bool");
-    assertEquals(
-        "{\n"
-            + "  if (bool) {\n"
-            + "    return 1;\n"
-            + "  }\n"
-            + "}\n",
+    assertThat(
         optimize(
             Expressions.ifThenElse(bool,
                 Expressions.return_(null, ONE),
                 FALSE,
-                Expressions.return_(null, TWO))));
+                Expressions.return_(null, TWO))),
+        is("{\n"
+            + "  if (bool) {\n"
+            + "    return 1;\n"
+            + "  }\n"
+            + "}\n"));
   }
 
   @Test void testConditionalIfBoolFalseElse() {
     // if (bool) {return 1} else if (false) {return 2} else {return 3}
     Expression bool = Expressions.parameter(boolean.class, "bool");
-    assertEquals(
-        "{\n"
-            + "  if (bool) {\n"
-            + "    return 1;\n"
-            + "  } else {\n"
-            + "    return 3;\n"
-            + "  }\n"
-            + "}\n",
+    assertThat(
         optimize(
             Expressions.ifThenElse(bool,
                 Expressions.return_(null, ONE),
                 FALSE,
                 Expressions.return_(null, TWO),
-                Expressions.return_(null, THREE))));
+                Expressions.return_(null, THREE))),
+        is("{\n"
+            + "  if (bool) {\n"
+            + "    return 1;\n"
+            + "  } else {\n"
+            + "    return 3;\n"
+            + "  }\n"
+            + "}\n"));
   }
 
   @Test void testConditionalIfBoolFalseTrue() {
     // if (bool) {1} else if (false) {2} if (true) {4} else {5}
     Expression bool = Expressions.parameter(boolean.class, "bool");
-    assertEquals(
-        "{\n"
-            + "  if (bool) {\n"
-            + "    return 1;\n"
-            + "  } else {\n"
-            + "    return 4;\n"
-            + "  }\n"
-            + "}\n",
+    assertThat(
         optimize(
             Expressions.ifThenElse(bool,
                 Expressions.return_(null, ONE),
@@ -691,96 +726,111 @@ class OptimizerTest {
                 Expressions.return_(null, TWO),
                 TRUE,
                 Expressions.return_(null, FOUR),
-                Expressions.return_(null, Expressions.constant(5)))));
+                Expressions.return_(null, Expressions.constant(5)))),
+        is("{\n"
+            + "  if (bool) {\n"
+            + "    return 1;\n"
+            + "  } else {\n"
+            + "    return 4;\n"
+            + "  }\n"
+            + "}\n"));
   }
 
   @Test void testCastIntToShort() {
     // return (short) 1 --> return (short) 1
-    assertEquals("{\n  return (short)1;\n}\n",
-        optimize(Expressions.convert_(ONE, short.class)));
+    assertThat(optimize(Expressions.convert_(ONE, short.class)),
+        is("{\n  return (short)1;\n}\n"));
   }
 
   @Test void testCastIntToInt() {
     // return (int) 1 --> return 1L
-    assertEquals("{\n  return 1;\n}\n",
-        optimize(Expressions.convert_(ONE, int.class)));
+    assertThat(optimize(Expressions.convert_(ONE, int.class)),
+        is("{\n  return 1;\n}\n"));
   }
 
   @Test void testCastIntToLong() {
     // return (long) 1 --> return 1L
-    assertEquals("{\n  return 1L;\n}\n",
-        optimize(Expressions.convert_(ONE, long.class)));
+    assertThat(optimize(Expressions.convert_(ONE, long.class)),
+        is("{\n  return 1L;\n}\n"));
   }
 
   @Test void testNotTrue() {
     // !true -> false
-    assertEquals("{\n  return false;\n}\n", optimize(Expressions.not(TRUE)));
+    assertThat(optimize(Expressions.not(TRUE)),
+        is("{\n  return false;\n}\n"));
   }
 
   @Test void testNotFalse() {
     // !false -> true
-    assertEquals("{\n  return true;\n}\n", optimize(Expressions.not(FALSE)));
+    assertThat(optimize(Expressions.not(FALSE)),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testNotNotA() {
     // !!a -> a
-    assertEquals("{\n  return a;\n}\n",
-        optimize(Expressions.not(Expressions.not(bool("a")))));
+    assertThat(optimize(Expressions.not(Expressions.not(bool("a")))),
+        is("{\n  return a;\n}\n"));
   }
 
   @Test void testNotEq() {
     // !(a == b) -> a != b
-    assertEquals("{\n  return a != b;\n}\n",
-        optimize(Expressions.not(Expressions.equal(bool("a"), bool("b")))));
+    assertThat(
+        optimize(Expressions.not(Expressions.equal(bool("a"), bool("b")))),
+        is("{\n  return a != b;\n}\n"));
   }
 
   @Test void testNotNeq() {
     // !(a != b) -> a == b
-    assertEquals("{\n  return a == b;\n}\n",
+    assertThat(
         optimize(
-            Expressions.not(Expressions.notEqual(bool("a"), bool("b")))));
+            Expressions.not(Expressions.notEqual(bool("a"), bool("b")))),
+        is("{\n  return a == b;\n}\n"));
   }
 
   @Test void testNotGt() {
     // !(a > b) -> a <= b
-    assertEquals("{\n  return a <= b;\n}\n",
+    assertThat(
         optimize(
-            Expressions.not(Expressions.greaterThan(bool("a"), bool("b")))));
+            Expressions.not(Expressions.greaterThan(bool("a"), bool("b")))),
+        is("{\n  return a <= b;\n}\n"));
   }
 
   @Test void testNotGte() {
     // !(a >= b) -> a < b
-    assertEquals("{\n  return a < b;\n}\n",
+    assertThat(
         optimize(
             Expressions.not(
-                Expressions.greaterThanOrEqual(bool("a"), bool("b")))));
+                Expressions.greaterThanOrEqual(bool("a"), bool("b")))),
+        is("{\n  return a < b;\n}\n"));
   }
 
   @Test void testNotLt() {
     // !(a < b) -> a >= b
-    assertEquals("{\n  return a >= b;\n}\n",
+    assertThat(
         optimize(
-            Expressions.not(Expressions.lessThan(bool("a"), bool("b")))));
+            Expressions.not(Expressions.lessThan(bool("a"), bool("b")))),
+        is("{\n  return a >= b;\n}\n"));
   }
 
   @Test void testNotLte() {
     // !(a <= b) -> a > b
-    assertEquals("{\n  return a > b;\n}\n",
+    assertThat(
         optimize(
             Expressions.not(
-                Expressions.lessThanOrEqual(bool("a"), bool("b")))));
+                Expressions.lessThanOrEqual(bool("a"), bool("b")))),
+        is("{\n  return a > b;\n}\n"));
   }
 
   @Test void booleanValueOfTrue() {
     // Boolean.valueOf(true) -> true
-    assertEquals("{\n  return true;\n}\n",
-        optimize(Expressions.call(Boolean.class, "valueOf", TRUE)));
+    assertThat(optimize(Expressions.call(Boolean.class, "valueOf", TRUE)),
+        is("{\n  return true;\n}\n"));
   }
 
   @Test void testBooleanValueOfFalse() {
     // Boolean.valueOf(false) -> false
-    assertEquals("{\n  return false;\n}\n",
-        optimize(Expressions.call(Boolean.class, "valueOf", FALSE)));
+    assertThat(optimize(Expressions.call(Boolean.class, "valueOf", FALSE)),
+        is("{\n  return false;\n}\n"));
   }
 
   @Test void testAssign() {
