@@ -64,9 +64,10 @@ import java.util.stream.Collectors;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.CoreMatchers.nullValue;
 import static org.hamcrest.MatcherAssert.assertThat;
-import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.fail;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * Testing mongo adapter functionality. By default, runs with
  * Mongo Java Server unless {@code IT} maven profile is enabled
@@ -77,7 +78,9 @@ import static org.junit.jupiter.api.Assertions.fail;
 public class MongoAdapterTest implements SchemaFactory {
 
   /** Connection factory based on the "mongo-zips" model. */
-  protected static final URL MODEL = MongoAdapterTest.class.getResource("/mongo-model.json");
+  protected static final URL MODEL =
+      requireNonNull(MongoAdapterTest.class.getResource("/mongo-model.json"),
+          "url");
 
   /** Number of records in local file. */
   protected static final int ZIPS_SIZE = 149;
@@ -91,10 +94,15 @@ public class MongoAdapterTest implements SchemaFactory {
   public static void setUp() throws Exception {
     MongoDatabase database = POLICY.database();
 
-    populate(database.getCollection("zips"), MongoAdapterTest.class.getResource("/zips-mini.json"));
-    populate(database.getCollection("store"), FoodmartJson.class.getResource("/store.json"));
+    populate(database.getCollection("zips"),
+        requireNonNull(MongoAdapterTest.class.getResource("/zips-mini.json"),
+            "url"));
+    populate(database.getCollection("store"),
+        requireNonNull(FoodmartJson.class.getResource("/store.json"),
+            "url"));
     populate(database.getCollection("warehouse"),
-        FoodmartJson.class.getResource("/warehouse.json"));
+        requireNonNull(FoodmartJson.class.getResource("/warehouse.json"),
+            "url"));
 
     // Manually insert data for data-time test.
     MongoCollection<BsonDocument> datatypes =  database.getCollection("datatypes")
@@ -116,7 +124,7 @@ public class MongoAdapterTest implements SchemaFactory {
 
   private static void populate(MongoCollection<Document> collection, URL resource)
       throws IOException {
-    Objects.requireNonNull(collection, "collection");
+    requireNonNull(collection, "collection");
 
     if (collection.countDocuments() > 0) {
       // delete any existing documents (run from a clean set)
@@ -125,7 +133,7 @@ public class MongoAdapterTest implements SchemaFactory {
 
     MongoCollection<BsonDocument> bsonCollection = collection.withDocumentClass(BsonDocument.class);
     Resources.readLines(resource, StandardCharsets.UTF_8, new LineProcessor<Void>() {
-      @Override public boolean processLine(String line) throws IOException {
+      @Override public boolean processLine(String line) {
         bsonCollection.insertOne(BsonDocument.parse(line));
         return true;
       }
@@ -151,7 +159,7 @@ public class MongoAdapterTest implements SchemaFactory {
   }
 
   private CalciteAssert.AssertThat assertModel(URL url) {
-    Objects.requireNonNull(url, "url");
+    requireNonNull(url, "url");
     try {
       return assertModel(Resources.toString(url, StandardCharsets.UTF_8));
     } catch (IOException e) {
@@ -809,10 +817,8 @@ public class MongoAdapterTest implements SchemaFactory {
             .map(b -> b.toJson(settings)).collect(Collectors.joining("\n"));
 
         // used to pretty print Assertion error
-        assertEquals(
-            prettyFn.apply(expectedBsons),
-            prettyFn.apply(actualBsons),
-            "expected and actual Mongo queries (pipelines) do not match");
+        assertThat("expected and actual Mongo queries (pipelines) do not match",
+            prettyFn.apply(actualBsons), is(prettyFn.apply(expectedBsons)));
 
         fail("Should have failed previously because expected != actual is known to be true");
       }
