@@ -39,6 +39,8 @@ import org.apache.calcite.util.Bug;
 
 import org.checkerframework.checker.nullness.qual.Nullable;
 
+import java.util.ArrayList;
+import java.util.List;
 import java.util.function.Consumer;
 import java.util.function.UnaryOperator;
 
@@ -71,7 +73,9 @@ public interface SqlOperatorFixture extends AutoCloseable {
   // TODO: Change message
   String INVALID_CHAR_MESSAGE = "(?s).*";
 
-  String OUT_OF_RANGE_MESSAGE = ".* out of range";
+  String OUT_OF_RANGE_MESSAGE = ".* out of range.*";
+
+  String WRONG_FORMAT_MESSAGE = "Number has wrong format.*";
 
   // TODO: Change message
   String DIVISION_BY_ZERO_MESSAGE = "(?s).*";
@@ -82,27 +86,22 @@ public interface SqlOperatorFixture extends AutoCloseable {
   // TODO: Change message
   String BAD_DATETIME_MESSAGE = "(?s).*";
 
-  // Error messages when an invalid time unit is given as
-  // input to extract for a particular input type.
-  String INVALID_EXTRACT_UNIT_CONVERTLET_ERROR =
-      "Was not expecting value '.*' for enumeration.*";
-
-  String INVALID_EXTRACT_UNIT_VALIDATION_ERROR =
-      "Cannot apply 'EXTRACT' to arguments of type .*'\n.*";
-
   String LITERAL_OUT_OF_RANGE_MESSAGE =
       "(?s).*Numeric literal.*out of range.*";
 
   String INVALID_ARGUMENTS_NUMBER =
       "Invalid number of arguments to function .* Was expecting .* arguments";
 
+  String INVALID_ARGUMENTS_TYPE_VALIDATION_ERROR =
+      "Cannot apply '.*' to arguments of type .*";
+
   //~ Enums ------------------------------------------------------------------
 
   /**
    * Name of a virtual machine that can potentially implement an operator.
    */
   enum VmName {
-    FENNEL, JAVA, EXPAND
+    JAVA, EXPAND
   }
 
   //~ Methods ----------------------------------------------------------------
@@ -456,6 +455,24 @@ public interface SqlOperatorFixture extends AutoCloseable {
       String[] inputValues,
       ResultChecker checker);
 
+  /**
+   * Checks that an aggregate expression returns the expected result.
+   *
+   * <p>For example, <code>checkAgg("AVG(DISTINCT x)", new String[] {"2", "3",
+   * null, "3" }, "INTEGER", isSingle([2, 3]));</code>
+   *
+   * @param expr        Aggregate expression, e.g. <code>SUM(DISTINCT x)</code>
+   * @param inputValues Array of input values, e.g. <code>["1", null,
+   *                    "2"]</code>.
+   * @param type        Expected result type
+   * @param checker     Result checker
+   */
+  void checkAgg(
+      String expr,
+      String[] inputValues,
+      String type,
+      ResultChecker checker);
+
   /**
    * Checks that an aggregate expression with multiple args returns the expected
    * result.
@@ -553,6 +570,19 @@ public interface SqlOperatorFixture extends AutoCloseable {
                 .with(CalciteConnectionProperty.FUN, library.fun));
   }
 
+  default SqlOperatorFixture withLibraries(SqlLibrary... libraries) {
+    List<String> names = new ArrayList<>();
+    for (SqlLibrary lib : libraries) {
+      names.add(lib.fun);
+    }
+    return withOperatorTable(
+        SqlLibraryOperatorTableFactory.INSTANCE
+            .getOperatorTable(libraries))
+        .withConnectionFactory(cf ->
+            cf.with(ConnectionFactories.add(CalciteAssert.SchemaSpec.HR))
+                .with(CalciteConnectionProperty.FUN, String.join(",", names)));
+  }
+
   /** Applies this fixture to some code for each of the given libraries. */
   default void forEachLibrary(Iterable<? extends SqlLibrary> libraries,
       Consumer<SqlOperatorFixture> consumer) {
@@ -643,8 +673,14 @@ public interface SqlOperatorFixture extends AutoCloseable {
 
   default void checkCastFails(String value, String targetType,
       String expectedError, boolean runtime, CastType castType) {
-    final String castString = getCastString(value, targetType, !runtime, castType);
-    checkFails(castString, expectedError, runtime);
+    // Safe casts should never fail
+    boolean shouldFail = castType == CastType.CAST;
+    final String castString = getCastString(value, targetType, shouldFail && !runtime, castType);
+    if (shouldFail) {
+      checkFails(castString, expectedError, runtime);
+    } else {
+      checkNull(castString);
+    }
   }
 
   default void checkCastToString(String value, @Nullable String type,
