@@ -40,7 +40,6 @@ import org.apache.calcite.sql.test.SqlTester;
 import org.apache.calcite.tools.RelBuilder;
 import org.apache.calcite.util.ImmutableBitSet;
 
-import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSortedSet;
 import com.google.common.collect.Iterables;
@@ -58,13 +57,14 @@ import java.util.function.Function;
 import java.util.function.Supplier;
 import java.util.function.UnaryOperator;
 
+import static com.google.common.base.Preconditions.checkArgument;
+
 import static org.hamcrest.CoreMatchers.equalTo;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.CoreMatchers.not;
 import static org.hamcrest.CoreMatchers.notNullValue;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.hasSize;
-import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
@@ -196,7 +196,7 @@ public class RelMetadataFixture {
     metadataConfig.applyMetadata(rel.getCluster());
     if (convertAsCalc) {
       Project project = (Project) rel;
-      Preconditions.checkArgument(project.getVariablesSet().isEmpty(),
+      checkArgument(project.getVariablesSet().isEmpty(),
           "Calc does not allow variables");
       RexProgram program =
           RexProgram.create(project.getInput().getRowType(),
@@ -351,9 +351,10 @@ public class RelMetadataFixture {
     final RelMetadataQuery mq = rel.getCluster().getMetadataQuery();
     Set<ImmutableBitSet> result = mq.getUniqueKeys(rel);
     assertThat(result, notNullValue());
-    assertEquals(ImmutableSortedSet.copyOf(expectedUniqueKeys),
+    assertThat("unique keys, sql: " + relSupplier
+            + ", rel: " + RelOptUtil.toString(rel),
         ImmutableSortedSet.copyOf(result),
-        () -> "unique keys, sql: " + relSupplier + ", rel: " + RelOptUtil.toString(rel));
+        is(ImmutableSortedSet.copyOf(expectedUniqueKeys)));
     checkUniqueConsistent(rel);
     return this;
   }
@@ -371,9 +372,10 @@ public class RelMetadataFixture {
         ImmutableBitSet.range(0, rel.getRowType().getFieldCount());
     for (ImmutableBitSet key : allCols.powerSet()) {
       Boolean result2 = mq.areColumnsUnique(rel, key);
-      assertEquals(isUnique(uniqueKeys, key), SqlFunctions.isTrue(result2),
-          () -> "areColumnsUnique. key: " + key + ", uniqueKeys: " + uniqueKeys
-              + ", rel: " + RelOptUtil.toString(rel));
+      assertThat("areColumnsUnique. key: " + key
+          + ", uniqueKeys: " + uniqueKeys
+          + ", rel: " + RelOptUtil.toString(rel),
+          SqlFunctions.isTrue(result2), is(isUnique(uniqueKeys, key)));
     }
   }
 
