@@ -74,6 +74,7 @@ import org.apache.calcite.sql.SqlAccessType;
 import org.apache.calcite.sql.SqlCall;
 import org.apache.calcite.sql.SqlKind;
 import org.apache.calcite.sql.SqlNode;
+import org.apache.calcite.sql.validate.SemanticTable;
 import org.apache.calcite.sql.validate.SqlModality;
 import org.apache.calcite.sql.validate.SqlMonotonicity;
 import org.apache.calcite.sql.validate.SqlNameMatcher;
@@ -90,6 +91,7 @@ import org.apache.calcite.util.Pair;
 import org.apache.calcite.util.Util;
 
 import com.google.common.collect.ImmutableList;
+import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Iterables;
 
@@ -105,6 +107,8 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * Mock implementation of {@link SqlValidatorCatalogReader} which returns tables
  * "EMP", "DEPT", "BONUS", "SALGRADE" (same as Oracle's SCOTT schema).
@@ -131,7 +135,7 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
     super(CalciteSchema.createRootSchema(false, false, DEFAULT_CATALOG),
         SqlNameMatchers.withCaseSensitive(caseSensitive),
         ImmutableList.of(PREFIX, ImmutableList.of()),
-        typeFactory, null);
+        typeFactory, CalciteConnectionConfig.DEFAULT);
   }
 
   @Override public boolean isCaseSensitive() {
@@ -189,6 +193,7 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
     final CalciteSchema schema =
         SqlValidatorUtil.getSchema(rootSchema,
             schemaPath, SqlNameMatchers.withCaseSensitive(true));
+    requireNonNull(schema, "schema");
     schema.add(Util.last(names), relProtoDataType);
   }
 
@@ -219,6 +224,7 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
     CalciteSchema schema =
         SqlValidatorUtil.getSchema(rootSchema,
             schemaPath, SqlNameMatchers.withCaseSensitive(true));
+    requireNonNull(schema, "schema");
     schema.removeTable(tableName);
     schema.add(tableName, table);
   }
@@ -230,6 +236,7 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
     final CalciteSchema schema =
         SqlValidatorUtil.getSchema(rootSchema,
             schemaPath, SqlNameMatchers.withCaseSensitive(true));
+    requireNonNull(schema, "schema");
     schema.add(tableName, table);
   }
 
@@ -238,8 +245,10 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
   }
 
   private void registerNestedSchema(MockSchema parentSchema, MockSchema schema) {
-    rootSchema.getSubSchema(parentSchema.getName(), true)
-        .add(schema.name, new AbstractSchema());
+    final CalciteSchema subSchema =
+        rootSchema.getSubSchema(parentSchema.getName(), true);
+    requireNonNull(subSchema, "subSchema");
+    subSchema.add(schema.name, new AbstractSchema());
   }
 
   private static List<RelCollation> deduceMonotonicity(
@@ -276,7 +285,7 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
   /** Mock schema. */
   public static class MockSchema {
     private final List<String> tableNames = new ArrayList<>();
-    private String name;
+    private final String name;
 
     public MockSchema(String name) {
       this.name = name;
@@ -316,7 +325,7 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
     protected final Double maxRowCount;
     protected final Set<String> monotonicColumnSet = new HashSet<>();
     protected StructKind kind = StructKind.FULLY_QUALIFIED;
-    protected final ColumnResolver resolver;
+    protected final @Nullable ColumnResolver resolver;
     private final boolean temporal;
     protected final InitializerExpressionFactory initializerFactory;
     protected final Set<String> rolledUpColumns = new HashSet<>();
@@ -328,7 +337,7 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
 
     public MockTable(MockCatalogReader catalogReader, String catalogName,
         String schemaName, String name, boolean stream, boolean temporal,
-        double rowCount, ColumnResolver resolver,
+        double rowCount, @Nullable ColumnResolver resolver,
         InitializerExpressionFactory initializerFactory) {
       this(catalogReader, ImmutableList.of(catalogName, schemaName, name),
           stream, temporal, rowCount, resolver, initializerFactory,
@@ -337,7 +346,7 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
 
     public MockTable(MockCatalogReader catalogReader, String catalogName,
         String schemaName, String name, boolean stream, boolean temporal,
-        double rowCount, ColumnResolver resolver,
+        double rowCount, @Nullable ColumnResolver resolver,
         InitializerExpressionFactory initializerFactory, Double maxRowCount) {
       this(catalogReader, ImmutableList.of(catalogName, schemaName, name),
           stream, temporal, rowCount, resolver, initializerFactory,
@@ -350,7 +359,7 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
 
     private MockTable(MockCatalogReader catalogReader, List<String> names,
         boolean stream, boolean temporal, double rowCount,
-        ColumnResolver resolver,
+        @Nullable ColumnResolver resolver,
         InitializerExpressionFactory initializerFactory, List<Object> wraps,
         Double maxRowCount) {
       this.catalogReader = catalogReader;
@@ -371,7 +380,8 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
         boolean temporal, double rowCount,
         List<Map.Entry<String, RelDataType>> columnList, List<Integer> keyList,
         RelDataType rowType, List<RelCollation> collationList, List<String> names,
-        Set<String> monotonicColumnSet, StructKind kind, ColumnResolver resolver,
+        Set<String> monotonicColumnSet, StructKind kind,
+        @Nullable ColumnResolver resolver,
         InitializerExpressionFactory initializerFactory) {
       this.catalogReader = catalogReader;
       this.stream = stream;
@@ -417,22 +427,22 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
       }
 
       @Override public Collection getModifiableCollection() {
-        return null;
+        throw new UnsupportedOperationException();
       }
 
       @Override public <E> Queryable<E>
       asQueryable(QueryProvider queryProvider, SchemaPlus schema,
           String tableName) {
-        return null;
+        throw new UnsupportedOperationException();
       }
 
       @Override public Type getElementType() {
-        return null;
+        return Void.class;
       }
 
       @Override public Expression getExpression(SchemaPlus schema,
           String tableName, Class clazz) {
-        return null;
+        throw new UnsupportedOperationException();
       }
 
       @Override public <C> C unwrap(Class<C> aClass) {
@@ -469,7 +479,9 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
           SqlCall call, @Nullable SqlNode parent, @Nullable CalciteConnectionConfig config) {
         // For testing
         return call.getKind() != SqlKind.MAX
-            && (parent.getKind() == SqlKind.SELECT || parent.getKind() == SqlKind.FILTER);
+            && parent != null
+            && (parent.getKind() == SqlKind.SELECT
+                || parent.getKind() == SqlKind.FILTER);
       }
     }
 
@@ -500,21 +512,21 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
 
     public static MockTable create(MockCatalogReader catalogReader,
         MockSchema schema, String name, boolean stream, double rowCount,
-        ColumnResolver resolver) {
+        @Nullable ColumnResolver resolver) {
       return create(catalogReader, schema, name, stream, rowCount, resolver,
           NullInitializerExpressionFactory.INSTANCE, false);
     }
 
     public static MockTable create(MockCatalogReader catalogReader,
         MockSchema schema, String name, boolean stream, double rowCount,
-        ColumnResolver resolver, double maxRowCount) {
+        @Nullable ColumnResolver resolver, double maxRowCount) {
       return create(catalogReader, schema, name, stream, rowCount, resolver,
           NullInitializerExpressionFactory.INSTANCE, false, maxRowCount);
     }
 
     public static MockTable create(MockCatalogReader catalogReader,
         MockSchema schema, String name, boolean stream, double rowCount,
-        ColumnResolver resolver,
+        @Nullable ColumnResolver resolver,
         InitializerExpressionFactory initializerExpressionFactory,
         boolean temporal, Double maxRowCount) {
       MockTable table =
@@ -527,7 +539,7 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
 
     public static MockTable create(MockCatalogReader catalogReader,
         MockSchema schema, String name, boolean stream, double rowCount,
-        ColumnResolver resolver,
+        @Nullable ColumnResolver resolver,
         InitializerExpressionFactory initializerExpressionFactory,
         boolean temporal) {
       MockTable table =
@@ -666,11 +678,13 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
 
       ModifiableTableWithCustomColumnResolving(String tableName) {
         super(tableName);
+        requireNonNull(resolver, "resolver");
       }
 
       @Override public List<Pair<RelDataTypeField, List<String>>> resolveColumn(
           RelDataType rowType, RelDataTypeFactory typeFactory,
           List<String> names) {
+        requireNonNull(resolver, "resolver");
         return resolver.resolveColumn(rowType, typeFactory, names);
       }
     }
@@ -700,7 +714,8 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
         MockCatalogReader catalogReader, boolean stream, double rowCount,
         List<Map.Entry<String, RelDataType>> columnList, List<Integer> keyList,
         RelDataType rowType, List<RelCollation> collationList, List<String> names,
-        Set<String> monotonicColumnSet, StructKind kind, ColumnResolver resolver,
+        Set<String> monotonicColumnSet, StructKind kind,
+        @Nullable ColumnResolver resolver,
         InitializerExpressionFactory initializerFactory) {
       super(catalogReader, stream, false, rowCount, columnList, keyList,
           rowType, collationList, names,
@@ -709,8 +724,9 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
     }
 
     public static MockModifiableViewRelOptTable create(MockModifiableViewTable modifiableViewTable,
-        MockCatalogReader catalogReader, String catalogName, String schemaName, String name,
-        boolean stream, double rowCount, ColumnResolver resolver) {
+        MockCatalogReader catalogReader, String catalogName,
+        String schemaName, String name,
+        boolean stream, double rowCount, @Nullable ColumnResolver resolver) {
       final Table underlying = modifiableViewTable.unwrap(Table.class);
       final InitializerExpressionFactory initializerExpressionFactory =
           underlying instanceof Wrapper
@@ -797,8 +813,9 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
     private final ViewTable viewTable;
 
     private MockRelViewTable(ViewTable viewTable,
-        MockCatalogReader catalogReader, String catalogName, String schemaName, String name,
-        boolean stream, double rowCount, ColumnResolver resolver,
+        MockCatalogReader catalogReader, String catalogName,
+        String schemaName, String name,
+        boolean stream, double rowCount, @Nullable ColumnResolver resolver,
         InitializerExpressionFactory initializerExpressionFactory) {
       super(catalogReader, ImmutableList.of(catalogName, schemaName, name),
           stream, false, rowCount, resolver, initializerExpressionFactory,
@@ -807,8 +824,9 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
     }
 
     public static MockRelViewTable create(ViewTable viewTable,
-        MockCatalogReader catalogReader, String catalogName, String schemaName, String name,
-        boolean stream, double rowCount, ColumnResolver resolver) {
+        MockCatalogReader catalogReader, String catalogName,
+        String schemaName, String name,
+        boolean stream, double rowCount, @Nullable ColumnResolver resolver) {
       Table underlying = viewTable.unwrap(Table.class);
       InitializerExpressionFactory initializerExpressionFactory =
           underlying instanceof Wrapper
@@ -847,7 +865,8 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
 
     MockViewTable(MockCatalogReader catalogReader, String catalogName,
         String schemaName, String name, boolean stream, double rowCount,
-        MockTable fromTable, ImmutableIntList mapping, ColumnResolver resolver,
+        MockTable fromTable, ImmutableIntList mapping,
+        @Nullable ColumnResolver resolver,
         InitializerExpressionFactory initializerFactory) {
       super(catalogReader, catalogName, schemaName, name, stream, false,
           rowCount, resolver, initializerFactory);
@@ -911,9 +930,13 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
      */
     private class ModifiableViewWithCustomColumnResolving
         extends ModifiableView implements CustomColumnResolvingTable, Wrapper {
+      ModifiableViewWithCustomColumnResolving() {
+        requireNonNull(resolver, "resolver");
+      }
 
       @Override public List<Pair<RelDataTypeField, List<String>>> resolveColumn(
           RelDataType rowType, RelDataTypeFactory typeFactory, List<String> names) {
+        requireNonNull(resolver, "resolver");
         return resolver.resolveColumn(rowType, typeFactory, names);
       }
 
@@ -968,7 +991,8 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
   }
 
   /**
-   * Mock implementation of {@link AbstractQueryableTable} with dynamic record type.
+   * Mock implementation of {@link AbstractQueryableTable} with dynamic record
+   * type.
    */
   public static class MockDynamicTable
       extends AbstractQueryableTable implements TranslatableTable {
@@ -995,6 +1019,49 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
     }
   }
 
+  /** Mock implementation of {@link MockTable} that supports must-filter fields.
+   *
+   * <p>Must-filter fields are declared via methods in the {@link SemanticTable}
+   * interface. */
+  public static class MustFilterMockTable
+      extends MockTable implements SemanticTable {
+    private final Map<String, String> fieldFilters;
+
+    MustFilterMockTable(MockCatalogReader catalogReader, String catalogName,
+        String schemaName, String name, boolean stream, boolean temporal,
+        double rowCount, @Nullable ColumnResolver resolver,
+        InitializerExpressionFactory initializerExpressionFactory,
+        Map<String, String> fieldFilters) {
+      super(catalogReader, catalogName, schemaName, name, stream, temporal,
+          rowCount, resolver, initializerExpressionFactory);
+      this.fieldFilters = ImmutableMap.copyOf(fieldFilters);
+    }
+
+    /** Creates a MustFilterMockTable. */
+    public static MustFilterMockTable create(MockCatalogReader catalogReader,
+        MockSchema schema, String name, boolean stream, double rowCount,
+        @Nullable ColumnResolver resolver,
+        InitializerExpressionFactory initializerExpressionFactory,
+        boolean temporal, Map<String, String> fieldFilters) {
+      MustFilterMockTable table =
+          new MustFilterMockTable(catalogReader, schema.getCatalogName(),
+              schema.name, name, stream, temporal, rowCount, resolver,
+              initializerExpressionFactory, fieldFilters);
+      schema.addTable(name);
+      return table;
+    }
+
+    @Override public @Nullable String getFilter(int column) {
+      String columnName = columnList.get(column).getKey();
+      return fieldFilters.get(columnName);
+    }
+
+    @Override public boolean mustFilter(int column) {
+      String columnName = columnList.get(column).getKey();
+      return fieldFilters.containsKey(columnName);
+    }
+  }
+
   /** Wrapper around a {@link MockTable}, giving it a {@link Table} interface.
    * You can get the {@code MockTable} by calling {@link #unwrap(Class)}. */
   private static class WrapperTable implements Table, Wrapper {
@@ -1050,7 +1117,9 @@ public abstract class MockCatalogReader extends CalciteCatalogReader {
         SqlCall call, @Nullable SqlNode parent, @Nullable CalciteConnectionConfig config) {
       // For testing
       return call.getKind() != SqlKind.MAX
-              && (parent.getKind() == SqlKind.SELECT || parent.getKind() == SqlKind.FILTER);
+          && parent != null
+          && (parent.getKind() == SqlKind.SELECT
+              || parent.getKind() == SqlKind.FILTER);
     }
 
     @Override public Schema.TableType getJdbcTableType() {
